Package: shinyGovstyle
Title: Custom Gov Style Inputs for Shiny
Version: 0.2.0
Authors@R: c(
    person("Ross", "Wyatt", , "ross.wyatt@justice.gov.uk", role = "aut"),
    person("Cam", "Race", , "camrace8@gmail.com", role = c("aut", "cre")),
    person("Sarah", "Wong", role = "aut"),
    person("Richard", "Bielby", , "richard.bielby@education.gov.uk", role = "ctb"),
    person("Charlotte", "Foster", , "charlotte.foster@education.gov.uk", role = "ctb"),
    person("Jeni", "Martin", role = "ctb"),
    person("Andrew", "Baker", , "andrew.baker1@dhsc.gov.uk", role = "ctb")
  )
Description: Collection of 'shiny' application styling that are based on
    the GOV.UK Design System. See
    <https://design-system.service.gov.uk/components/> for details.
License: GPL (>= 3)
URL: https://github.com/dfe-analytical-services/shinyGovstyle
BugReports: https://github.com/dfe-analytical-services/shinyGovstyle/issues
Depends: R (>= 4.1.0)
Imports: htmltools, jsonlite, lifecycle, purrr, reactable, readODS,
        readr, shiny (>= 0.14), shinyjs, stringr, writexl
Suggests: bslib, knitr, rlang, rmarkdown, shinytest2, testthat
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-13 12:07:26 UTC; camra
Author: Ross Wyatt [aut],
  Cam Race [aut, cre],
  Sarah Wong [aut],
  Richard Bielby [ctb],
  Charlotte Foster [ctb],
  Jeni Martin [ctb],
  Andrew Baker [ctb]
Maintainer: Cam Race <camrace8@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-13 12:40:02 UTC
Built: R 4.5.2; ; 2026-04-15 05:14:24 UTC; unix

Package: shinyGovstyle
Title: Custom Gov Style Inputs for Shiny
Version: 0.2.0
Authors@R: c(
    person("Ross", "Wyatt", , "ross.wyatt@justice.gov.uk", role = "aut"),
    person("Cam", "Race", , "camrace8@gmail.com", role = c("aut", "cre")),
    person("Sarah", "Wong", role = "aut"),
    person("Richard", "Bielby", , "richard.bielby@education.gov.uk", role = "ctb"),
    person("Charlotte", "Foster", , "charlotte.foster@education.gov.uk", role = "ctb"),
    person("Jeni", "Martin", role = "ctb"),
    person("Andrew", "Baker", , "andrew.baker1@dhsc.gov.uk", role = "ctb")
  )
Description: Collection of 'shiny' application styling that are based on
    the GOV.UK Design System. See
    <https://design-system.service.gov.uk/components/> for details.
License: GPL (>= 3)
URL: https://github.com/dfe-analytical-services/shinyGovstyle
BugReports: https://github.com/dfe-analytical-services/shinyGovstyle/issues
Depends: R (>= 4.1.0)
Imports: htmltools, jsonlite, lifecycle, purrr, reactable, readODS,
        readr, shiny (>= 0.14), shinyjs, stringr, writexl
Suggests: bslib, knitr, rlang, rmarkdown, shinytest2, testthat
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-13 12:07:26 UTC; camra
Author: Ross Wyatt [aut],
  Cam Race [aut, cre],
  Sarah Wong [aut],
  Richard Bielby [ctb],
  Charlotte Foster [ctb],
  Jeni Martin [ctb],
  Andrew Baker [ctb]
Maintainer: Cam Race <camrace8@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-13 12:40:02 UTC
Built: R 4.6.0; ; 2026-04-30 03:33:02 UTC; unix

# Generated by roxygen2: do not edit by hand

export(accordion)
export(backlink_Input)
export(banner)
export(button_Input)
export(checkbox_Input)
export(contents_link)
export(cookieBanner)
export(date_Input)
export(details)
export(download_button)
export(download_link)
export(download_radios)
export(download_radios_handler)
export(error_off)
export(error_on)
export(error_summary)
export(error_summary_update)
export(external_link)
export(file_Input)
export(font)
export(footer)
export(full_width_overrides)
export(govReactable)
export(govReactableOutput)
export(govTable)
export(govTabs)
export(gov_box)
export(gov_layout)
export(gov_list)
export(gov_main_layout)
export(gov_row)
export(gov_summary)
export(gov_text)
export(header)
export(heading_text)
export(input_field)
export(insert_text)
export(label_hint)
export(noti_banner)
export(panel_output)
export(radio_button_Input)
export(renderGovReactable)
export(run_example)
export(select_Input)
export(service_navigation)
export(skip_to_main)
export(tag_Input)
export(text_Input)
export(text_area_Input)
export(update_service_navigation)
export(value_box)
export(warning_text)
export(word_count)
importFrom(htmltools,attachDependencies)
importFrom(htmltools,findDependencies)
importFrom(htmltools,htmlDependency)
importFrom(shiny,addResourcePath)
importFrom(utils,packageVersion)

# shinyGovstyle <img src="man/figures/logo.png" align="right" height="139" style="padding-left: 1rem;" />

<!-- badges: start -->

[![CRAN status](https://www.r-pkg.org/badges/version/shinyGovstyle)](https://cran.r-project.org/package=shinyGovstyle)
[![R-CMD-check](https://github.com/dfe-analytical-services/shinyGovstyle/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/dfe-analytical-services/shinyGovstyle/actions/workflows/R-CMD-check.yaml)
[![](https://cranlogs.r-pkg.org/badges/shinyGovstyle)](https://cran.r-project.org/package=shinyGovstyle)
[![Lifecycle: experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)

<!-- badges: end -->

> Apply GOV.UK styled components and formats in shiny


## Overview

This package provides custom widgets to style R Shiny apps using the GOV.UK design system. The components can be previewed in our [example showcase app](https://department-for-education.shinyapps.io/shinygovstyle-example-app/).
<br> 

![example app demonstration](man/figures/example_app.gif)

To view details and advice on how to use the GOV.UK components please visit https://design-system.service.gov.uk/components/, most components should be available to use through this package.

### Installation

You can install the latest stable version from CRAN
```r
install.packages("shinyGovstyle")
```

If you want to make use of the development version then install directly from GitHub.
```r
pak::pak("dfe-analytical-services/shinyGovstyle")
```

To use error and word count elements you will need to load `useShinyjs()` from shinyjs in your ui.R file
```r
shinyjs::useShinyjs()
```

### Contributing

Ideas, bug reports, and requests for new components should be [raised as GitHub issue](https://github.com/dfe-analytical-services/shinyGovstyle/issues/new). It's often worth checking the existing [issues log](https://github.com/dfe-analytical-services/shinyGovstyle/issues) incase there is already an existing discussion you can conrtibute to.

More details on contributing can be found in the `.github/CONTRIBUTING.md` file.

This package is also released with a `.github/CODE_OF_CONDUCT.md`. By contributing to this project, you agree to abide by its terms.

### Available components

Details of all available components and how to use them can be found in the [package documentation](https://dfe-analytical-services.github.io/shinyGovstyle/reference/index.html).

The package contains an [example showcase app](https://department-for-education.shinyapps.io/shinygovstyle-example-app/) you can view or run yourself, showcasing available components. The code for the example app is in the `inst/example_app/` folder. You can easily run the app from the console using:

```r
shinyGovstyle::run_example()
```

# shinyGovstyle example app

A Shiny app that showcases the components available in the `shinyGovstyle` package. The main branch version of this app is deployed at:
https://department-for-education.shinyapps.io/shinygovstyle-example-app/

## Code structure

```
inst/example_app/
├── global.R          # Loads libraries and sources all module files
├── ui.R              # Page layout: header, service navigation, tab container, footer
├── server.R          # Tab navigation observers and cookie banner event handlers
├── modules/
│   ├── mod_action_types.R    # Button and download components
│   ├── mod_cookies.R         # Cookie settings page
│   ├── mod_feedback_types.R  # Banners, warning text, error summaries
│   ├── mod_select_types.R    # Radio buttons, checkboxes, date inputs, selects
│   ├── mod_tables_tabs.R     # Tables, accordions, tabs
│   └── mod_text_types.R      # Text inputs, text areas, labels
└── tests/
    ├── testthat.R                        
    └── testthat/
        ├── setup-shinytest2.R            # shinytest2 setup file
        └── test-basic_load.R             # Checks the app title loads correctly
```

Each module follows the standard Shiny module pattern: a `mod_<name>_ui()` function for the UI and a `mod_<name>_server()` function for the server logic, both namespaced with the same `id`.

Navigation between tabs is driven by the `service_navigation` component and by next / back buttons within modules. The `server.R` contains observers that update the hidden `tabsetPanel` (`tab-container`) in response to both.

## Running the app locally

From the project root in R:

```r
shiny::runApp("inst/example_app")
```

## Running the tests

The example app has its own `shinytest2` tests, these are skipped on CRAN. To run the app's own internal test suite directly (from the project root)

```r
shinytest2::test_app("inst/example_app")
```

mod_text_types_ui <- function(id) {
  shiny::tagList(
    shinyGovstyle::heading_text("backlink_Input", size = "s", level = 2),
    shinyGovstyle::backlink_Input(shiny::NS(id, "select_types_back")),
    shinyGovstyle::heading_text("Text Types", size = "l"),
    shinyGovstyle::label_hint(
      shiny::NS(id, "label2"),
      "These are some examples of the types of user
           text inputs that you can use"
    ),
    shinyGovstyle::heading_text("date_Input", size = "s", level = 2),
    shinyGovstyle::date_Input(
      inputId = shiny::NS(id, "date1"),
      label = "What is your date of birth?",
      hint_label = "For example, 31 3 1980"
    ),
    shinyGovstyle::heading_text("text_Input", size = "s", level = 2),
    shinyGovstyle::text_Input(
      inputId = shiny::NS(id, "txt1"),
      label = "Event name"
    ),
    shinyGovstyle::heading_text("text_area_Input", size = "s", level = 2),
    shinyGovstyle::text_area_Input(
      inputId = shiny::NS(id, "text_area1"),
      label = "Can you provide more detail?",
      hint_label = paste0(
        "Do not include personal or financial information, ",
        "like your National Insurance number or credit card details."
      )
    ),
    shinyGovstyle::text_area_Input(
      inputId = shiny::NS(id, "text_area2"),
      label = "How are you today?",
      hint_label = "Leave blank to trigger error",
      error = TRUE,
      error_message = "Please do not leave blank",
      word_limit = 300
    ),
    shinyGovstyle::heading_text(
      "button_Input",
      size = "s",
      level = 2,
      id = "button_input_text_types"
    ),
    shinyGovstyle::button_Input(
      shiny::NS(id, "btn_error"),
      "Check for errors",
      type = "warning"
    ),
    shinyGovstyle::heading_text("gov_list", size = "s", level = 2),
    shinyGovstyle::gov_text("List:"),
    shinyGovstyle::gov_list(list = c("a", "b", "c")),
    shinyGovstyle::gov_text("Bulleted list:"),
    shinyGovstyle::gov_list(list = c("a", "b", "c"), style = "bullet"),
    shinyGovstyle::gov_text("Numbered list:"),
    shinyGovstyle::gov_list(list = c("one", "two", "three"), style = "number"),
  )
}

mod_text_types_server <- function(id) {
  shiny::moduleServer(id, function(input, output, session) {
    shiny::observeEvent(
      input$text_area2,
      shinyGovstyle::word_count("text_area2", input$text_area2)
    )

    shiny::observeEvent(input$btn_error, {
      if (input$text_area2 == "") {
        shinyGovstyle::error_on("text_area2")
      } else {
        shinyGovstyle::error_off("text_area2")
      }
    })

    list(
      prev_page = shiny::reactive(input$select_types_back)
    )
  })
}

bslib::page_fluid(
  theme = bs_theme(version = 5),
  title = "ShinyGovstyle component showcase",
  shinyGovstyle::cookieBanner("shinyGovstyle component showcase"),
  shinyjs::useShinyjs(),

  shiny::tags$head(shiny::HTML("<html lang='en'>")),

  shinyGovstyle::full_width_overrides(), # TODO: remove when built in
  shinyGovstyle::skip_to_main(),
  shinyGovstyle::header(
    org_name = "Department for Education",
    service_name = "shinyGovstyle showcase app"
  ),
  shinyGovstyle::service_navigation(
    c(
      "Select Types" = "sn_select_types",
      "Text Types" = "sn_text_types",
      "Action Types" = "sn_action_types",
      "Tables, tabs and accordions" = "sn_tables_tabs",
      "Feedback types" = "sn_feedback_types",
      "Cookies" = "sn_cookies"
    )
  ),

  shinyGovstyle::banner(
    "banner",
    "Beta",
    paste0(
      'This is a new service \u002D your <a class="govuk-link" href=',
      '"https://github.com/dfe-analytical-services/shinyGovstyle/issues/new',
      '/choose">feedback</a> will help us to improve it.'
    )
  ),

  shinyGovstyle::gov_main_layout(
    shinyGovstyle::gov_row(
      size = "two-thirds",
      shinyGovstyle::gov_text(
        "This example app showcases the components available in the",
        "latest development version of the shinyGovstyle package.",
        "The source code for the app can be found on the ",
        shinyGovstyle::external_link(
          href = paste0(
            "https://github.com/dfe-analytical-services/shinyGovstyle/",
            "blob/main/inst/example_app"
          ),
          link_text = "main GitHub branch"
        ),
        ". The page layout has some custom CSS overrides, and is still being",
        " developed to work with the ",
        shinyGovstyle::external_link(
          href = "https://rstudio.github.io/bslib/",
          link_text = "bslib package"
        ),
        ", it may change in future releases."
      )
    ),

    shinyGovstyle::gov_row(
      shinyGovstyle::gov_box(
        size = "two-thirds",
        shiny::tabsetPanel(
          type = "hidden",
          id = "tab-container",

          shiny::tabPanel(
            "Select Types",
            value = "select_types",
            mod_select_types_ui("select_types")
          ),

          shiny::tabPanel(
            "Text Types",
            value = "text_types",
            mod_text_types_ui("text_types")
          ),

          shiny::tabPanel(
            "Action types",
            value = "action_types",
            mod_action_types_ui("action_types")
          ),

          shiny::tabPanel(
            "Tables, tabs and accordions",
            value = "tables_tabs_and_accordions",
            mod_tables_tabs_ui("tables_tabs")
          ),

          shiny::tabPanel(
            "Feedback Types",
            value = "feedback_types",
            mod_feedback_types_ui("feedback_types")
          ),

          shiny::tabPanel(
            "Cookies",
            value = "panel-cookies",
            mod_cookies_ui("cookies")
          )
        )
      )
    )
  ),

  shinyGovstyle::footer(
    TRUE,
    links = c(
      `Cookies` = "cookies_footer_link",
      `GitHub repository` = paste(
        "https://github.com/dfe-analytical-services/shinyGovstyle"
      )
    )
  )
) # end of page_fluid

function(input, output, session) {
  # Module servers ============================================================
  select_types <- mod_select_types_server("select_types")
  text_types <- mod_text_types_server("text_types")
  mod_action_types_server("action_types")
  mod_tables_tabs_server("tables_tabs")

  # download_radios is itself a Shiny module and must be called at the top
  # level so its namespace matches the plain IDs rendered by download_radios()
  shinyGovstyle::download_radios_handler(
    file_name = "example_file",
    file_contents = data.frame(x = c(1, 2, 3), y = c(4, 5, 6))
  )

  # Tab nav - button-driven (next / back cross module boundaries) =============
  shiny::observeEvent(
    select_types$next_page(),
    {
      shiny::updateTabsetPanel(
        session,
        "tab-container",
        selected = "text_types"
      )
      update_service_navigation(session, "sn_text_types")
    },
    ignoreInit = TRUE
  )

  shiny::observeEvent(
    text_types$prev_page(),
    {
      shiny::updateTabsetPanel(
        session,
        "tab-container",
        selected = "select_types"
      )
      update_service_navigation(session, "sn_select_types")
    },
    ignoreInit = TRUE
  )

  # Service navigation link observers =========================================
  shiny::observeEvent(
    input$sn_select_types,
    {
      shiny::updateTabsetPanel(
        session,
        "tab-container",
        selected = "select_types"
      )
    },
    ignoreInit = TRUE
  )

  shiny::observeEvent(
    input$sn_text_types,
    {
      shiny::updateTabsetPanel(
        session,
        "tab-container",
        selected = "text_types"
      )
    },
    ignoreInit = TRUE
  )

  shiny::observeEvent(
    input$sn_action_types,
    {
      shiny::updateTabsetPanel(
        session,
        "tab-container",
        selected = "action_types"
      )
    },
    ignoreInit = TRUE
  )

  shiny::observeEvent(
    input$sn_tables_tabs,
    {
      shiny::updateTabsetPanel(
        session,
        "tab-container",
        selected = "tables_tabs_and_accordions"
      )
    },
    ignoreInit = TRUE
  )

  shiny::observeEvent(
    input$sn_feedback_types,
    {
      shiny::updateTabsetPanel(
        session,
        "tab-container",
        selected = "feedback_types"
      )
    },
    ignoreInit = TRUE
  )

  shiny::observeEvent(
    input$sn_cookies,
    {
      shiny::updateTabsetPanel(
        session,
        "tab-container",
        selected = "panel-cookies"
      )
    },
    ignoreInit = TRUE
  )

  # Cookie banner events ======================================================
  shiny::observeEvent(input$cookieLink, {
    shiny::updateTabsetPanel(
      session,
      "tab-container",
      selected = "panel-cookies"
    )
    update_service_navigation(session, "sn_cookies")
  })

  shiny::observeEvent(
    input$cookies_footer_link,
    {
      shiny::updateTabsetPanel(
        session,
        "tab-container",
        selected = "panel-cookies"
      )
      update_service_navigation(session, "sn_cookies")
    },
    ignoreInit = TRUE
  )

  shiny::observeEvent(input$cookieAccept, {
    shinyjs::show(id = "cookieAcceptDiv")
    shinyjs::hide(id = "cookieMain")
  })

  shiny::observeEvent(input$cookieReject, {
    shinyjs::show(id = "cookieRejectDiv")
    shinyjs::hide(id = "cookieMain")
  })

  shiny::observeEvent(input$hideAccept, {
    shinyjs::toggle(id = "cookieDiv")
  })

  shiny::observeEvent(input$hideReject, {
    shinyjs::toggle(id = "cookieDiv")
  })
} # end of server

mod_feedback_types_ui <- function(id) {
  shiny::tagList(
    shinyGovstyle::heading_text("Feedback page", size = "l"),
    shinyGovstyle::label_hint(
      shiny::NS(id, "label-feedback"),
      "These are some examples of the types of user
           feedback inputs that you can use"
    ),
    shinyGovstyle::heading_text("tag_Input", size = "s", level = 2),
    shinyGovstyle::tag_Input(shiny::NS(id, "tag1"), "Default"),
    shinyGovstyle::tag_Input(shiny::NS(id, "tag2"), "Grey", "grey"),
    shinyGovstyle::tag_Input(shiny::NS(id, "tag3"), "Green", "green"),
    shinyGovstyle::tag_Input(shiny::NS(id, "tag4"), "Teal", "teal"),
    shinyGovstyle::tag_Input(shiny::NS(id, "tag5"), "Blue", "blue"),
    shinyGovstyle::tag_Input(shiny::NS(id, "tag6"), "Purple", "purple"),
    shinyGovstyle::tag_Input(shiny::NS(id, "tag7"), "Magenta", "magenta"),
    shinyGovstyle::tag_Input(shiny::NS(id, "tag8"), "Red", "red"),
    shinyGovstyle::tag_Input(shiny::NS(id, "tag9"), "Orange", "orange"),
    shinyGovstyle::tag_Input(shiny::NS(id, "tag10"), "Yellow", "yellow"),
    shiny::tags$br(),
    shiny::tags$br(),
    shinyGovstyle::heading_text("details", size = "s", level = 2),
    shinyGovstyle::details(
      inputId = shiny::NS(id, "detID"),
      label = "Help with nationality",
      help_text = paste(
        "We need to know your nationality so we can work out",
        "which elections you\u0027re entitled to vote in. If you",
        "cannot provide your nationality\u002C you\u0027ll have to",
        "send copies of identity documents through the post."
      )
    ),
    shinyGovstyle::heading_text("insert_text", size = "s", level = 2),
    shinyGovstyle::insert_text(
      inputId = shiny::NS(id, "insertId"),
      text = "It can take up to 8 weeks to register a lasting
                power of attorney if there are no mistakes in the
                application."
    ),
    shinyGovstyle::heading_text("warning_text", size = "s", level = 2),
    shinyGovstyle::warning_text(
      inputId = shiny::NS(id, "warn"),
      text = "You can be fined up to \u00A35\u002C000 if you do
      not register."
    ),
    shinyGovstyle::heading_text("value_box", size = "s", level = 2),
    shinyGovstyle::value_box(
      value = "Default (no description included)"
    ),
    shinyGovstyle::value_box(
      value = "1,000,000",
      text = "This is an example value box in purple.",
      colour = "purple"
    ),
    shinyGovstyle::value_box(
      value = "58.3%",
      text = paste(
        "This is another example value box in red.",
        "More colours are available."
      ),
      colour = "red"
    ),
    shinyGovstyle::heading_text("panel_output", size = "s", level = 2),
    shinyGovstyle::panel_output(
      inputId = shiny::NS(id, "panId"),
      main_text = "Application complete",
      sub_text = paste(
        "Your reference number <br>",
        "<strong>HDJ2123F</strong>"
      )
    ),
    shinyGovstyle::heading_text("noti_banner", size = "s", level = 2),
    shinyGovstyle::noti_banner(
      shiny::NS(id, "notId"),
      title_txt = "Important",
      body_txt = "You have 7 days left to send your application.",
      type = "standard"
    ),
    shinyGovstyle::heading_text("gov_summary", size = "s", level = 2),
    shinyGovstyle::gov_summary(
      shiny::NS(id, "sumID"),
      c(
        "Name",
        "Date of birth",
        "Contact information",
        "Contact details"
      ),
      c(
        "Sarah Philips",
        "5 January 1978",
        "72 Guild Street <br> London <br> SE23 6FH",
        "07700 900457 <br> sarah.phillips@example.com"
      ),
      action = FALSE
    )
  )
}

mod_cookies_ui <- function(id) {
  shiny::tagList(
    shinyGovstyle::heading_text("Cookie page", size = "l"),
    shinyGovstyle::label_hint(
      shiny::NS(id, "label-cookies"),
      "This an example cookie page that could be
                   expanded"
    )
  )
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(shinyGovstyle)

## ----eval = FALSE-------------------------------------------------------------
# install.packages("shinyGovstyle")

## ----eval = FALSE-------------------------------------------------------------
# pak::pak("dfe-analytical-services/shinyGovstyle")

## ----eval = FALSE-------------------------------------------------------------
# ui <- bslib::page_fluid(
#   shinyjs::useShinyjs(),
#   # rest of your UI
# )

## ----eval = FALSE-------------------------------------------------------------
# library(shiny)
# library(shinyGovstyle)
# 
# ui <- bslib::page_fluid(
#   skip_to_main(),
#   header(
#     org_name = "My department",
#     service_name = "My dashboard"
#   ),
#   banner(
#     inputId = "phase",
#     type = "Beta",
#     label = "This is a new service."
#   ),
#   gov_main_layout(
#     gov_row(
#       gov_box(
#         size = "two-thirds",
#         heading_text("Welcome", size = "l", level = 1),
#         gov_text("Select a report from the navigation above.")
#       )
#     )
#   ),
#   footer()
# )
# 
# server <- function(input, output, session) {}
# 
# shiny::shinyApp(ui, server)

mod_action_types_ui <- function(id) {
  shiny::tagList(
    shinyGovstyle::heading_text("Action types", size = "l"),
    shinyGovstyle::label_hint(
      "label3",
      "These are some examples of the types of user
           action elements that you can use"
    ),
    shinyGovstyle::heading_text("external_link", size = "s", level = 2),
    shinyGovstyle::gov_text(
      paste0(
        "You can add external links with automatic ",
        "formatting such as to our "
      ),
      shinyGovstyle::external_link(
        href = paste0(
          "https://github.com/dfe-analytical-services/",
          "shinyGovstyle"
        ),
        link_text = "GitHub repository",
      ),
      "."
    ),
    shinyGovstyle::gov_text(
      paste(
        "You can also add external links that don't have the",
        "warning in brackets but do have the warning for screen",
        "readers, such as this link to our "
      ),
      shinyGovstyle::external_link(
        href = paste0(
          "https://dfe-analytical-services.github.io/",
          "shinyGovstyle/"
        ),
        link_text = "package documentation site",
        add_warning = FALSE
      ),
      "."
    ),
    shinyGovstyle::heading_text("download_button", size = "s", level = 2),
    shinyGovstyle::download_button(
      shiny::NS(id, "download_button_data"),
      "Download a demo data set",
      file_type = "CSV",
      file_size = "1 KB"
    ),
    shinyGovstyle::heading_text("download_link", size = "s", level = 2),
    shinyGovstyle::gov_text(
      shinyGovstyle::download_link(
        shiny::NS(id, "download_data"),
        "Download a demo data set",
        file_type = "CSV",
        file_size = "1 KB"
      )
    ),
    shinyGovstyle::heading_text("download_radios", size = "s", level = 2),
    shinyGovstyle::gov_text(
      shinyGovstyle::download_radios(
        file_types = c("CSV", "XLSX", "ODS")
      )
    )
  )
}

mod_action_types_server <- function(id) {
  shiny::moduleServer(id, function(input, output, session) {
    output$download_data <- shiny::downloadHandler(
      filename = "demo_data.csv",
      content = function(file) {
        data <- data.frame(
          x = 1:10,
          y = 101:110
        )
        write.csv(data, file)
      }
    )

    output$download_button_data <- shiny::downloadHandler(
      filename = "demo_button_data.csv",
      content = function(file) {
        data <- data.frame(
          x = 1:10,
          y = 1:10**3
        )
        write.csv(data, file)
      }
    )
  })
}

# Deployed at
# https://department-for-education.shinyapps.io/shinygovstyle-example-app/

library(bslib)
library(shinyGovstyle) # needs to come after as there's name clashes / masking

lapply(list.files("modules", full.names = TRUE, pattern = "\\.R$"), source)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(shinyGovstyle)

## ----eval = FALSE-------------------------------------------------------------
# skip_to_main()

## ----eval = FALSE-------------------------------------------------------------
# shinyjs::useShinyjs()
# cookieBanner("My service name")

## ----eval = FALSE-------------------------------------------------------------
# header(
#   org_name = "Department for Education",
#   service_name = "My dashboard"
# )

## ----eval = FALSE-------------------------------------------------------------
# banner(
#   inputId = "phase-banner",
#   type = "Beta",
#   label = paste0(
#     "This is a new service \u2014 your ",
#     '<a class="govuk-link" href="#">feedback</a> will help us to improve it.'
#   )
# )

## ----eval = FALSE-------------------------------------------------------------
# # Minimal footer
# footer()
# 
# # Footer with support links
# footer(
#   links = c(
#     `Accessibility statement` = "accessibility_footer_link",
#     `Cookies` = "cookies_footer_link"
#   )
# )

## ----eval = FALSE-------------------------------------------------------------
# gov_main_layout(
#   # your content here
# )

## ----eval = FALSE-------------------------------------------------------------
# gov_main_layout(
#   gov_row(
#     # columns go here
#   ),
#   gov_row(
#     # another row
#   )
# )

## ----eval = FALSE-------------------------------------------------------------
# gov_main_layout(
#   gov_row(
#     gov_box(
#       size = "two-thirds",
#       heading_text("Main content", size = "l"),
#       # inputs, text, etc.
#     ),
#     gov_box(
#       size = "one-third",
#       heading_text("Sidebar", size = "m"),
#       # supporting content
#     )
#   )
# )

## ----eval = FALSE-------------------------------------------------------------
# gov_main_layout(
#   gov_row(
#     gov_box(
#       size = "full",
#       heading_text("Page title", size = "l")
#     )
#   )
# )

## ----eval = FALSE-------------------------------------------------------------
# gov_layout(
#   size = "two-thirds",
#   heading_text("Page title", size = "l"),
#   # content
# )

## ----eval = FALSE-------------------------------------------------------------
# service_navigation(
#   c(
#     "Summary" = "nav_summary",
#     "Detailed data" = "nav_detail",
#     "User guide" = "nav_guide"
#   )
# )

## ----eval = FALSE-------------------------------------------------------------
# # ui.R — shiny tabsetPanel
# shiny::tabsetPanel(
#   type = "hidden",
#   id = "main_panels",
#   shiny::tabPanel("Summary",       value = "nav_summary",  "Content"),
#   shiny::tabPanel("Detailed data", value = "nav_detail",   "Content"),
#   shiny::tabPanel("User guide",    value = "nav_guide",    "Content")
# )
# 
# # server.R — nav link click: JS handles the active state, just switch the panel
# shiny::observeEvent(input$nav_summary, {
#   shiny::updateTabsetPanel(session, "main_panels", selected = "nav_summary")
# })

## ----eval = FALSE-------------------------------------------------------------
# # ui.R — bslib navset_hidden
# bslib::navset_hidden(
#   id = "main_panels",
#   bslib::nav_panel("Summary",       value = "nav_summary",  "Content"),
#   bslib::nav_panel("Detailed data", value = "nav_detail",   "Content"),
#   bslib::nav_panel("User guide",    value = "nav_guide",    "Content")
# )
# 
# # server.R
# shiny::observeEvent(input$nav_summary, {
#   bslib::nav_select("main_panels", "nav_summary")
# })

## ----eval = FALSE-------------------------------------------------------------
# # server.R — programmatic navigation: must update both the panel and the nav
# shiny::observeEvent(input$next_btn, {
#   shiny::updateTabsetPanel(session, "main_panels", selected = "nav_detail")
#   shinyGovstyle::update_service_navigation(session, "nav_detail")
# })

## ----eval = FALSE-------------------------------------------------------------
# # ui.R — footer link, no entry in service_navigation()
# footer(
#   full = TRUE,
#   links = c(`Accessibility statement` = "accessibility_footer_link")
# )
# 
# # ui.R — tab panel exists in the hidden tabset but not in service_navigation()
# shiny::tabsetPanel(
#   type = "hidden",
#   id = "main_panels",
#   shiny::tabPanel("Summary", value = "nav_summary", "Content"),
#   shiny::tabPanel("Accessibility statement", value = "accessibility_panel",
#                   "Content")
# )
# 
# # server.R — navigate to the footer page (no update_service_navigation needed)
# shiny::observeEvent(input$accessibility_footer_link, {
#   shiny::updateTabsetPanel(session, "main_panels",
#                            selected = "accessibility_panel")
# })

## ----eval = FALSE-------------------------------------------------------------
# library(shiny)
# library(shinyGovstyle)
# 
# ui <- bslib::page_fluid(
#   skip_to_main(),
#   header(
#     org_name = "My department",
#     service_name = "My dashboard"
#   ),
#   service_navigation(
#     c(
#       "Summary"  = "nav_summary",
#       "About"    = "nav_about"
#     )
#   ),
#   banner(
#     inputId = "phase",
#     type = "Beta",
#     label = "This is a new service."
#   ),
# 
#   gov_main_layout(
#     shiny::tabsetPanel(
#       type = "hidden",
#       id = "main_panels",
# 
#       shiny::tabPanel(
#         "Summary", value = "nav_summary",
#         gov_row(
#           gov_box(
#             size = "two-thirds",
#             heading_text("Summary", size = "l"),
#             gov_text("Welcome to the summary page.")
#           ),
#           gov_box(
#             size = "one-third",
#             heading_text("Quick facts", size = "m"),
#             gov_text("Supporting information goes here.")
#           )
#         )
#       ),
# 
#       shiny::tabPanel(
#         "About", value = "nav_about",
#         gov_row(
#           gov_box(
#             size = "full",
#             heading_text("About this dashboard", size = "l"),
#             gov_text("This page describes the dashboard.")
#           )
#         )
#       ),
# 
#       shiny::tabPanel(
#         "Accessibility statement", value = "accessibility_panel",
#         gov_row(
#           gov_box(
#             size = "full",
#             heading_text("Accessibility statement", size = "l"),
#             gov_text("This page describes the accessibility of the dashboard.")
#           )
#         )
#       )
#     )
#   ),
# 
#   footer(
#     links = c(`Accessibility statement` = "accessibility_footer_link")
#   )
# )
# 
# server <- function(input, output, session) {
#   shiny::observeEvent(input$nav_summary, {
#     shiny::updateTabsetPanel(session, "main_panels", selected = "nav_summary")
#   })
#   shiny::observeEvent(input$nav_about, {
#     shiny::updateTabsetPanel(session, "main_panels", selected = "nav_about")
#   })
#   shiny::observeEvent(input$accessibility_footer_link, {
#     shiny::updateTabsetPanel(session, "main_panels",
#                              selected = "accessibility_panel")
#   })
# }
# 
# shiny::shinyApp(ui, server)

mod_tables_tabs_ui <- function(id) {
  shiny::tagList(
    shinyGovstyle::heading_text("Tables, tabs and accordions", size = "l"),
    shinyGovstyle::label_hint(
      shiny::NS(id, "label3"),
      "These are some examples of using tabs and tables"
    ),

    shinyGovstyle::heading_text("govTable", size = "s", level = 2),
    shinyGovstyle::govTable(
      shiny::NS(id, "tab1"),
      shinyGovstyle::transport_data_small,
      "Static example",
      "l",
      num_col = c(2, 3)
    ),

    shinyGovstyle::heading_text(
      "govReactable with static data",
      size = "s",
      level = 2
    ),
    shinyGovstyle::heading_text("Caption added separately", size = "l"),
    shinyGovstyle::govReactable(
      shinyGovstyle::transport_data,
      right_col = c("bikes", "vans", "buses"),
      page_size = 5
    ),

    shinyGovstyle::heading_text(
      "govReactable with reactive data",
      size = "s",
      level = 2
    ),
    shinyGovstyle::select_Input(
      inputId = shiny::NS(id, "colourFilter"),
      label = "Select Colour",
      select_text = c(sort(unique(
        shinyGovstyle::transport_data$colours
      ))),
      select_value = c(sort(unique(
        shinyGovstyle::transport_data$colours
      )))
    ),
    shinyGovstyle::govReactableOutput(
      shiny::NS(id, "interactive_table_test"),
      caption = "Caption in output function"
    ),

    shinyGovstyle::heading_text("govTabs", size = "s", level = 2),
    shinyGovstyle::govTabs(
      shiny::NS(id, "govTabs"),
      shinyGovstyle::case_data,
      "tabs"
    ),
    shiny::tags$br(),
    shiny::tags$br(),
    shinyGovstyle::heading_text("accordions", size = "s", level = 2),
    shinyGovstyle::accordion(
      shiny::NS(id, "acc1"),
      c(
        "Writing well for the web",
        "Writing well for specialists",
        "Know your audience",
        "How people read"
      ),
      c(
        "This is the content for Writing well for the web.",
        "This is the content for Writing well for specialists.",
        "This is the content for Know your audience.",
        "This is the content for How people read."
      )
    )
  )
}

mod_tables_tabs_server <- function(id) {
  shiny::moduleServer(id, function(input, output, session) {
    filtered_data <- shiny::reactive({
      subset(shinyGovstyle::transport_data, colours == input$colourFilter)
    })

    output$interactive_table_test <- shinyGovstyle::renderGovReactable({
      shinyGovstyle::govReactable(
        df = filtered_data(),
        right_col = c("bikes", "vans", "buses"),
        page_size = 3,
      )
    })
  })
}

mod_select_types_ui <- function(id) {
  shiny::tagList(
    shinyGovstyle::heading_text("Select Types", size = "l"),
    shinyGovstyle::label_hint(
      shiny::NS(id, "label1"),
      "These are some examples of the types of user
           select type inputs that you can use"
    ),
    shinyGovstyle::heading_text(
      "radio_button_Input (inline)",
      size = "s",
      level = 2
    ),
    shinyGovstyle::radio_button_Input(
      inputId = shiny::NS(id, "name_changed"),
      label = "Have you changed your name?",
      choices = c("Yes", "No"),
      inline = TRUE,
      hint_label = "This includes changing your last name or spelling
                    your name differently."
    ),
    shinyGovstyle::heading_text(
      "radio_button_Input (stacked)",
      size = "s",
      level = 2
    ),
    shinyGovstyle::radio_button_Input(
      inputId = shiny::NS(id, "name_changed_stacked"),
      label = "Have you changed your name?",
      choices = c("Yes", "No"),
      inline = FALSE,
      hint_label = "This includes changing your last name or spelling
                    your name differently."
    ),
    shinyGovstyle::heading_text("checkbox_Input", size = "s", level = 2),
    shinyGovstyle::checkbox_Input(
      inputId = shiny::NS(id, "checkID"),
      cb_labels = c(
        "Waste from animal carcasses",
        "Waste from mines or quarries",
        "Farm or agricultural waste"
      ),
      checkboxIds = c(
        shiny::NS(id, "op1"),
        shiny::NS(id, "op2"),
        shiny::NS(id, "op3")
      ),
      label = "Which types of waste do you transport?",
      hint_label = "Select all that apply."
    ),
    shinyGovstyle::heading_text("select_Input", size = "s", level = 2),
    shinyGovstyle::select_Input(
      inputId = shiny::NS(id, "sorter"),
      label = "Sort by",
      select_text = c(
        "Recently published",
        "Recently updated",
        "Most views",
        "Most comments"
      ),
      select_value = c("published", "updated", "view", "comments")
    ),
    shinyGovstyle::heading_text("file_Input", size = "s", level = 2),
    shinyGovstyle::file_Input(
      inputId = shiny::NS(id, "file1"),
      label = "Upload a file"
    ),
    shinyGovstyle::heading_text("button_Input", size = "s", level = 2),
    shinyGovstyle::button_Input(
      shiny::NS(id, "text_types_next"),
      "Go to next page"
    )
  )
}

mod_select_types_server <- function(id) {
  shiny::moduleServer(id, function(input, output, session) {
    list(
      next_page = shiny::reactive(input$text_types_next)
    )
  })
}

#' Case data
#'
#' Simple example dataset containing case management data, used to demonstrate
#' tabs in the example app.
#'
#' @format A data frame with 12 rows and 4 variables:
#' \describe{
#' \item{tabs}{The tab the data belongs to}
#' \item{case_manager}{The name of the case manager}
#' \item{cases_open}{Number of open cases}
#' \item{cases_closed}{Number of closed cases}
#' }
#' @source Random data generated for the shinyGovstyle package
"case_data"

#' Transport data
#'
#' Simple example dataset containing transport data, used to demonstrate
#' various table styles in the example app.
#'
#' @format A data frame with 10 rows and 5 variables:
#' \describe{
#' \item{months}{The month the data belongs to}
#' \item{colours}{The colour of the vehicle}
#' \item{bikes}{Cost of bikes}
#' \item{vans}{Cost of vans}
#' \item{buses}{Cost of buses}
#' }
#' @source Random data generated for the shinyGovstyle package
"transport_data"

#' Transport data small
#'
#' Smaller example dataset containing transport data, used to demonstrate
#' various table styles in the example app.
#'
#' @format A data frame with 3 rows and 3 variables:
#' \describe{
#' \item{months}{The month the data belongs to}
#' \item{bikes}{Cost of bikes}
#' \item{cars}{Cost of cars}
#' }
#' @source Random data generated for the shinyGovstyle package
"transport_data_small"

#' Select Function
#'
#' This function inserts a select box
#' @param inputId Input Id for the component
#' @param label Insert the text for the label
#' @param select_text Add the text that will apply in the drop down as a list
#' @param select_value Add the value that will be used for each selection
#' @return a select input HTML shiny tag object
#' @family Govstyle select inputs
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png",
#'     logo_alt_text = "Ministry of Justice logo"
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "full",
#'     select_Input(
#'       inputId = "sorter",
#'       label = "Sort by",
#'       select_text = c(
#'         "Recently published",
#'         "Recently updated",
#'         "Most views",
#'         "Most comments"
#'       ),
#'       select_value = c("published", "updated", "view", "comments")
#'     ),
#'     shiny::tags$br()
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {}
#' if (interactive()) shinyApp(ui = ui, server = server)
select_Input <- # nolint
  function(
    inputId, # nolint
    label,
    select_text,
    select_value
  ) {
    gov_select <- shiny::tags$div(
      class = "govuk-form-group",
      shiny::tags$label(
        shiny::HTML(label),
        class = "govuk-label",
        `for` = inputId
      ),
      shiny::tags$select(
        id = inputId,
        class = "govuk-select",
        Map(
          function(x, y) {
            shiny::tags$option(value = y, x)
          },
          x = select_text,
          y = select_value
        )
      )
    )
    attachDependency(gov_select)
  }

#' Panel output
#'
#' This function inserts a panel.  Normally used for confirmation screens
#' @param inputId The input slot that will be used to access the value.
#' @param main_text Add the header for the panel
#' @param sub_text Add the main body of text for the panel
#' @return a panel HTML shiny tag object
#' @family Govstyle feedback types
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::gov_layout(size = "full",
#'     shinyGovstyle::panel_output(
#'       inputId = "panel1",
#'       main_text = "Application Complete",
#'       sub_text = paste(
#'         "Thank you for submitting your application.",
#'         "Your reference is xvsiq"
#'       )
#'     ),
#'     shinyGovstyle::footer(full = TRUE)
#'   )
#' )
#'
#' server <- function(input, output, session) {}
#' if (interactive()) shinyApp(ui = ui, server = server)
panel_output <- function(
  inputId, # nolint
  main_text,
  sub_text
) {
  gov_panel <- shiny::tags$div(
    class = "govuk-panel govuk-panel--confirmation",
    id = inputId,
    shiny::tags$h1(main_text, class = "govuk-panel__title"),
    shiny::tags$div(shiny::HTML(sub_text), class = "govuk-panel__body")
  )
  attachDependency(gov_panel)
}

test_that("test panel", {
  check_panel <- panel_output("panelId", "Main", "Second")

  expect_identical(
    check_panel$children[[1]]$children[[1]],
    "Main"
  )

  expect_identical(
    check_panel$children[[2]]$children[[1]],
    shiny::HTML("Second")
  )
})

#' Font Function
#'
#' Loads the GDS Transport font for use in your app. GDS Transport is a
#' restricted typeface that must only be used on GOV.UK domains. If your app
#' is not hosted on a GOV.UK domain, do not call this function — the GOV.UK
#' Frontend CSS will fall back to Helvetica or Arial automatically.
#'
#' See the
#' \href{https://design-system.service.gov.uk/styles/typeface/}{GOV.UK
#' typeface guidance} for full details on when GDS Transport is permitted.
#' @return no value returned. This loads the font CSS file
#' @export
#' @family Govstyle styling
#' @examples
#' ui <- shiny::fluidPage(
#'   font(),
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png")
#' )
#'
#' server <- function(input, output, session) {}
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
font <- function() {
  version <- as.character(packageVersion("shinyGovstyle")[[1]])

  htmltools::htmlDependency(
    name = "font",
    version = version,
    src = c(href = "shinyGovstyle/css"),
    stylesheet = "font.css"
  )
}

#' Word Count Function
#'
#' This function create tracks the word count and should be used with the
#' text area function.
#' @param inputId The input slot of the text area that you want to affect
#' @param input The text input that is associated with the box
#' @param word_limit Change the word limit if needed. Default will keep as
#' what was used in text area component
#' @return no value returned. Updates the word count in a shiny app
#' @family Govstyle text types
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::header(
#'     "Justice", "", logo = "shinyGovstyle/images/moj_logo.png"
#'   ),
#'   gov_layout(
#'     size = "full",
#'     text_area_Input(
#'       inputId = "text_area",
#'       label = "Can you provide more detail?",
#'       hint_label = paste(
#'         "Do not include personal or financial information,",
#'         "like your National Insurance number or credit card details."
#'       ),
#'       word_limit = 300
#'     )
#'   ),
#'   footer(TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   shiny::observeEvent(input$text_area,
#'     word_count(
#'       inputId = "text_area",
#'       input = input$text_area
#'     )
#'   )
#' }
#' if (interactive()) shinyApp(ui = ui, server = server)
word_count <- function(
  inputId, # nolint
  input,
  word_limit = NULL
) {
  if (input == "") {
    word_no <- 0
  } else {
    word_no <- sapply(gregexpr("\\S+", input), length)
  }

  shinyjs::html(id = paste0(inputId, "wc"), html = word_no)

  if (!is.null(word_limit)) {
    shinyjs::html(
      id = paste0(inputId, "wl"),
      html = paste("of the", word_limit, "allowed")
    )
  }
}

#' Skip to main content link
#'
#' This function generates a 'Skip to main content' link, which is
#' typically used by keyboard users to bypass content and
#' navigate directly to the main content of a page.
#'
#' @param id An optional parameter to specify the Id of the main content
#' section, will be automatically preceeded by a hash '#'. Default is
#' "main" to match the "#main" Id within `gov_main()`.
#'
#' @return A Shiny tag representing the 'Skip to main content' link
#' @export
#' @family Govstyle page structure
#' @examples
#' ui <- shiny::fluidPage(
#'   skip_to_main(),
#'   header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"
#'   ),
#'   gov_main_layout(
#'     heading_text("Example heading"),
#'   )
#' )
#'
#' server <- function(input, output, session){}
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
skip_to_main <- function(id = "main") {
  shiny::tags$a(
    href = paste0("#", id),
    class = "govuk-skip govuk-link",
    "Skip to main content"
  )
}

#' Button Function
#'
#' This function create a gov style button
#' @param inputId The input slot that will be used to access the value
#' @param label Display label for the control, or `NULL` for no label
#' @param type The type of button. Options are default, start, secondary and
#' warning. Defaults to "default"
#' @return a HTML button shiny tag object
#' @export
#' @family Govstyle select inputs
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     shinyGovstyle::button_Input(
#'       inputId = "btn1",
#'       label = "Continue",
#'       type = "default"
#'     )
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {}
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
button_Input <- # nolint
  function(
    inputId, # nolint
    label,
    type = "default"
  ) {
    class_input <- "govuk-button"
    if (type == "start") {
      class_input <- "govuk-button govuk-button--start"
    } else if (type == "secondary") {
      class_input <- "govuk-button govuk-button--secondary"
    } else if (type == "warning") {
      class_input <- "govuk-button govuk-button--warning"
    }

    value <- shiny::restoreInput(id = inputId, default = NULL)

    if (type == "start") {
      gov_button <- shiny::tags$button(
        label,
        id = inputId,
        class = paste0(class_input, " action-button"),
        `data-val` = value,
        shiny::tag(
          "svg",
          list(
            class = "govuk-button__start-icon",
            xmlns = "http://www.w3.org/2000/svg",
            width = "17.5",
            height = "19",
            viewBox = "0 0 33 40",
            `aria-hidden` = "true",
            focusable = "false",
            shiny::tag(
              "path",
              list(fill = "currentColor", d = "M0 0h13l20 20-20 20H0l20-20z")
            )
          )
        )
      )
    } else {
      gov_button <- shiny::tags$button(
        label,
        id = inputId,
        class = paste0(class_input, " action-button"),
        `data-val` = value
      )
    }

    attachDependency(gov_button)
  }

#' Tabs Function
#'
#' This function creates a tabs based table. It requires a single dataframe
#' with a grouping variable.
#' @param inputId The Id to access the summary list
#' @param headers input for the row headers value
#' @param info summary information values for the table
#' @param action whenever a change link is needed. Sets input to the value of
#' the headers using lowercase and with underscore to replace gaps. Default
#' set to `FALSE`
#' @param border set if the table should have borders. Default set
#' to `TRUE`
#' @return a summary list table HTML shiny tag object
#' @family Govstyle feedback types
#' @export
#' @examples
#' # Create an example dataset
#' headers <- c(
#'   "Name",
#'   "Date of birth",
#'   "Contact information",
#'   "Contact details"
#' )
#' info <- c(
#'   "Sarah Philips",
#'   "5 January 1978",
#'   "72 Guild Street <br> London <br> SE23 6FH",
#'   "07700 900457 <br> sarah.phillips@example.com"
#' )
#'
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     shinyGovstyle::gov_summary("sumID", headers, info, action = FALSE)
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {}
#' if (interactive()) shinyApp(ui = ui, server = server)
gov_summary <- function(
  inputId, # nolint
  headers,
  info,
  action = FALSE,
  border = TRUE
) {
  if (border) {
    border_class <- "govuk-summary-list"
  } else {
    border_class <- "govuk-summary-list govuk-summary-list--no-border"
  }

  shiny::tags$dl(
    class = border_class,
    id = inputId,
    Map(
      function(x, y, z) {
        shiny::tags$div(
          class = "govuk-summary-list__row",
          shiny::tags$dt(
            class = "govuk-summary-list__key",
            x
          ),
          shiny::tags$dd(
            class = "govuk-summary-list__value",
            shiny::HTML(y)
          ),
          if (action) {
            shiny::tags$dd(
              class = "govuk-summary-list__actions",
              shiny::tags$button(
                "Change",
                id = z,
                class = "govuk-link action-button",
                `data-val` = shiny::restoreInput(id = z, default = NULL)
              )
            )
          }
        )
      },
      x = headers,
      y = info,
      z = action
    )
  )
}

test_that("summary list works", {
  headers <- c(
    "Name",
    "Date of birth",
    "Contact information",
    "Contact details"
  )
  info <- c(
    "Sarah Philips",
    "5 January 1978",
    "72 Guild Street <br> London <br> SE23 6FH",
    "07700 900457 <br> sarah.phillips@example.com"
  )

  summary_check <- gov_summary("sumID", headers, info, action = FALSE)

  expect_equal(length(summary_check$children[[1]]), 4)

  summary_check <- gov_summary(
    "sumID",
    headers,
    info,
    action = TRUE,
    border = FALSE
  )

  expect_identical(
    summary_check$children[[1]]$Name$children[[3]][[3]][[1]][[1]],
    "button"
  )
})

#' Adds the content of www to shinyGovstyle/
#'
#' @importFrom shiny addResourcePath
#'
#' @noRd
.onLoad <- function(...) {
  shiny::addResourcePath(
    "shinyGovstyle",
    system.file("www", package = "shinyGovstyle")
  )
}

#' Checkbox Function
#'
#' This function inserts a checkbox group
#' @param inputId Input Id for the group of checkboxes
#' @param cb_labels Add the names of the options that will appear
#' @param checkboxIds Add the values for each checkbox
#' @param label Insert the text for the checkbox group
#' @param hint_label Insert optional hint/secondary text. Defaults to NULL
#' @param small change the sizing to a small version of the checkbox. Defaults
#' to `FALSE`
#' @param error Whenever you want to include error handle on the component
#' @param error_message If you want a default error message
#' @return a checkbox HTML shiny tag object
#' @family Govstyle select inputs
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   # Required for error handling function
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", 'This is a new service'),
#'   shinyGovstyle::gov_layout(size = "two-thirds",
#'     # Simple checkbox
#'     shinyGovstyle::checkbox_Input(
#'       inputId = "check1",
#'       cb_labels = c("Option 1", "Option 2", "Option 3"),
#'       checkboxIds = c("op1", "op2", "op3"),
#'       label = "Choice option"
#'     ),
#'     # Error checkbox
#'     shinyGovstyle::checkbox_Input(
#'       inputId = "check2",
#'       cb_labels = c("Option 1", "Option 2", "Option 3"),
#'       checkboxIds = c("op1", "op2", "op3"),
#'       label = "Choice option",
#'       hint_label = "Select the best fit",
#'       error = TRUE,
#'       error_message = "Select one"
#'     ),
#'     # Button to trigger error
#'     shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   # Trigger error on blank submit of eventId2
#'   observeEvent(input$submit, {
#'     if (is.null(input$check2)){
#'       shinyGovstyle::error_on(inputId = "check2")
#'     } else {
#'       shinyGovstyle::error_off(inputId = "check2")
#'     }
#'   })
#' }
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
checkbox_Input <- # nolint
  function(
    inputId, # nolint
    cb_labels,
    checkboxIds, # nolint
    label,
    hint_label = NULL,
    small = FALSE,
    error = FALSE,
    error_message = NULL
  ) {
    if (small) {
      class_build <- "govuk-checkboxes govuk-checkboxes--small"
    } else {
      class_build <- "govuk-checkboxes"
    }

    gov_checkboxes <- shiny::tags$div(
      class = "shiny-input-checkboxgroup",
      id = inputId,
      shiny::tags$div(
        class = "govuk-form-group",
        id = paste0(inputId, "div"),
        shiny::tags$fieldset(
          class = "govuk-fieldset",
          shiny::tags$label(label, class = "govuk-label"),
          shiny::tags$div(hint_label, class = "govuk-hint"),
          if (error == TRUE) {
            shinyjs::hidden(
              shiny::tags$p(
                error_message,
                class = "govuk-error-message",
                id = paste0(inputId, "error"),
                shiny::tags$span("Error:", class = "govuk-visually-hidden")
              )
            )
          },
          shiny::tags$div(
            class = class_build,
            Map(
              function(x, y) {
                value <- shiny::restoreInput(id = y, default = FALSE) # nolint
                shiny::tags$div(
                  class = "govuk-checkboxes__item",
                  id = paste0("div_", y),
                  shiny::tags$input(
                    class = "govuk-checkboxes__input",
                    id = y,
                    name = inputId,
                    type = "checkbox",
                    value = y
                  ),
                  shiny::tags$label(
                    x,
                    class = "govuk-label govuk-checkboxes__label"
                  )
                )
              },
              x = cb_labels,
              y = checkboxIds
            )
          )
        )
      )
    )
    attachDependency(gov_checkboxes)
  }

#' Page Layout Functions
#'
#' These function loads the page layout in a gov layout. There is a selection
#' of components that can sit within each other. The gov_main_layout is the
#' overarching layout. The gov_row creates a each row and gov_box creates
#' a box within the row. The gov_text is a container for text bodies.
#' @name layouts
#' @param inputID ID of the main div. Defaults to "main"
#' @param size size of the box in the row. Optional are full, one-half,
#' two-thirds, one-third and one-quarter. Defaults to "full"
#' @param ... include the components of the UI that you want within the
#' main page. These components are made to flow through each other. See
#' example
#' @return a HTML shiny layout div
#' @family Govstyle page structure
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"),
#'   shinyGovstyle::gov_main_layout(
#'     shinyGovstyle::gov_row(
#'       shinyGovstyle::gov_box(
#'         size = "full",
#'         shinyGovstyle::gov_text("govuk-grid-column-full")
#'       )
#'     ),
#'     shinyGovstyle::gov_row(
#'       shinyGovstyle::gov_box(
#'         size = "one-half",
#'         shinyGovstyle::gov_text("govuk-grid-column-one-half")
#'       ),
#'       shinyGovstyle::gov_box(
#'         size = "one-half",
#'         shinyGovstyle::gov_text("govuk-grid-column-one-half")
#'       )
#'     ),
#'     shinyGovstyle::gov_row(
#'       shinyGovstyle::gov_box(
#'         size = "one-third",
#'         shinyGovstyle::gov_text("govuk-grid-column-one-third")
#'       ),
#'       shinyGovstyle::gov_box(
#'         size = "two-third",
#'         shinyGovstyle::gov_text("govuk-grid-column-two-third")
#'       )
#'     ),
#'     shinyGovstyle::gov_row(
#'       shinyGovstyle::gov_box(
#'         size = "one-quarter",
#'         shinyGovstyle::gov_text("govuk-grid-column-one-quarter")
#'       ),
#'       shinyGovstyle::gov_box(
#'         size = "three-quarters",
#'         shinyGovstyle::gov_text("govuk-grid-column-three-quarters")
#'       )
#'     )
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {}
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
NULL

#' @rdname layouts
#' @export
gov_main_layout <- function(
  ...,
  inputID = "main" # nolint
) {
  gov_main <- shiny::tags$div(
    class = "govuk-width-container",
    shiny::tags$main(
      id = inputID,
      role = "main",
      tabindex = "-1",
      class = "govuk-main-wrapper",
      ...
    )
  )
  attachDependency(gov_main)
}

#' @rdname layouts
#' @export
gov_row <- function(...) {
  gov_row <- shiny::tags$div(
    class = "govuk-grid-row",
    ...
  )
  attachDependency(gov_row)
}

#' @rdname layouts
#' @export
gov_box <- function(..., size = "full") {
  gov_box <- shiny::tags$div(
    class = paste0("govuk-grid-column-", size),
    ...
  )
  attachDependency(gov_box)
}

#' @rdname layouts
#' @export
gov_text <- function(...) {
  gov_text <- # nolint
    shiny::tags$p(
      class = "govuk-body",
      ...
    )
}

#' Create a value text box with optional description and colour
#'
#' This function generates a value text box with an optional description and
#' customisable colour.
#'
#' The text box can be used in Shiny applications to display highlighted
#' information, such as statistics or key metrics.
#'
#' @param value Character. The primary value to display in the value box.
#' Defaults to "your value goes here"
#' @param text Character or NA. An optional description to appear below the
#' value. If not provided (default is NA), the description will not be
#' displayed
#' @param colour Character. A colour to apply to the value box. Defaults
#' to "blue". Choose from the following: "grey", "purple", "teal",
#' "blue", "yellow", "orange", "red", "magenta", or "green"
#'
#' @return A Shiny `div` tag representing the value box, styled according
#' to the specified parameters
#' @family Govstyle feedback types
#' @export
#'
#' @examples
#' value_box(
#'   value = "1,000,000",
#'   text = "This is the latest value for the selected inputs.",
#'   colour = "purple"
#' )
value_box <- function(
  value = "your value goes here",
  text = NA,
  colour = "blue"
) {
  #check for deprecated colours
  if (colour == "light-blue") {
    warning(
      "'light-blue' is no longer a supported colour.
      Please select an alternative from:
      'grey', 'purple', 'teal', 'blue', 'yellow',
      'orange', 'red', 'magenta', or 'green'."
    )
  }
  if (colour == "turquoise") {
    warning(
      "'turquoise' is no longer a supported colour.
      Please select an alternative from:
      'grey', 'purple', 'teal', 'blue', 'yellow',
      'orange', 'red', 'magenta', or 'green'."
    )
  }
  if (colour == "pink") {
    warning(
      "'pink' is no longer a supported colour.
      Please select an alternative from:
      'grey', 'purple', 'teal', 'blue', 'yellow',
      'orange', 'red', 'magenta', or 'green'."
    )
  }

  # Use the govuk-tag--<colour> class for coloring
  class_colour <- paste0("govuk-tag--", colour)

  # Conditional rendering of text only if it's provided
  text_tag <- if (!is.na(text)) {
    shiny::tags$div(
      shiny::tags$br(),
      shiny::tags$strong(
        text,
        class = "value-box-description"
      )
    )
  } else {
    NULL
  }

  # Define the value box with the value and optional text
  gov_value <- shiny::tags$div(
    class = paste("value-box-container", class_colour),
    shiny::tags$strong(
      value,
      class = "value-box-value"
    ),
    text_tag
  )

  attachDependency(gov_value)
}

# Test for color class when a specific color is set
test_that("value_box color class works", {
  box_yellow <- value_box("testId2", "Test Value", colour = "yellow")

  # Check that the container class includes both
  # "value-box-container" and the color
  expect_identical(
    box_yellow$attribs$class,
    "value-box-container govuk-tag--yellow"
  )
})

# Test default values (when no text is provided)
test_that("value_box default values work", {
  box_default <- value_box()

  # Check default value
  expect_identical(
    box_default$children[[1]]$children[[1]],
    "your value goes here"
  )

  # Check that text is NULL when not provided
  expect_null(box_default$children[[2]])

  # Check default color class
  expect_identical(
    box_default$attribs$class,
    "value-box-container govuk-tag--blue"
  )
})


test_that("deprecated colours are warned against", {
  expect_warning(
    value_box("testId3", "Test Value", colour = "light-blue"),
    paste(
      "'light-blue' is no longer a supported colour.
      Please select an alternative from:
      'grey', 'purple', 'teal', 'blue', 'yellow',
      'orange', 'red', 'magenta', or 'green'."
    )
  )
})

#' File Input Function
#'
#' This function create a file upload component. It uses the basis of the
#' shiny fileInput function, but restyles the label and adds error onto it.
#'
#' @param inputId The input slot that will be used to access the value
#' @param label Display label for the control, or `NULL` for no label
#' @param multiple Whether the user should be allowed to select and upload
#' multiple files at once. Does not work on older browsers, including Internet
#' Explorer 9 and earlier
#' @param accept A character vector of MIME types; gives the browser a hint of
#' what kind of files the server is expecting
#' @param width	The width of the input,  e.g. `'400px'`, or `'100\%'`
#' @param buttonLabel The label used on the button. Can be text or an HTML tag
#' object
#' @param placeholder	The text to show before a file has been uploaded
#' @param error Whenever to icnlud error handling. Defaults to FALSE
#' @param error_message Message to display on error. Defaults to NULL
#' @return a file input HTML shiny tag object
#' @family Govstyle select inputs
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   # Required for error handling function
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", 'This is a new service'
#'   ),
#'   shinyGovstyle::gov_layout(size = "two-thirds",
#'     # Simple file input
#'     shinyGovstyle::file_Input(inputId = "file1", label = "Upload a file"),
#'     # Error file
#'     shinyGovstyle::file_Input(
#'       inputId = "file2",
#'       label = "Upload a file",
#'       error = TRUE
#'     ),
#'     # Button to trigger error
#'     shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   # Trigger error on blank submit of file2
#'   observeEvent(input$submit, {
#'     if (is.null(input$file2)){
#'       shinyGovstyle::error_on(inputId = "file2")
#'     } else {
#'       shinyGovstyle::error_off(
#'         inputId = "file2"
#'       )
#'     }
#'   })
#' }
#' if (interactive()) shinyApp(ui = ui, server = server)
file_Input <- # nolint
  function(
    inputId, # nolint
    label,
    multiple = FALSE,
    accept = NULL,
    width = NULL,
    buttonLabel = "Choose file", # nolint
    placeholder = "No file chosen",
    error = FALSE,
    error_message = NULL
  ) {
    restored_value <- shiny::restoreInput(id = inputId, default = NULL)

    # Catch potential edge case - ensure that it's either NULL or a data frame.
    if (!is.null(restored_value) && !is.data.frame(restored_value)) {
      warning("Restored value for ", inputId, " has incorrect format.")
      restored_value <- NULL
    }

    if (!is.null(restored_value)) {
      restored_value <- jsonlite::toJSON(restored_value, strict_atomic = FALSE)
    }

    input_tag <- shiny::tags$input(
      id = inputId,
      name = inputId,
      type = "file",
      style = paste0(
        "position: absolute; width: 1px; height: 1px; padding: 0;",
        " margin: -1px; overflow: hidden; clip: rect(0,0,0,0); border: 0;",
        if (!is.null(width)) {
          paste0(" left: ", shiny::validateCssUnit(width), ";")
        } else {
          ""
        }
      ),
      `data-restore` = restored_value
    )

    if (multiple) {
      input_tag$attribs$multiple <- "multiple"
    }
    if (length(accept) > 0) {
      input_tag$attribs$accept <- paste(accept, collapse = ",")
    }

    gov_file <- shiny::div(
      id = paste0(inputId, "div"),
      class = "govuk-form-group",
      style = if (!is.null(width)) {
        paste0("width: ", shiny::validateCssUnit(width), ";")
      },
      shiny::tags$label(label, class = "govuk-label", tabindex = "-1"),
      if (error == TRUE) {
        shinyjs::hidden(
          shiny::tags$p(
            error_message,
            class = "govuk-error-message",
            id = paste0(inputId, "error"),
            shiny::tags$span("Error:", class = "govuk-visually-hidden")
          )
        )
      },
      shiny::div(
        id = paste0(inputId, "file_div"),
        class = "input-group govuk-file-upload",
        style = "display: flex; align-items: center;",
        shiny::tags$label(
          class = paste(
            "govuk-button govuk-button--secondary",
            "govuk-file-upload-button__pseudo-button"
          ),
          style = "margin-bottom: 0; position: relative; overflow: hidden;",
          buttonLabel,
          input_tag
        ),
        shiny::tags$input(
          type = "text",
          class = "govuk-body",
          style = paste0(
            "margin: 0; border: 0; outline: none; width: 98%; flex: 1 1 auto;"
          ),
          placeholder = placeholder,
          readonly = "readonly",
          tabindex = "-1"
        )
      )
    )
    attachDependency(gov_file)
  }

#' Clean heading text function
#' This internal function is used in `heading_text()`
#' to create an automated link between the two
#' @param text string
#' @return a string
#' @keywords internal
clean_heading_text <- function(text) {
  text |>
    stringr::str_replace_all(" ", "_") |>
    stringr::str_to_lower() |>
    stringr::str_remove_all("[^_[a-z]]")
}

#' Radio Button Function
#'
#' This function create radio buttons
#' @param inputId The `input` slot that will be used to access the value
#' @param label Input label
#' @param choices List of values to select from (if elements of the list are
#' named then that name rather than the value is displayed to the user)
#' @param selected The initially selected value.
#' @param inline  If you want the radio inline or not,  Default is FALSE
#' @param small  If you want the smaller versions of radio buttons,  Default
#' is FALSE
#' @param choiceNames,choiceValues Same as in
#' [shiny::checkboxGroupInput()]. List of names and values,
#' respectively, that are displayed to the user in the app and correspond to
#' the each choice (for this reason they must have the same length). If either
#' of these arguments is provided, then the other must be provided and choices
#' must not be provided. The advantage of using both of these over a named list
#' for choices is that choiceNames allows any type of UI object to be passed
#' through (tag objects, icons, HTML code, ...), instead of just simple text
#' @param hint_label Additional hint text you may want to display below the
#' label. Defaults to NULL
#' @param error Whenever you want to include error handle on the component
#' @param error_message If you want a default error message
#' @param custom_class If you want to add additional classes to the radio
#' buttons
#' @return radio buttons HTML shiny tag object
#' @family Govstyle select inputs
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   # Required for error handling function
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", "This is a new service"
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     # Simple radio
#'     shinyGovstyle::radio_button_Input(
#'       inputId = "radio1",
#'       choices = c("Yes", "No", "Maybe"),
#'       label = "Choice option"
#'     ),
#'     # Error radio
#'     shinyGovstyle::radio_button_Input(
#'       inputId = "radio2",
#'       choices = c("Yes", "No", "Maybe"),
#'       label = "Choice option",
#'       hint_label = "Select the best fit",
#'       inline = TRUE,
#'       error = TRUE,
#'       error_message = "Select one"
#'     ),
#'     # Button to trigger error
#'     shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   # Trigger error on blank submit of eventId2
#'   observeEvent(input$submit, {
#'     if (is.null(input$radio2)) {
#'       shinyGovstyle::error_on(inputId = "radio2")
#'     } else {
#'       shinyGovstyle::error_off(
#'         inputId = "radio2"
#'       )
#'     }
#'   })
#' }
#' if (interactive()) shinyApp(ui = ui, server = server)
radio_button_Input <- # nolint
  function(
    inputId, # nolint
    label,
    choices = NULL,
    selected = NULL,
    inline = FALSE,
    small = FALSE,
    choiceNames = NULL, # nolint
    choiceValues = NULL, # nolint
    hint_label = NULL,
    error = FALSE,
    error_message = NULL,
    custom_class = ""
  ) {
    args <- normalizeChoicesArgs2(choices, choiceNames, choiceValues)
    selected <- shiny::restoreInput(id = inputId, default = selected)
    selected <- as.character(selected)
    if (length(selected) > 1) {
      stop("The 'selected' argument must be of length 1")
    }
    options <- generateOptions2(
      inputId,
      selected,
      inline,
      small,
      "radio",
      args$choiceNames,
      args$choiceValues
    )
    div_class <- paste("govuk-form-group govuk-radios", custom_class)
    gov_radio <- shiny::tags$div(
      id = inputId,
      class = div_class,
      shiny::tags$div(
        class = "govuk-form-group",
        id = paste0(inputId, "div"),
        controlLabel2(inputId, label),
        shiny::tags$div(
          hint_label,
          class = "govuk-hint"
        ),
        if (error == TRUE) {
          shinyjs::hidden(
            shiny::tags$p(
              error_message,
              class = "govuk-error-message",
              id = paste0(inputId, "error"),
              shiny::tags$span(
                "Error:",
                class = "govuk-visually-hidden"
              )
            )
          )
        },
        options
      )
    )

    attachDependency(gov_radio, "radio")
  }

controlLabel2 <- # nolint
  function(
    controlName, # nolint
    label
  ) {
    label %AND%
      htmltools::tags$label(
        class = "govuk-label",
        `for` = controlName,
        label
      )
  }

generateOptions2 <- # nolint
  function(
    inputId, # nolint
    selected,
    inline,
    small,
    type = "checkbox",
    choiceNames, # nolint
    choiceValues, # nolint
    session = shiny::getDefaultReactiveDomain()
  ) {
    options <- mapply(
      choiceValues,
      choiceNames,
      FUN = function(value, name) {
        inputTag <- # nolint
          shiny::tags$input(
            type = type,
            name = inputId,
            value = value,
            class = "govuk-radios__input"
          )
        if (is.null(selected) == FALSE & value %in% selected) {
          inputTag$attribs$checked <- "checked" # nolint
        }
        pd <- processDeps2(name, session)
        shiny::tags$div(
          class = "govuk-radios__item",
          inputTag,
          shiny::tags$label(
            pd$html,
            pd$deps,
            class = "govuk-label govuk-radios__label"
          )
        )
      },
      SIMPLIFY = FALSE,
      USE.NAMES = FALSE
    )

    class_build <- "govuk-radios"

    if (inline) {
      class_build <- paste(class_build, "govuk-radios--inline")
    }

    if (small) {
      class_build <- paste(class_build, "govuk-radios--small")
    }

    shiny::div(class = class_build, options)
  }

`%AND%` <- # nolint
  function(x, y) {
    if (!is.null(x) && !anyNA(x) && !is.null(y) && !anyNA(y)) {
      y
    }
  }

processDeps2 <- # nolint
  function(tags, session) {
    ui <- htmltools::takeSingletons(
      tags,
      session$singletons,
      desingleton = FALSE
    )$ui
    ui <- htmltools::surroundSingletons(ui)
    dependencies <- lapply(
      htmltools::resolveDependencies(
        htmltools::findDependencies(ui)
      ),
      shiny::createWebDependency
    )
    names(dependencies) <- NULL
    list(html = htmltools::doRenderTags(ui), deps = dependencies)
  }

normalizeChoicesArgs2 <- # nolint
  function(
    choices,
    choiceNames, # nolint
    choiceValues, # nolint
    mustExist = TRUE # nolint
  ) {
    if (!is.null(choices)) {
      if (!is.null(choiceNames) || !is.null(choiceValues)) {
        warning(
          "Using `choices` argument; ignoring `choiceNames`
              and `choiceValues`."
        )
      }
      choices <- choicesWithNames2(choices)
      choiceNames <- names(choices) # nolint
      choiceValues <- unname(choices) # nolint
      list(
        choiceNames = as.list(choiceNames),
        choiceValues = as.list(as.character(choiceValues))
      )
    } else if (is.null(choiceNames) || is.null(choiceValues)) {
      if (mustExist) {
        stop(
          "Please specify a non-empty vector for `choices` (or, ",
          "alternatively, for both `choiceNames` AND `choiceValues`)."
        )
      } else if (is.null(choiceNames) && is.null(choiceValues)) {
        list(choiceNames = NULL, choiceValues = NULL)
      } else {
        stop(
          "One of `choiceNames` or `choiceValues` was set to ",
          "NULL, but either both or none should be NULL."
        )
      }
    } else {
      if (length(choiceNames) != length(choiceValues)) {
        stop("`choiceNames` and `choiceValues` must have the same length.")
      }
      if (anyNamed2(choiceNames) || anyNamed2(choiceValues)) {
        stop("`choiceNames` and `choiceValues` must not be named.")
      }
      list(
        choiceNames = as.list(choiceNames),
        choiceValues = as.list(as.character(choiceValues))
      )
    }
  }


choicesWithNames2 <- # nolint
  function(choices) {
    listify <- function(obj) {
      makeNamed <- # nolint
        function(x) {
          if (is.null(names(x))) {
            names(x) <- character(length(x))
          }
          x
        }
      res <- lapply(obj, function(val) {
        if (is.list(val)) {
          listify(val)
        } else if (length(val) == 1 && is.null(names(val))) {
          as.character(val)
        } else {
          makeNamed(as.list(val))
        }
      })
      makeNamed(res)
    }
    choices <- listify(choices)
    if (length(choices) == 0) {
      choices
    } else {
      choices <- mapply(
        choices,
        names(choices),
        FUN = function(choice, name) {
          if (!is.list(choice)) {
            choice
          } else {
            if (name == "") {
              stop("All sub-lists in \"choices\" must be named.")
            }
            choicesWithNames2(choice)
          }
        },
        SIMPLIFY = FALSE
      )
      missing <- names(choices) == ""
      names(choices)[missing] <- as.character(choices)[missing]
      choices
    }
  }

anyNamed2 <- # nolint
  function(x) {
    if (length(x) == 0 || is.null(names(x))) {
      FALSE
    } else {
      any(nzchar(names(x)))
    }
  }

#' External link
#'
#' Intentionally basic wrapper for HTML anchor elements making it easier to
#' create safe external links with standard and accessible behaviour. For more
#' information on how the tag is generated, see [htmltools::tags()].
#'
#' @description
#' It is commonplace for external links to open in a new tab, and when we do
#' this we should be careful...
#'
#' This function automatically adds the following to your link:
#' * `target="_blank"` to open in new tab
#' * `rel="noopener noreferrer"` to prevent [reverse tabnabbing](
#' https://owasp.org/www-community/attacks/Reverse_Tabnabbing)
#'
#' By default this function also adds "(opens in new tab)" to your link text
#' to warn users of the behaviour.
#'
#' This also adds "This link opens in a new tab" as a visually hidden span
#' element within the HTML outputted to warn non-visual users of the behaviour.
#'
#' The function will error if you end with a full stop, give a warning for
#' particularly short link text and will automatically trim any leading or
#' trailing white space inputted into link_text.
#'
#' If you are displaying lots of links together and want to save space by
#' avoiding repeating (opens in new tab), then you can set add_warning = FALSE
#' and add a line of text above all of the links saying something like 'The
#' following links open in a new tab'.
#'
#' Related links and guidance:
#'
#' * [Government digital services guidelines on the use of links](
#' https://design-system.service.gov.uk/styles/links/)
#'
#' * [Anchor tag HTML element and its properties](
#' https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a)
#'
#' * [WCAG 2.2 success criteria 2.4.4: Link Purpose (In Context)](
#' https://www.w3.org/WAI/WCAG22/Understanding/link-purpose-in-context)
#'
#' * [Web Accessibility standards link text behaviour](
#' https://www.w3.org/TR/WCAG20-TECHS/G200.html)
#'
#' @param href URL that you want the link to point to
#' @param link_text Text that will appear describing your link, must be
#' descriptive of the page you are linking to. Vague text like 'click here' or
#' 'here' will cause an error, as will ending in a full stop. Leading and
#' trailing white space will be automatically trimmed. If the string is shorter
#' than 7 characters a console warning will be thrown. There is no way to hush
#' this other than providing more detail
#' @param add_warning Boolean for adding "(opens in new tab)" at the end of the
#' link text to warn users of the behaviour. Be careful and consider
#' accessibility before removing the visual warning
#' @param footer Apply standard GDS footer CSS styling. Logical,
#' default = FALSE
#' @return shiny tag object
#' @family Govstyle actions
#' @export
#'
#' @examples
#' external_link("https://shiny.posit.co/", "R Shiny")
#'
#' external_link(
#'   "https://shiny.posit.co/",
#'   "R Shiny",
#'   add_warning = FALSE
#' )
#'
#' # This will trim and show as 'R Shiny'
#' external_link("https://shiny.posit.co/", "  R Shiny")
#'
#' # Example of within text
#' shiny::tags$p(
#'   "Oi, ", external_link("https://shiny.posit.co/", "R Shiny"), " is great."
#' )
#'
#' # Example of multiple links together
#' shiny::tags$h2("Related resources")
#' shiny::tags$p("The following links open in a new tab.")
#' shiny::tags$ul(
#'   shiny::tags$li(
#'     external_link(
#'       "https://shiny.posit.co/",
#'       "R Shiny documentation",
#'       add_warning = FALSE
#'     )
#'   ),
#'   shiny::tags$li(
#'     external_link(
#'       "https://www.python.org/",
#'       "Python documentation",
#'       add_warning = FALSE
#'     )
#'   ),
#'   shiny::tags$li(
#'     external_link(
#'       "https://nextjs.org/",
#'       "Next.js documentation",
#'       add_warning = FALSE
#'     )
#'   )
#' )
external_link <- function(
  href,
  link_text,
  add_warning = TRUE,
  footer = FALSE
) {
  if (!is.logical(add_warning)) {
    stop("add_warning must be a TRUE or FALSE value")
  }

  # Trim white space as I don't trust humans not to accidentally include
  link_text <- stringr::str_trim(link_text)

  # Create a basic check for raw URLs
  is_url <- function(text) {
    url_pattern <- "^(https://|http://|www\\.)"
    grepl(url_pattern, text)
  }

  # Check for vague link text on our list
  if (is_url(link_text)) {
    stop(paste0(
      link_text,
      " has been recognised as a raw URL, please change the link_text value",
      "to a description of the page being linked to instead"
    ))
  }

  # Check against curated data set for link text we should banish into room 101
  if (tolower(link_text) %in% shinyGovstyle::bad_link_text$bad_link_text) {
    stop(
      paste0(
        link_text,
        " is not descriptive enough and has has been recognised as bad link",
        " text, please replace the link_text argument with more descriptive",
        " text."
      )
    )
  }

  # Check if link text ends in a full stop
  if (grepl("\\.$", link_text)) {
    stop("link_text should not end with a full stop")
  }

  # Give a console warning if link text is under 7 characters
  # Arbritary number that allows for R Shiny to be link text without a warning
  if (nchar(link_text) < 7) {
    warning(paste0(
      "the link_text: ",
      link_text,
      ", is shorter than 7 characters, this is",
      " unlikely to be descriptive for users, consider having more detailed",
      " link text"
    ))
  }

  # Assuming all else has passed, make the link text a nice accessible link
  if (add_warning) {
    link_text <- paste(link_text, "(opens in new tab)")
    hidden_span <- NULL # don't have extra hidden text if clear in main text
  } else {
    hidden_span <-
      htmltools::span(class = "sr-only", " (opens in new tab)")
  }

  if (!is.logical(footer)) {
    stop(
      "The footer parameter should logical TRUE/FALSE. Received:\n",
      class(footer),
      ": ",
      footer
    )
  }

  if (footer) {
    link_gds_class <- "govuk-link govuk-footer__link"
  } else {
    link_gds_class <- "govuk-link"
  }

  # Create the link object
  link <- htmltools::tags$a(
    href = href,
    class = link_gds_class,
    htmltools::HTML(paste0(link_text, hidden_span)), # white space hack
    target = "_blank",
    rel = "noopener noreferrer",
    .noWS = c("outside")
  )

  # Attach CSS from inst/www/css/sr-only.css
  dependency <- htmltools::htmlDependency(
    name = "sr-only",
    version = as.character(utils::packageVersion("shinyGovstyle")[[1]]),
    src = c(href = "shinyGovstyle/css"),
    stylesheet = "sr-only.css"
  )

  # Return the link with the CSS attached
  htmltools::attachDependencies(link, dependency, append = TRUE)
}

# Create a test link ==========================================================
test_link <- external_link("https://shiny.posit.co/", "R Shiny")

# Run rest of tests against the test link -------------------------------------
test_that("Returns shiny.tag object", {
  expect_s3_class(test_link, "shiny.tag")
})

test_that("content and URL are correctly formatted", {
  expect_snapshot(test_link)
})

test_that("attributes are attached properly", {
  expect_equal(test_link$attribs$rel, "noopener noreferrer")
  expect_equal(test_link$attribs$target, "_blank")
})

# Rest of tests against the function ==========================================
test_that("Rejects dodgy link text", {
  expect_error(external_link("https://shiny.posit.co/", "Click here"))
  expect_error(external_link("https://shiny.posit.co/", "here"))
  expect_error(external_link("https://shiny.posit.co/", "PDF"))
  expect_error(external_link("https://shiny.posit.co/", "Full stop."))
  expect_error(external_link(
    "https://shiny.posit.co/",
    "https://shiny.posit.co/"
  ))
  expect_error(external_link(
    "https://shiny.posit.co/",
    "http://shiny.posit.co/"
  ))
  expect_error(external_link("https://shiny.posit.co/", "www.google.com"))
})

test_that("Rejects non-boolean for add_warning", {
  expect_error(
    external_link(
      "https://shiny.posit.co/",
      "R Shiny",
      add_warning = "Funky non-boolean"
    ),
    "add_warning must be a TRUE or FALSE value"
  )
})

test_that("New tab warning always stays for non-visual users", {
  test_link_hidden <-
    external_link("https://shiny.posit.co/", "R Shiny", add_warning = FALSE)

  expect_snapshot(
    external_link("https://shiny.posit.co/", "R Shiny", add_warning = FALSE)
  )
})

test_that("Surrounding whitespace shrubbery is trimmed", {
  expect_snapshot(external_link("https://shiny.posit.co/", "   R Shiny"))

  expect_snapshot(
    external_link("https://shiny.posit.co/", "R Shiny      ")
  )

  expect_snapshot(
    external_link("https://shiny.posit.co/", "   R Shiny   ")
  )

  expect_snapshot(
    external_link("https://shiny.posit.co/", "   R Shiny", add_warning = FALSE)
  )

  expect_snapshot(
    external_link("https://shiny.posit.co/", "R Shiny   ", add_warning = FALSE)
  )

  expect_snapshot(
    external_link(
      "https://shiny.posit.co/",
      "   R Shiny   ",
      add_warning = FALSE
    )
  )
})

test_that("Warning appears for short link text and not for long text", {
  expect_warning(
    external_link("https://shiny.posit.co/", "R"),
    paste0(
      "the link_text: R, is shorter than 7 characters, this is",
      " unlikely to be descriptive for users, consider having more detailed",
      " link text"
    )
  )

  expect_no_warning(external_link("https://shiny.posit.co/", "R Shiny"))
})

test_that("Footer flag works as expected", {
  expect_equal(
    external_link(
      "www.test-link.co.uk",
      "test link",
      footer = FALSE
    )$attribs$class,
    "govuk-link"
  )

  expect_equal(
    external_link(
      "www.test-link.co.uk",
      "test link",
      footer = TRUE
    )$attribs$class,
    "govuk-link govuk-footer__link"
  )

  expect_error(
    external_link("www.test-link.co.uk", "test link", footer = "TRUE"),
    paste0(
      "The footer parameter should logical TRUE/FALSE. Received:\n",
      "character: TRUE"
    )
  )

  expect_error(
    external_link("www.test-link.co.uk", "test link", footer = 0),
    paste0(
      "The footer parameter should logical TRUE/FALSE. Received:\n",
      "numeric: 0"
    )
  )
})

#' Gov List function
#'
#' @param list vector of list
#' @param style options: "none", "bullet", "number". defaults to "none"
#' @family Govstyle text types
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", 'This is a new service'
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     shinyGovstyle::heading_text("gov_list", size = "s"),
#'     shinyGovstyle::gov_text("List:"),
#'     gov_list(list = c("a", "b", "c")),
#'     shinyGovstyle::gov_text("Bulleted list:"),
#'     gov_list(list = c("a", "b", "c"), style = "bullet"),
#'     shinyGovstyle::gov_text("Numbered list:"),
#'     gov_list(list = c("one", "two", "three"), style = "number")
#'   )
#' )
#'
#' server <- function(input, output, session) {}
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
gov_list <- function(list, style = "none") {
  # check style argument
  if (!style %in% c("none", "bullet", "number")) {
    stop(
      cat(
        'style argument must be one of the following values:
        "none"
        "bullet"
        "number"
        '
      )
    )
  }

  # create list wrapper
  list_wrapper <- function(x) {
    # get list style class
    if (style == "bullet") {
      list_style_class <- "govuk-list--bullet"
    } else if (style == "number") {
      list_style_class <- "govuk-list--number"
    } else {
      list_style_class <- ""
    }

    # get shiny tag to use (ordered list or unordered list)
    if (style == "number") {
      shiny_tag <- shiny::tags$ol
    } else {
      shiny_tag <- shiny::tags$ul
    }

    # put together wrapper
    shiny_tag(
      class = stringr::str_c("govuk-list", list_style_class, sep = " "),
      x
    )
  }

  # apply wrapper over list to get full list
  result <- list_wrapper(purrr::map(list, function(x) {
    shiny::tags$li(x)
  }))
  attachDependency(result)
}

# Check list length
test_that("list length", {
  gov_list_check <- gov_list(list = c("a", "b", "c"))

  expect_equal(3, length(gov_list(list = c("a", "b", "c"))$children[[1]]))
})

# Check list type
test_that("bulleted list type", {
  gov_list_check <- gov_list(list = c("a", "b", "c"))

  expect_equal("ul", gov_list_check[[1]])
})

# Check list class
test_that("bulleted list class", {
  gov_list_check <- gov_list(list = c("a", "b", "c"))

  expect_equal("govuk-list ", gov_list_check$attribs$class[[1]])
})

# Check bullet list type
test_that("bulleted list type", {
  gov_list_check <- gov_list(list = c("a", "b", "c"), style = "bullet")

  expect_equal("ul", gov_list_check[[1]])
})

# Check bullet list class
test_that("bulleted list class", {
  gov_list_check <- gov_list(list = c("a", "b", "c"), style = "bullet")

  expect_equal(
    "govuk-list govuk-list--bullet",
    gov_list_check$attribs$class[[1]]
  )
})


# Check numbered list type
test_that("numbered list type", {
  gov_list_check <- gov_list(list = c("a", "b", "c"), style = "number")

  expect_equal("ol", gov_list_check[[1]])
})

# Check numbered list class
test_that("numbered list class", {
  gov_list_check <- gov_list(list = c("a", "b", "c"), style = "number")

  expect_equal(
    "govuk-list govuk-list--number",
    gov_list_check$attribs$class[[1]]
  )
})

#' Error Summary Function
#'
#' This function loads the error summary component to display error text.
#' This replicates the gov style error boxes linked below:
#' https://design-system.service.gov.uk/components/error-summary/
#' @param inputId The input slot that will be used to access the value
#' @param error_title The title for the error summary
#' @param error_list A list of text values to be displayed in the error body
#' @return an error_summary HTML shiny tag object
#' @family Govstyle errors
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     error_summary(
#'       inputId = "errorId",
#'       error_title = "Error title",
#'       error_list = c("error item1", "error item2")
#'     )
#'   ),
#'   shinyGovstyle::button_Input("btn1", "Change error summary"),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   shiny::observeEvent(input$btn1, {
#'     error_summary_update(
#'       "errorId",
#'       c("error item1", "error item2", "error item3")
#'     )},
#'     ignoreInit = TRUE
#'   )
#' }
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
error_summary <- function(
  inputId, # nolint
  error_title,
  error_list
) {
  gov_error_summary <-
    shiny::tags$div(
      id = inputId,
      class = "govuk-error-summary",
      shiny::tags$h2(error_title, class = "govuk-error-summary__title"),
      shiny::tags$div(
        id = paste0(inputId, "list"),
        class = "govuk-error-summary__body",
        shiny::tags$ul(
          class = "govuk-list govuk-error-summary__list",
          Map(
            function(x) {
              shiny::tags$li(x)
            },
            x = error_list
          )
        )
      )
    )
  attachDependency(gov_error_summary)
}

test_that("error_summary works", {
  error_summary <- error_summary(
    "error1",
    "Error Title",
    c("error entry 1", "error entry 2")
  )

  expect_identical(
    error_summary$attribs$id,
    "error1"
  )

  expect_identical(
    error_summary$attribs$class,
    "govuk-error-summary"
  )

  expect_snapshot(error_summary)
})

#' Input Field Function
#'
#' This function inserts number of text inputs. Useful for addresses.
#' @param legend Legend that goes above the fieldset
#' @param labels A list of labels for the text inputs
#' @param inputIds A list input slots that will be used to access the value
#' @param widths control the size of the box based on number of characters
#' required.  Options are 30, 20, 10, 5, 4, 3, 2. NULL will not limit the size
#' @param types text box types. Will default to text
#' @param error Whenever to icnlud error handling. Defaults to FALSE
#' @param error_message Message to display on error. Defaults to NULL
#' @return a input field of HTML as a shiny tag object
#' @family Govstyle text types
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   # Required for error handling function
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", 'This is a new service'
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     shinyGovstyle::input_field(
#'       legend ="List of three text boxes in a field",
#'       labels = c("Field 1", "Field 2", "Field 3"),
#'       inputIds = c("field1", "field2", "field3"),
#'       widths = c(30,20,10),
#'       error = TRUE
#'     ),
#'    # Button to trigger error
#'    shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   # Trigger error on blank submit of field2
#'   observeEvent(input$submit, {
#'     if (input$field2 == ""){
#'       shinyGovstyle::error_on(
#'         inputId = "field2",
#'         error_message = "Please complete"
#'       )
#'     } else {
#'       shinyGovstyle::error_off(
#'         inputId = "field2"
#'       )
#'     }
#'   })
#' }
#' if (interactive()) shinyApp(ui = ui, server = server)
input_field <- function(
  legend,
  labels,
  inputIds, # nolint
  widths = NULL,
  types = "text",
  error = FALSE,
  error_message = NULL
) {
  if (is.null(widths)) {
    widths <- rep_len(0, length(inputIds))
  }
  gov_input_field <- shiny::tags$fieldset(
    class = "govuk-fieldset",
    shiny::tags$legend(
      legend,
      class = "govuk-fieldset__legend govuk-fieldset__legend--xl"
    ),
    Map(
      function(x, y, z, a) {
        if (z == 0) {
          width_class <- "govuk-input"
        } else {
          width_class <- paste0("govuk-input govuk-input--width-", z)
        }
        shiny::tags$div(
          class = "govuk-form-group",
          id = paste0(y, "div"),
          shiny::tags$label(shiny::HTML(x), class = "govuk-label"),
          if (error) {
            shinyjs::hidden(
              shiny::tags$p(
                error_message,
                class = "govuk-error-message",
                id = paste0(y, "error"),
                shiny::tags$span("Error:", class = "govuk-visually-hidden")
              )
            )
          },
          shiny::tags$input(id = y, class = width_class, type = a)
        )
      },
      x = labels,
      y = inputIds,
      z = widths,
      a = types
    )
  )
  attachDependency(gov_input_field)
}

test_that("field works", {
  field_check <- input_field(
    legend = "List of three text boxes in a field",
    labels = c("Field 1", "Field 2", "Field 3"),
    inputIds = c("field1", "field2", "field3"),
    widths = c(30, 20, 10),
    error = TRUE
  )

  expect_equal(length(field_check), 3)

  expect_identical(
    paste(
      field_check$children[[2]]$`Field 1`$children[[2]]$attribs$class,
      field_check$children[[2]]$`Field 1`$children[[2]]$attribs[3]$class
    ),
    "govuk-error-message shinyjs-hide"
  )
})

test_that("field works with null width", {
  field_check <- input_field(
    legend = "List of three text boxes in a field",
    labels = c("Field 1", "Field 2", "Field 3"),
    inputIds = c("field1", "field2", "field3")
  )

  expect_equal(length(field_check), 3)
})

#' Text Input Function
#'
#' This function create a text input.
#' @param inputId The input slot that will be used to access the value
#' @param label Display label for the control, or `NULL` for no label
#' @param hint_label Display hint label for the control, or `NULL` for
#' no hint label
#' @param type Type of text input to accept. Defaults to text
#' @param width control the size of the box based on number of characters
#' required.  Options are 30, 20, 10, 5, 4, 3, 2.  NULL will not limit the size
#' @param error Whenever to include error handling. Defaults to FALSE
#' @param error_message Message to display on error. Defaults to NULL
#' @param prefix Add a prefix to the box. Defaults to NULL
#' @param suffix Add a suffix to the box. Defaults to NULL
#' @return a text input HTML shiny tag object
#' @family Govstyle text types
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   # Required for error handling function
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", 'This is a new service'
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     # Simple text box
#'     shinyGovstyle::text_Input(inputId = "eventId", label = "Event Name"),
#'     # Error text box
#'     shinyGovstyle::text_Input(
#'       inputId = "eventId2",
#'       label = "Event Name",
#'       hint_label = "This can be found on the letter",
#'       error = TRUE
#'     ),
#'     # Button to trigger error
#'     shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   # Trigger error on blank submit of eventId2
#'   observeEvent(input$submit, {
#'     if (input$eventId2 != "") {
#'       shinyGovstyle::error_off(inputId = "eventId2")
#'     } else {
#'       shinyGovstyle::error_on(
#'         inputId = "eventId2",
#'         error_message = "Please complete"
#'       )
#'     }
#'   })
#' }
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
text_Input <- # nolint
  function(
    inputId, # nolint
    label,
    hint_label = NULL,
    type = "text",
    width = NULL,
    error = FALSE,
    error_message = NULL,
    prefix = NULL,
    suffix = NULL
  ) {
    if (is.null(width)) {
      width_class <- "govuk-input"
    } else {
      width_class <- paste0("govuk-input govuk-input--width-", width)
    }
    gov_text <- shiny::tags$div(
      class = "govuk-form-group",
      id = paste0(inputId, "div"),
      shiny::tags$label(shiny::HTML(label), class = "govuk-label"),
      shiny::tags$div(hint_label, class = "govuk-hint"),
      if (error == TRUE) {
        shinyjs::hidden(
          shiny::tags$p(
            error_message,
            class = "govuk-error-message",
            id = paste0(inputId, "error"),
            shiny::tags$span("Error:", class = "govuk-visually-hidden")
          )
        )
      },
      if (is.null(prefix) & is.null(suffix)) {
        shiny::tags$input(id = inputId, class = width_class, type = type)
      } else if (is.null(suffix)) {
        shiny::tags$div(
          class = "govuk-input__wrapper",
          shiny::tags$div(
            prefix,
            class = "govuk-input__prefix",
            `aria-hidden` = "true"
          ),
          shiny::tags$input(id = inputId, class = width_class, type = type)
        )
      } else if (is.null(prefix)) {
        shiny::tags$div(
          class = "govuk-input__wrapper",
          shiny::tags$input(id = inputId, class = width_class, type = type),
          shiny::tags$div(
            suffix,
            class = "govuk-input__suffix",
            `aria-hidden` = "true"
          )
        )
      } else {
        shiny::tags$div(
          class = "govuk-input__wrapper",
          shiny::tags$div(
            prefix,
            class = "govuk-input__prefix",
            `aria-hidden` = "true"
          ),
          shiny::tags$input(id = inputId, class = width_class, type = type),
          shiny::tags$div(
            suffix,
            class = "govuk-input__suffix",
            `aria-hidden` = "true"
          )
        )
      }
    )
    attachDependency(gov_text)
  }

#' Error Summary Update Function
#'
#' This function changes the text that displays in the error summary box.
#' Requires `shinyjs::useShinyjs()` to work.
#' @param inputId The input Id of the error summary you want to update
#' @param error_list An updated list of text values to be displayed in the
#' error body
#' @return an update error summary box
#' @family Govstyle errors
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     error_summary(
#'       inputId = "errorId",
#'       error_title = "Error title",
#'       error_list = c("error item1", "error item2")
#'     )
#'   ),
#'   shinyGovstyle::button_Input("btn1", "Change error summary"),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   shiny::observeEvent(input$btn1, {
#'     error_summary_update(
#'       "errorId",
#'       c("error item1", "error item2", "error item3")
#'     )
#'   },
#'   ignoreInit = TRUE
#'   )
#' }
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
error_summary_update <- function(
  inputId, # nolint
  error_list
) {
  new_err_sum <-
    shiny::tags$ul(
      class = "govuk-list govuk-error-summary__list",
      Map(
        function(x) {
          shiny::tags$li(x)
        },
        x = error_list
      )
    )

  shinyjs::html(id = paste0(inputId, "list"), html = as.character(new_err_sum))
}

#' Warning Text Function
#'
#' This function create warning text.
#' @param inputId The input slot that will be used to access the value
#' @param text Text that goes in the main
#' @return a warning box HTML shiny tag object
#' @family Govstyle feedback types
#' @export
#' @examples
#' shinyGovstyle::warning_text(
#'   inputId = "warn1",
#'   text = "You can be fined up to £5,000 if you do not register."
#' )
warning_text <- function(
  inputId, # nolint
  text
) {
  gov_warning <- shiny::tags$div(
    class = "govuk-warning-text",
    id = inputId,
    shiny::tags$span(
      "!",
      class = "govuk-warning-text__icon",
      `aria-hidden` = "true"
    ),
    shiny::tags$strong(
      text,
      class = "govuk-warning-text__text",
      shiny::tags$span("Warning", class = "govuk-visually-hidden")
    )
  )
  attachDependency(gov_warning)
}

#' Heading Text Function
#'
#' This function createS heading text
#' @param text_input Text to display
#' @param size Text size using xl, l, m, s. Defaults to xl
#' @param id Custom header id
#' @param level Heading level, integer between 1 and 6. Defaults to 1
#' @return a heading text HTML shiny tag object
#' @family Govstyle text types
#' @export
#' @examples
#' shinyGovstyle::heading_text("This is great text")
#' shinyGovstyle::heading_text("This is great text", size = "l", level = 2)
heading_text <- function(text_input, size = "xl", id, level = 1) {
  if (missing(id)) {
    id <- clean_heading_text(text_input)
  }

  if (!level %in% 1:6) {
    stop("level must be an integer between 1 and 6")
  }

  heading_tag <- paste0("h", level)
  gov_heading <- do.call(
    shiny::tags[[heading_tag]],
    list(
      shiny::HTML(text_input),
      class = paste0("govuk-heading-", size),
      id = id
    )
  )
  attachDependency(gov_heading)
}

test_that("service_name renders with correct class", {
  h <- header(org_name = "Test", service_name = "My Service")
  html <- as.character(h)
  expect_true(grepl("govuk-header__service-name", html))
  expect_true(grepl("My Service", html))
})


test_that("omitting service_name omits header__content div", {
  h <- header(org_name = "Test")
  html <- as.character(h)
  expect_false(grepl("govuk-header__content", html))
})


test_that("main_text is deprecated in favour of org_name", {
  rlang::local_options(lifecycle_verbosity = "error")
  expect_error(header(main_text = "Test"), class = "defunctError")
})


test_that("secondary_text is deprecated in favour of service_name", {
  rlang::local_options(lifecycle_verbosity = "error")
  expect_error(header(secondary_text = "My Service"), class = "defunctError")
})


test_that("secondary_link is deprecated", {
  rlang::local_options(lifecycle_verbosity = "error")
  expect_error(header(secondary_link = "test text"), class = "defunctError")
})


test_that("main_link is deprecated", {
  rlang::local_options(lifecycle_verbosity = "error")
  expect_error(header(main_link = "http://example.com"), class = "defunctError")
})


test_that("main_alt_text is deprecated", {
  rlang::local_options(lifecycle_verbosity = "error")
  expect_error(header(main_alt_text = "Alt text"), class = "defunctError")
})


test_that("function still runs if using deprecated argument", {
  rlang::local_options(lifecycle_verbosity = "quiet")
  expect_silent(
    header(secondary_link = "test text")
  )
})


test_that("warning when logo used without logo_alt_text", {
  expect_warning(
    header(org_name = "Test", logo = "test.png", logo_alt_text = NULL),
    "Please use logo_alt_text"
  )
})


test_that("crown logo renders SVG and no img", {
  h <- header(org_name = "Test", logo = "crown")
  html <- as.character(h)
  expect_true(grepl("<svg", html))
  expect_false(grepl("<img", html))
})


test_that("custom logo renders img with correct src and alt", {
  h <- header(
    org_name = "Test",
    logo = "path/to/logo.png",
    logo_alt_text = "My Logo"
  )
  html <- as.character(h)
  expect_true(grepl("<img", html))
  expect_true(grepl("path/to/logo.png", html))
  expect_true(grepl("My Logo", html))
})


test_that("NULL logo renders no img or svg", {
  h <- header(org_name = "Test", logo = NULL)
  html <- as.character(h)
  expect_false(grepl("<img", html))
  expect_false(grepl("<svg", html))
})


test_that("heading_text returns correct heading level", {
  # Test default level (should be h1)
  h1 <- heading_text("Test", size = "xl")
  expect_true(grepl("<h1", as.character(h1)))

  # Test level 2
  h2 <- heading_text("Test", size = "l", level = 2)
  expect_true(grepl("<h2", as.character(h2)))

  # Test level 3
  h3 <- heading_text("Test", size = "m", level = 3)
  expect_true(grepl("<h3", as.character(h3)))

  # Test invalid level throws error
  expect_error(
    heading_text("Test", level = 0),
    "level must be an integer between 1 and 6"
  )
  expect_error(
    heading_text("Test", level = 7),
    "level must be an integer between 1 and 6"
  )
})

test_that("default", {
  head_text <- heading_text("Test Time")

  expect_identical(
    head_text$attribs$class,
    "govuk-heading-xl"
  )

  expect_identical(
    shiny::HTML("Test Time"),
    head_text$children[[1]]
  )
})

test_that("medium_works", {
  head_text <- heading_text("Test Time", "m")

  expect_identical(
    head_text$attribs$class,
    "govuk-heading-m"
  )

  expect_identical(
    shiny::HTML("Test Time"),
    head_text$children[[1]]
  )
})

#' Service navigation
#'
#' @description
#' Service navigation component consistent with the
#' [GDS service navigation](https://design-system.service.gov.uk/components/service-navigation/). # nolint
#'
#' **Note:** This component uses a hardcoded element ID
#' for the navigation list and mobile menu toggle. Using
#' multiple `service_navigation()` instances on the same
#' page will cause ID conflicts and the mobile menu toggle
#' may not work correctly.
#'
#' @param links A vector of actionLinks to be added to the
#' service navigation. inputIDs are auto-generated by
#' lowercasing the link text and replacing all
#' non-alphanumeric characters with underscores, e.g.
#' "Overview page" becomes `overview_page`. For precise
#' control over inputIDs, supply a named vector where names
#' are the page titles and values are the inputIDs.
#' @param service_name An optional character string
#' containing the service name to be displayed in the
#' navigation bar
#'
#' @returns Shiny tag object
#' @family Govstyle navigation
#' @export
#'
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header("Title", "Secondary heading"),
#'   shinyGovstyle::service_navigation(
#'      c("Summary data", "Detailed stats 1", "User guide")
#'   ),
#'   bslib::navset_hidden(
#'      id = "main_panels",
#'      bslib::nav_panel(
#'        "summary_data",
#'        shiny::tags$h2("Summary data")
#'      ),
#'      bslib::nav_panel(
#'        "detailed_stats_1",
#'        shiny::tags$h2("Detailed stats 1")
#'      ),
#'      bslib::nav_panel(
#'        "user_guide",
#'        shiny::tags$h2("User guide")
#'      )
#'    ),
#'    shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   observeEvent(
#'     input$summary_data,
#'     bslib::nav_select("main_panels", "summary_data")
#'   )
#'   observeEvent(
#'     input$detailed_stats_1,
#'     bslib::nav_select("main_panels", "detailed_stats_1")
#'   )
#'   observeEvent(
#'     input$user_guide,
#'     bslib::nav_select("main_panels", "user_guide")
#'   )
#' }
#'
#' if (interactive()) shiny::shinyApp(ui = ui, server = server)
service_navigation <- function(
  links,
  service_name = NULL
) {
  if (is.null(links) || length(links) == 0) {
    stop("links must be a non-empty character vector")
  }
  if (is.null(names(links))) {
    link_names <- links
  } else {
    link_names <- names(links)
    link_names[link_names == ""] <- links[link_names == ""]
  }
  navigation <- shiny::tags$section(
    `aria-label` = "Service information",
    class = "govuk-service-navigation",
    `data-module` = "govuk-service-navigation",
    shiny::tags$div(
      class = "govuk-width-container",
      shiny::tags$div(
        class = "govuk-service-navigation__container",
        if (!is.null(service_name)) {
          shiny::tags$span(
            class = "govuk-service-navigation__service-name",
            shiny::tags$a(
              href = "#",
              class = "govuk-service-navigation__link",
              service_name
            )
          )
        },
        shiny::tags$nav(
          `aria-label` = "Menu",
          class = "govuk-service-navigation__wrapper",
          shiny::tags$button(
            type = "button",
            class = paste(
              "govuk-service-navigation__toggle",
              "govuk-js-service-navigation-toggle"
            ),
            `aria-controls` = "navigation",
            hidden = TRUE,
            "Menu"
          ),
          shiny::tags$ul(
            class = "govuk-service-navigation__list",
            id = "navigation",
            mapply(
              service_nav_link,
              links,
              link_names,
              SIMPLIFY = FALSE,
              USE.NAMES = FALSE
            )
          )
        )
      )
    )
  )
  attachDependency(navigation, "service_navigation") # nolint
}


#' Update the active item in a service navigation component
#'
#' @description
#' Sends a message to the browser to update the highlighted active item in the
#' service navigation bar.
#'
#' **When you need this function:** when navigation is triggered
#' programmatically — for example, via a next / back button or a footer link
#' that points to a main page. In those cases the nav link itself is not
#' clicked, so the JavaScript binding does not fire and the active state does
#' not update automatically. Call `update_service_navigation()` alongside
#' your tab-switching call to keep them in sync.
#'
#' **When you don't need this function:** when the user clicks a service
#' navigation link directly. The JavaScript binding updates the active state
#' automatically, so you only need to switch the tab panel in your
#' `observeEvent()`.
#'
#' @param session The Shiny session object
#' @param inputId The inputId of the service navigation link to set as active
#'
#' @returns NULL, called for side effects
#' @family Govstyle navigation
#' @export
#'
#' @examples
#' # Nav link clicked — JS handles active state, just switch the panel.
#' # Works the same whether you use shiny or bslib tab panels.
#' if (interactive()) {
#'   server <- function(input, output, session) {
#'     # shiny tabsetPanel
#'     shiny::observeEvent(input$page_two, {
#'       shiny::updateTabsetPanel(session, "tabs", selected = "page_two")
#'     })
#'
#'     # bslib navset
#'     shiny::observeEvent(input$page_two, {
#'       bslib::nav_select("tabs", "page_two")
#'     })
#'   }
#' }
#'
#' # Programmatic navigation (e.g. a next / back button) — the nav link is not
#' # clicked, so you must also call update_service_navigation() explicitly.
#' if (interactive()) {
#'   server <- function(input, output, session) {
#'     # shiny tabsetPanel
#'     shiny::observeEvent(input$next_btn, {
#'       shiny::updateTabsetPanel(session, "tabs", selected = "page_two")
#'       shinyGovstyle::update_service_navigation(session, "page_two")
#'     })
#'
#'     # bslib navset
#'     shiny::observeEvent(input$next_btn, {
#'       bslib::nav_select("tabs", "page_two")
#'       shinyGovstyle::update_service_navigation(session, "page_two")
#'     })
#'   }
#' }
update_service_navigation <- function(
  session,
  inputId # nolint: object_name_linter.
) {
  session$sendCustomMessage(
    "update_service_navigation",
    inputId
  )
}


#' Create a service navigation link for use in `service_navigation()` function
#'
#' @param link Character string containing either link text or url
#' @param link_name Name of a link where a URL has been provided in link_text
#'
#' @returns HTML tag list item
#' @noRd
#' @keywords internal
#' @examples
#' # Internal (i.e. within dashboard) link
#' shinyGovstyle:::service_nav_link("Cookie statement")
#' # Named internal link
#' shinyGovstyle:::service_nav_link("cookie_statement", "Cookies")
service_nav_link <- function(
  link,
  link_name = NULL
) {
  if (is.null(link_name)) {
    warning("Link name provided is NULL for ", link)
    link_name <- link
  }
  shiny::tags$li(
    class = "govuk-service-navigation__item",
    shiny::actionLink(
      class = "govuk-service-navigation__link",
      inputId = tolower(gsub("[^a-zA-Z0-9]", "_", link)),
      label = link_name
    )
  )
}

test_that("service_navigation works", {
  expect_no_error(service_navigation(links = c("Page 1", "Page 2", "Page 3")))

  expect_no_error(
    service_navigation(
      service_name = "My app",
      c(
        "Page 1" = "p1",
        "Page 2" = "p2",
        "Page 3" = "p3"
      )
    )
  )

  expect_no_error(service_navigation("My app"))
})

test_that("service_navigation HTML is as expected", {
  local_edition(3)
  expect_snapshot(service_navigation(links = c("Page 1", "Page 2", "Page 3")))
  expect_snapshot(
    service_navigation(
      service_name = "My app",
      c(
        "Page 1" = "p1",
        "Page 2" = "p2",
        "Page 3" = "p3"
      )
    )
  )
})

test_that("service_navigation works with a single link", {
  expect_no_error(service_navigation(links = "Solo page"))
  local_edition(3)
  expect_snapshot(service_navigation(links = "Solo page"))
})

test_that("service_navigation errors with empty vector", {
  expect_error(service_navigation(links = character(0)))
})

test_that("service_navigation errors with NULL links", {
  expect_error(service_navigation(links = NULL))
})

#' Cookie Banner Function
#'
#' This function creates a cookie banner. You need to have
#' `shinyjs::useShinyjs()` enabled for this to work. All the Ids are preset.
#' See example for how to structure.
#' @param service_name Name for this service to add to banner
#' @return a cookie banner HTML shiny tag object
#' @family Govstyle page structure
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"),
#'   #Needs shinyjs to work
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::cookieBanner("The best thing"),
#'   shinyGovstyle::gov_layout(size = "two-thirds"),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   # Set of observeEvent to create a path through the cookie banner
#'   observeEvent(input$cookieAccept, {
#'     shinyjs::show(id = "cookieAcceptDiv")
#'     shinyjs::hide(id = "cookieMain")
#'   })
#'
#'   observeEvent(input$cookieReject, {
#'     shinyjs::show(id = "cookieRejectDiv")
#'     shinyjs::hide(id = "cookieMain")
#'   })
#'
#'   observeEvent(input$hideAccept, {
#'     shinyjs::toggle(id = "cookieDiv")
#'   })
#'
#'   observeEvent(input$hideReject, {
#'     shinyjs::toggle(id = "cookieDiv")
#'   })
#'
#'   observeEvent(input$cookieLink, {
#'     # Need to link here to where further info is located. You can use
#'     # updateTabsetPanel to have a cookie page for instance
#'   })
#'
#' }
#' if (interactive()) shinyApp(ui = ui, server = server)
cookieBanner <- # nolint
  function(service_name) {
    gov_cookie_link <- shiny::actionLink(
      inputId = "cookieLink",
      label = "View cookies",
      class = "govuk-link"
    )

    cookie_banner_input <-
      shiny::tags$div(
        id = "cookieDiv",
        class = "govuk-cookie-banner",
        `data-nosnippet role` = "region",
        `aria-label` = paste("Cookies on", service_name),
        shiny::tags$div(
          id = "cookieMain",
          class = "govuk-cookie-banner__message govuk-width-container",
          shiny::tags$div(
            class = "govuk-grid-row",
            shiny::tags$div(
              class = "govuk-grid-column-two-thirds",
              shiny::tags$h2(
                class = "govuk-cookie-banner__heading govuk-heading-m",
                paste("Cookies on", service_name)
              ),
              shiny::tags$div(
                class = "govuk-cookie-banner__content",
                shiny::tags$p(
                  class = "govuk-body",
                  "We use some essential cookies to make this service work."
                ),
                shiny::tags$p(
                  class = "govuk-body",
                  "We\u0027d also use analytics cookies so we can understand
                how you use the service and make improvements."
                )
              )
            )
          ),
          shiny::tags$div(
            class = "govuk-button-group",
            button_Input("cookieAccept", "Accept analytics cookies"),
            button_Input("cookieReject", "Reject analytics cookies"),
            gov_cookie_link
          )
        ),
        shinyjs::hidden(
          shiny::tags$div(
            id = "cookieAcceptDiv",
            class = "govuk-cookie-banner__message govuk-width-container",
            shiny::tags$div(
              class = "govuk-grid-row",
              shiny::tags$div(
                class = "govuk-grid-column-two-thirds",
                shiny::tags$div(
                  class = "govuk-cookie-banner__content",
                  shiny::tags$p(
                    class = "govuk-body",
                    paste(
                      "You\u0027ve accepted additional cookies. You can",
                      "change your cookie settings at any time."
                    )
                  )
                )
              )
            ),
            shiny::tags$div(
              class = "govuk-button-group",
              button_Input("hideAccept", "Hide this message")
            )
          )
        ),
        shinyjs::hidden(
          shiny::tags$div(
            id = "cookieRejectDiv",
            class = "govuk-cookie-banner__message govuk-width-container",
            shiny::tags$div(
              class = "govuk-grid-row",
              shiny::tags$div(
                class = "govuk-grid-column-two-thirds",
                shiny::tags$div(
                  class = "govuk-cookie-banner__content",
                  shiny::tags$p(
                    class = "govuk-body",
                    paste(
                      "You\u0027ve rejected additional cookies. You can",
                      "change your cookie settings at any time."
                    )
                  )
                )
              )
            ),
            shiny::tags$div(
              class = "govuk-button-group",
              button_Input("hideReject", "Hide this message")
            )
          )
        )
      )

    attachDependency(cookie_banner_input)
  }

#' Label with Hint Function
#'
#' This function inserts a label and optional hint.
#' @param inputId The input slot that will be used to access the value
#' @param label Display label for the control, or `NULL` for no label
#' @param hint_input Display hint label for the control, or `NULL` for
#' no hint label
#' @return a label hint HTML shiny tag object
#' @family Govstyle feedback types
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     label_hint(
#'       inputId = "label1",
#'       label = "This is a label",
#'       hint_input = "This is a hint"
#'     )
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {}
#' if (interactive()) shinyApp(ui = ui, server = server)
label_hint <- function(
  inputId, # nolint
  label,
  hint_input = NULL
) {
  gov_label <- shiny::tags$div(
    class = "govuk-form-group",
    id = inputId,
    shiny::tags$label(shiny::HTML(label), class = "govuk-label"),
    shiny::tags$div(hint_input, class = "govuk-hint")
  )
  attachDependency(gov_label)
}

test_that("label hint workss", {
  label_check <- label_hint("hintID", "Upper", "Lower")

  expect_identical(
    label_check$children[[1]]$children[[1]],
    shiny::HTML("Upper")
  )

  expect_identical(
    label_check$children[[2]]$children[[1]],
    "Lower"
  )
})

#' Back Link Function
#'
#' This function adds a back link to the page
#' @param inputId The input slot that will be used to access the value
#' @param label The link text for the backlink, default is "Back"
#' @return a backlink HTML shiny tag object
#' @family Govstyle navigation
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shiny::navlistPanel(
#'     "",
#'     id = "nav",
#'     widths = c(2, 10),
#'     well = FALSE,
#'     # Create first panel
#'     shiny::tabPanel(
#'       "Select Types",
#'       value = "panel1",
#'       gov_layout(
#'         size = "two-thirds",
#'         backlink_Input("link1"),
#'         shiny::tags$br(),
#'         shiny::tags$br()
#'       )
#'     ),
#'     shiny::tabPanel(
#'       "Tab2",
#'       value = "panel2"
#'     )
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   # Slightly confused in that it goes forward rather than back
#'   # but shows how to use
#'   observeEvent(input$link1, {
#'     updateTabsetPanel(session, "nav", selected = "panel2")
#'   })
#' }
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
backlink_Input <- # nolint
  function(
    inputId, # nolint
    label = "Back"
  ) {
    value <- shiny::restoreInput(id = inputId, default = NULL)
    gov_back_link <- shiny::actionLink(
      label = label,
      inputId = inputId,
      class = paste0("govuk-back-link"),
      `data-val` = value
    )
    attachDependency(gov_back_link)
  }

#' Lookup for bad link text
#'
#' A single column data frame, listing out known examples of bad link text that
#' check for in the `external_link()` function.
#'
#' We've started curating this list so we can create automated checks to help
#' all link text to be as descriptive as possible in line with
#' [WCAG 2.2 success criteria 2.4.4: Link Purpose (In Context)](
#' https://www.w3.org/WAI/WCAG22/Understanding/link-purpose-in-context).
#'
#' @format ## `bad_link_text`
#' A data frame with 53 rows and 1 columns:
#' \describe{
#'   \item{bad_link_text}{Lower cased examples of non-descriptive link text}
#' }
#' @source Curated by explore.statistics@@education.gov.uk
"bad_link_text"

#' Insert Text Function
#'
#' This function loads the insert text component to display additional
#' information in a special format.
#' @param inputId The input slot that will be used to access the value
#' @param text Text that you want to display on the insert
#' @return a insert text HTML shiny tag object
#' @family Govstyle feedback types
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     shinyGovstyle::insert_text(
#'       inputId = "note",
#'       text = paste(
#'         "It can take up to 8 weeks to register a lasting power of",
#'         "attorney if there are no mistakes in the application."
#'       )
#'     )
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {}
#' if (interactive()) shinyApp(ui = ui, server = server)
insert_text <- # nolint
  function(
    inputId, # nolint
    text
  ) {
    gov_insert <- shiny::tags$div(
      shiny::HTML(text),
      id = inputId,
      class = "govuk-inset-text"
    )
    attachDependency(gov_insert)
  }

#' Download link
#'
#' @description
#' The `download_link()` provides a standard way to provide a download
#' link, which facilitates important accessible / positive user experience
#' elements, namely:
#' \itemize{
#' \item file type
#' \item file size
#' }
#' These are necessary in order for users to understand what they are
#' downloading, both in terms of being able to decide if they are comfortable
#' with downloading the file over their current connection and if it's in a form
#' they're able to deal with once it is downloaded. If the exact file size is
#' not easily determined, then it may be acceptable to provide an estimate or an
#' upper limit.
#'
#' @param outputId The name of the output slot that the
#' `shiny::downloadHandler()` is assigned to
#' @param link_text Text that will appear describing the download action.
#' Vague text like 'click here' or 'here' will cause an error, as will ending in
#' a full stop. Leading and trailing white space will be automatically trimmed.
#' If the string is shorter than 7 characters a console warning will be thrown.
#' There is no way to hush this other than providing more detail
#' @param file_type The file type to be download (default: CSV)
#' @param file_size The file size if known. Needs to be a string ending in one
#' of KB, MB, GB or rows
#'
#' @returns shiny tag object
#' @family Govstyle actions
#' @export
#'
#' @examples
#' ui <- shiny::fluidPage(
#'   gov_text("Choose a data set to download."),
#'   select_Input(
#'     "dataset",
#'     "Data set",
#'     select_text = c("Car road tests", "New York air quality"),
#'     select_value = c("mtcars", "airquality")
#'   ),
#'   gov_text(
#'     download_link(
#'       "download_data",
#'       "Download selected data set",
#'       file_size = "4 KB"
#'     )
#'   )
#' )
#'
#' server <- function(input, output) {
#'   # The requested data set
#'   data <- reactive({
#'     get(input$dataset)
#'   })
#'
#'   output$download_data <- downloadHandler(
#'     filename = function() {
#'       # Use the selected dataset as the suggested file name
#'       paste0(input$dataset, ".csv")
#'     },
#'     content = function(file) {
#'       # Write the dataset to the `file` that will be downloaded
#'       write.csv(data(), file)
#'     }
#'   )
#' }
#'
#' if (interactive()) shiny::shinyApp(ui, server)
download_link <- function(
  outputId, # nolint
  link_text,
  file_type = "CSV",
  file_size = NULL
) {
  # Trim white space as I don't trust humans not to accidentally include
  link_text <- stringr::str_trim(link_text)

  # Create a basic check for raw URLs
  is_url <- function(text) {
    url_pattern <- "^(https://|http://|www\\.)"
    grepl(url_pattern, text)
  }

  # Check for vague link text on our list
  if (is_url(link_text)) {
    stop(paste0(
      link_text,
      " has been recognised as a raw URL, please change the link_text value",
      "to a description of the page being linked to instead"
    ))
  }

  # Check against curated data set for link text we should banish into room 101
  if (tolower(link_text) %in% shinyGovstyle::bad_link_text$bad_link_text) {
    stop(
      paste0(
        link_text,
        " is not descriptive enough and has has been recognised as bad link",
        " text, please replace the link_text argument with more descriptive",
        " text."
      )
    )
  }

  # Check if link text ends in a full stop
  if (grepl("\\.$", link_text)) {
    stop("link_text should not end with a full stop")
  }

  # Give a console warning if link text is under 7 characters
  # Arbritary number that allows for R Shiny to be link text without a warning
  if (nchar(link_text) < 7) {
    warning(paste0(
      "the link_text: ",
      link_text,
      ", is shorter than 7 characters, this is",
      " unlikely to be descriptive for users, consider having more detailed",
      " link text"
    ))
  }

  # Assuming all else has passed, make the link text a nice accessible link
  # Note the file_size is manually assigned and optional right now. Ideally,
  # this would be updated dynamically when linking to a dynamically created
  # file, such as the CSV version of a table in an app.
  if (!is.null(file_size)) {
    if (!grepl("KB$|MB$|GB$| rows$", file_size)) {
      stop("File size should be a string ending in one of KB, MB, GB or rows.")
    }
    file_info <- paste0(file_type, ", ", file_size)
  } else {
    warning(
      paste0(
        outputId,
        ": download_link file_size is NULL. ",
        "Please add a file_size estimate or upper limit wherever possible."
      )
    )
    file_info <- file_type
  }
  link_text <- paste0(link_text, " (", file_info, ")")

  # Create the link object
  link <- htmltools::tags$a(
    id = outputId,
    class = "shiny-download-link disabled",
    href = "",
    target = "_blank",
    download = NA,
    htmltools::HTML(link_text)
  )

  # Attach CSS from inst/www/css/visually-hidden.css
  dependency <- htmltools::htmlDependency(
    name = "sr-only",
    version = as.character(utils::packageVersion("shinyGovstyle")[[1]]),
    src = c(href = "shinyGovstyle/css"),
    stylesheet = "sr-only.css"
  )

  # Return the link with the CSS attached
  htmltools::attachDependencies(link, dependency, append = TRUE)
}

# Create a test link ==========================================================
test_link <- download_link(
  "download_data",
  "Download specific data set",
  file_size = "12 KB"
)

# Run rest of tests against the test link -------------------------------------
test_that("Returns shiny.tag object", {
  expect_s3_class(test_link, "shiny.tag")
})

test_that("content and URL are correctly formatted", {
  expect_equal(test_link$attribs$href, "")
  expect_true(grepl("Download specific data set", test_link$children[[1]]))
})

test_that("File type and size correctly append", {
  expect_true(grepl("\\(CSV", test_link$children[[1]]))
  expect_true(grepl("12 KB\\)", test_link$children[[1]]))
})

test_that("attributes are attached properly", {
  expect_equal(test_link$attribs$target, "_blank")
})

# Rest of tests against the function ==========================================
test_that("Rejects dodgy link text", {
  expect_error(download_link("download_data", "Download data"))
  expect_error(download_link("download_data", "Click here"))
  expect_error(download_link("download_data", "here"))
  expect_error(download_link("download_data", "PDF"))
  expect_error(download_link("download_data", "Full stop."))
  expect_error(download_link("download_data", "http://shiny.posit.co/"))
  expect_error(download_link("download_data", "www.google.com"))
})

test_that("Surrounding whitespace shrubbery is trimmed", {
  expect_equal(
    paste0(
      download_link(
        "download_data",
        "   Download specific data set",
        file_size = "96 MB"
      )$children[[1]]
    ),
    "Download specific data set (CSV, 96 MB)"
  )

  expect_equal(
    paste0(download_link(
      "download_data",
      "Download specific data set    ",
      file_size = "96 MB"
    )$children[[1]]),
    "Download specific data set (CSV, 96 MB)"
  )

  expect_equal(
    paste0(download_link(
      "download_data",
      "   Download specific data set   ",
      file_size = "96 MB"
    )$children[[1]]),
    "Download specific data set (CSV, 96 MB)"
  )
})

test_that("Warning appears for short link text and not for long text", {
  expect_warning(
    download_link("download_data", "R", file_size = "96 MB"),
    paste0(
      "the link_text: R, is shorter than 7 characters, this is",
      " unlikely to be descriptive for users, consider having more detailed",
      " link text"
    )
  )

  expect_no_warning(
    download_link(
      "download_data",
      "Download specific data set",
      file_size = "96 MB"
    )
  )
})

test_that("file_size is valid", {
  expect_warning(
    download_link("download_data", "Download your data"),
    paste0(
      "download_data",
      ": download_link file_size is NULL. ",
      "Please add a file_size estimate or upper limit wherever possible."
    )
  )

  expect_no_warning(
    download_link("download_data", "Download your data", file_size = "12 KB")
  )

  expect_error(
    download_link("download_data", "Download your data", file_size = "12 JV")
  )
})

#' Accordion Function
#'
#' This function inserts a accordion
#' @param inputId Input Id for the accordion
#' @param titles Add the titles for the accordion
#' @param descriptions Add the main text for the accordion
#' @return an accordion HTML shiny tag object
#' @family Govstyle tables tabs and accordions
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png",
#'     logo_alt_text = "Ministry of Justice logo"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", 'This is a new service'
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     accordion(
#'       "acc1",
#'       c(
#'         "Writing well for the web",
#'         "Writing well for specialists",
#'         "Know your audience",
#'         "How people read"
#'       ),
#'       c(
#'         "This is the content for Writing well for the web.",
#'         "This is the content for Writing well for specialists.",
#'         "This is the content for Know your audience.",
#'         "This is the content for How people read."
#'       )
#'     )
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {}
#'
#' if (interactive()) shiny::shinyApp(ui = ui, server = server)
accordion <- function(
  inputId, # nolint
  titles,
  descriptions
) {
  count_array <- seq_along(titles)

  accordion_div <-
    shiny::tags$div(
      class = "govuk-accordion js-enabled",
      `data-module` = "govuk-accordion",
      id = inputId,
      shiny::tags$div(
        class = "govuk-accordion__controls",
        shiny::tags$button(
          class = "govuk-accordion__show-all",
          `aria-expanded` = "false",
          shiny::tags$span(
            id = "show-all-chevron",
            class = paste(
              "govuk-accordion-nav__chevron",
              "govuk-accordion-nav__chevron--down"
            ),
          ),
          shiny::tags$span(
            class = "govuk-accordion__show-all-text",
            "Show all sections"
          )
        )
      ),
      Map(
        function(x, y, z) {
          if (z < 10) {
            z_str <- paste0("0", z)
          } else {
            z_str <- paste0(z)
          }

          shiny::tags$div(
            class = "govuk-accordion__section",
            shiny::tags$div(
              class = "govuk-accordion__section-header",
              shiny::tags$h2(
                class = "govuk-accordion__section-header",
                shiny::tags$button(
                  class = "govuk-accordion__section-button",
                  id = paste0("accordion-default-heading-", z_str),
                  name = paste0("accordion-default-heading-", z_str),
                  shiny::tags$span(
                    class = "govuk-accordion__section-heading-text",
                    shiny::tags$span(
                      class = "govuk-accordion__section-heading-text-focus",
                      x
                    )
                  ),
                  shiny::tags$span(
                    class = paste(
                      "govuk-visually-hidden",
                      "govuk-accordion__section-heading-divider"
                    )
                  ),
                  shiny::tags$span(
                    class = "govuk-accordion__section-toggle",
                    shiny::tags$span(
                      class = "govuk-accordion__section-toggle-focus",
                      shiny::tags$span(
                        class = paste(
                          "govuk-accordion-nav__chevron",
                          "govuk-accordion-nav__chevron--down"
                        )
                      ),
                      shiny::tags$span(
                        class = "govuk-accordion__section-toggle-text",
                        "Show",
                        shiny::tags$span(
                          class = "govuk-visually-hidden",
                          "this section"
                        )
                      )
                    )
                  )
                )
              )
            ),
            shiny::tags$div(
              id = "accordion-default-content-1",
              class = "govuk-accordion__section-content",
              `aria-labelledby` = paste0("accordion-default-heading-", z_str),
              shiny::tags$p(
                class = "govuk-body",
                y
              )
            )
          )
        },
        x = titles,
        y = descriptions,
        z = count_array
      )
    )
  attachDependency(accordion_div, "accordion")
}

test_that("accordion works", {
  accordion_check <- accordion(
    "acc1",
    c(
      "Writing well for the web",
      "Writing well for specialists",
      "Know your audience",
      "How people read"
    ),
    c(
      "This is the content for Writing well for the web.",
      "This is the content for Writing well for specialists.",
      "This is the content for Know your audience.",
      "This is the content for How people read."
    )
  )

  expect_equal(length(accordion_check$children[[2]]), 4)
})


test_that("accordion numbering works past 9", {
  accordion_numbering_check <- accordion(
    "acc1",
    c(
      "Writing well for the web",
      "Writing well for specialists",
      "Know your audience",
      "How people read",
      "Accordian title 5",
      "Accordion title 6",
      "Accordion title 7",
      "Accordion title 8",
      "Accordion title 9",
      "Accordion title 10",
      "Accordion title 11",
      "Accordion title 12"
    ),
    c(
      "This is the content for Writing well for the web.",
      "This is the content for Writing well for specialists.",
      "This is the content for Know your audience.",
      "This is the content for How people read.",
      "Accordian content 5",
      "Accordion content 6",
      "Accordion content 7",
      "Accordion content 8",
      "Accordion content 9",
      "Accordion content 10",
      "Accordion content 11",
      "Accordion content 12"
    )
  )

  heading1 <- accordion_numbering_check$children[[2]][[1]]$children[[
    1
  ]]$children[[1]]$children[[1]]$attribs$name
  heading11 <- accordion_numbering_check$children[[2]][[11]]$children[[
    1
  ]]$children[[1]]$children[[1]]$attribs$name

  expect_equal(stringr::str_sub(heading1, -2), "01")
  expect_equal(stringr::str_sub(heading11, -2), "11")
})

#' Date Input Function
#'
#' This function create a date input that follows GDS component
#' @param inputId The input slot that will be used to access the value
#' @param label Display label for the control, or `NULL` for no label
#' @param hint_label Display hint label for the control, or `NULL` for no
#' hint label
#' @param error Whenever to include error components.Defaults to `FALSE`
#' @param error_message Error handling message? Defaults to `NULL`
#' @param day Select a default day on start up. Defaults to `NULL`
#' @param month Select a default month on start up. Defaults to `NULL`
#' @param year Select a default year on start up. Defaults to `NULL`
#' @return a data input HTML shiny tag object
#' @family Govstyle text types
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   # Required for error handling function.
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", 'This is a new service'
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     # Simple date input
#'     shinyGovstyle::date_Input(
#'       inputId = "dob_input",
#'       label = "Please enter your birthday"
#'     ),
#'     # Error date input
#'     shinyGovstyle::date_Input(
#'       inputId = "dob_input2",
#'       label = "Please enter your birthday",
#'       hint_label = "For example, 12 11 2007",
#'       error = TRUE
#'     ),
#'     # Button to trigger error
#'     shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   # Trigger error on blank submit of dob_input2
#'   observeEvent(input$submit, {
#'     if (input$dob_input2 == "//") {
#'       shinyGovstyle::error_on(inputId = "dob_input2")
#'     } else {
#'       shinyGovstyle::error_off(inputId = "dob_input2")
#'     }
#'   })
#' }
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
date_Input <- # nolint
  function(
    inputId, # nolint
    label,
    hint_label = NULL,
    error = FALSE,
    error_message = NULL,
    day = NULL,
    month = NULL,
    year = NULL
  ) {
    value <- shiny::restoreInput(id = inputId, default = FALSE) # nolint
    gov_date <- shiny::tags$div(
      class = "govuk-form-group",
      id = paste0(inputId, "div"),
      shiny::tags$fieldset(
        class = "govuk-fieldset",
        shiny::tags$label(shiny::HTML(label), class = "govuk-label"),
        if (error == TRUE) {
          shinyjs::hidden(
            shiny::tags$p(
              error_message,
              id = paste0(inputId, "error"),
              class = "govuk-error-message",
              shiny::tags$span("Error:", class = "govuk-visually-hidden")
            )
          )
        },
        shiny::tags$div(hint_label, class = "govuk-hint"),
        shiny::tags$div(
          class = "govuk-date-input",
          id = inputId,
          shiny::tags$div(
            class = "govuk-date-input__item",
            shiny::tags$div(
              class = "govuk-form-group",
              shiny::tags$label(
                "Day",
                class = "govuk-label govuk-date-input__label"
              ),
              shiny::tags$input(
                class = paste(
                  "govuk-input govuk-date-input__input",
                  "
                            govuk-input--width-2"
                ),
                id = paste0(inputId, "_day"),
                name = inputId,
                type = "number",
                pattern = "[0-9]*",
                value = day
              )
            )
          ),
          shiny::tags$div(
            class = "govuk-date-input__item",
            shiny::tags$div(
              class = "govuk-form-group",
              shiny::tags$label(
                "Month",
                class = "govuk-label govuk-date-input__label"
              ),
              shiny::tags$input(
                class = paste(
                  "govuk-input govuk-date-input__input",
                  "govuk-input--width-2"
                ),
                id = paste0(inputId, "_month"),
                name = inputId,
                type = "number",
                pattern = "[0-9]*",
                value = month
              )
            )
          ),
          shiny::tags$div(
            class = "govuk-date-input__item",
            shiny::tags$div(
              class = "govuk-form-group",
              shiny::tags$label(
                "Year",
                class = "govuk-label govuk-date-input__label"
              ),
              shiny::tags$input(
                class = paste(
                  "govuk-input govuk-date-input__input",
                  "govuk-input--width-4"
                ),
                id = paste0(inputId, "_year"),
                name = inputId,
                type = "number",
                pattern = "[0-9]*",
                value = year
              )
            )
          )
        )
      )
    )
    attachDependency(gov_date, "date")
  }

#' Tag Function
#'
#' This function creates a tag.
#' @param inputId The Id to access the tag
#' @param text The text in the tag
#' @param colour The colour of the tag. Default is navy. Other options are
#' grey, green, teal, blue, purple, magenta, red, orange and yellow
#' @return a tag HTML shiny tag object
#' @family Govstyle feedback types
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     shinyGovstyle::tag_Input("tag1", "Complete"),
#'     shinyGovstyle::tag_Input("tag2", "Incomplete", "red")
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {}
#' if (interactive()) shinyApp(ui = ui, server = server)
tag_Input <- # nolint
  function(
    inputId, # nolint
    text,
    colour = "navy"
  ) {
    #check for deprecated colours
    if (colour == "light-blue") {
      warning(
        "'light-blue' is no longer a supported colour.
        Please select an alternative from:
       'navy', 'grey', 'purple', 'teal', 'blue', 'yellow',
        'orange', 'red', 'magenta', or 'green'."
      )
    }
    if (colour == "turquoise") {
      warning(
        "'turquoise' is no longer a supported colour.
      Please select an alternative from:
      'grey', 'purple', 'teal', 'blue', 'yellow',
      'orange', 'red', 'magenta', or 'green'."
      )
    }
    if (colour == "pink") {
      warning(
        "'pink' is no longer a supported colour.
      Please select an alternative from:
      'grey', 'purple', 'teal', 'blue', 'yellow',
      'orange', 'red', 'magenta', or 'green'."
      )
    }

    class_colour <- "govuk-tag"
    if (colour != "navy") {
      class_colour <- paste0("govuk-tag govuk-tag--", colour)
    }

    gov_tag <- shiny::tags$strong(
      id = inputId,
      class = class_colour,
      text
    )
    attachDependency(gov_tag)
  }

#' Download with extension radios
#'
#' @param id Shiny element Id. Default is "download_radios", but must be
#' customised to be unique if multiple instances of this module are being
#' used in a single app. Must match up to the ID
#' of a `download_radios_helper()` instance in server code
#' @param download_type Element being downloaded. Expected to be along the
#' lines of "underlying data", "table", "current data view". Default is "table"
#' @param file_types File formats to offer, can be any combination of
#' "CSV", "XLSX", "ODS". Default is a vector of all 3
#' @param file_sizes Estimated file sizes for each file format. This needs to
#' be a vector of the same length as file_types
#' @param small Set radio buttons to small size (logical, default: FALSE)
#'
#' @returns UI containing radio selection and download button
#' @family Govstyle actions
#' @export
#'
#' @examples
#' ui <- shiny::fluidPage(
#'   download_radios("download_file",
#'     file_types = c("CSV", "ODS"),
#'     file_sizes = c("2 KB", "5 KB")
#'   )
#' )
#'
#' server <- function(input, output, session) {
#'   output$download_file <- download_radios_handler(
#'     "download_file",
#'     file_name = "simple_data_frame",
#'     file_contents = mtcars
#'   )
#' }
#'
#' if (interactive()) shinyApp(ui, server)
download_radios <- function(
  id = "download_radios",
  download_type = "table",
  file_types = c("CSV", "ODS", "XLSX"),
  file_sizes = c("< 1 GB", "< 1 GB", "< 1 GB"),
  small = FALSE
) {
  allowed_extensions <- c("CSV", "ODS", "XLSX")
  if (is.null(file_types)) {
    file_types <- allowed_extensions
  }
  if (any(!(toupper(file_types) %in% allowed_extensions))) {
    stop(
      "File types must one of \"CSV\", \"ODS\", \"XLSX\".\nValue provided: ",
      setdiff(file_types, allowed_extensions) |> paste(collapse = ", ")
    )
  }
  shiny::tags$div(
    radio_button_Input(
      inputId = shiny::NS(id, "file_extension"),
      choiceNames = paste0(file_types, " (", file_sizes, ")"),
      choiceValues = file_types,
      selected = file_types[1],
      label = "Select file format for download:",
      small = small
    ),
    gov_text(
      htmltools::tags$a(
        id = shiny::NS(id, "download_file"),
        class = "shiny-download-link govuk-button",
        href = "",
        target = "_blank",
        download = NA,
        paste("Download", download_type)
      )
    )
  )
}

#' Download with extension radios handler
#'
#' @param id Shiny element Id. Default is "download_radios", but must be
#' customised to be unique if multiple instances of this module are being
#' used in a single app. Must match up to the ID of a `download_radios()`
#' instance in UI code
#' @param file_name Name of the file to be downloaded
#' @param file_contents Contents to write to the download file
#' @family Govstyle actions
#' @returns Output for use with `download_radios()`
#' @export
#'
#' @inherit download_radios examples
download_radios_handler <- function(
  id = "download_radios",
  file_name,
  file_contents
) {
  if (!(is.character(file_name) && length(file_name) == 1)) {
    stop("The provided file_name must be a single string variable.")
  }
  if (!is.data.frame(file_contents)) {
    stop("The provided file_contents need to be a data frame.")
  }
  shiny::moduleServer(
    id,
    module = function(input, output, session) {
      output$download_file <- shiny::downloadHandler(
        filename = function() {
          # Use the selected dataset as the suggested file name
          paste0(file_name, ".", tolower(input$file_extension))
        },
        content = function(file) {
          # Write the dataset to the `file` that will be downloaded
          if (input$file_extension == "CSV") {
            readr::write_csv(file_contents, file)
          } else if (input$file_extension == "XLSX") {
            writexl::write_xlsx(file_contents, file)
          } else if (input$file_extension == "ODS") {
            readODS::write_ods(file_contents, file, row_names = FALSE)
          } else {
            warning(
              "Unrecognised file format provided: ",
              input$file_extension
            )
          }
        }
      )
    }
  )
}
#

test_that("Suffixes present", {
  expect_true(
    download_radios() |>
      paste(collapse = ",") |>
      stringr::str_detect(c("CSV", "ODS", "XLSX")) |>
      all()
  )
  expect_true(
    download_radios(
      file_types = c("CSV", "ODS"),
      file_sizes = c("1 KB", "3 KB")
    ) |>
      paste(collapse = ",") |>
      stringr::str_detect(c("CSV", "ODS")) |>
      all()
  )
  expect_true(
    download_radios(
      file_types = c("CSV", "ODS"),
      file_sizes = c("1 KB", "3 KB")
    ) |>
      paste(collapse = ",") |>
      stringr::str_detect(c("XLSX")) |>
      isFALSE()
  )
  expect_error(
    download_radios_handler(
      file_name = c("sdf", "sdfsdgf"),
      file_contents = data.frame(z = c(1))
    )
  )
  expect_error(
    download_radios_handler(file_name = "sdfsdf", file_contents = 1)
  )
})

#' Header Function
#'
#' This function create a header banner. For use at top of the screen
#' @param org_name Organisation name that goes in the header
#' @param service_name Service name to supplement the organisation name
#' @param logo Add a link to a logo which will apply in the header. Use crown to
#' use the crown SVG version on GOV UK
#' @param main_text `r lifecycle::badge("deprecated")` Use `org_name` instead
#' @param secondary_text `r lifecycle::badge("deprecated")` Use `service_name`
#' instead
#' @param main_link Add a link for clicking on main text
#' `r lifecycle::badge("deprecated")`
#' @param secondary_link Add a link for clicking on secondary
#' header `r lifecycle::badge("deprecated")`
#' @param logo_alt_text Add alternative text for the logo. Should be used when a
#' logo is used
#' @param main_alt_text Add alternative text for the main link. Should be used
#' when a main link is used `r lifecycle::badge("deprecated")`
#' @param secondary_alt_text Add alternative text for the secondary link. Should
#' be used when a secondary link is used `r lifecycle::badge("deprecated")`
#' @param logo_width Change the logo size width CSS to improve fit
#' @param logo_height Change the logo size height CSS to improve fit
#' @return a header HTML shiny tag object
#' @family Govstyle page structure
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png",
#'     logo_alt_text = "Ministry of Justice Logo"
#'   )
#' )
#'
#' server <- function(input, output, session) {}
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
header <- function(
  org_name = "Shiny example app",
  service_name = NULL,
  logo = "shinyGovstyle/images/Dept_logo.svg",
  main_text = lifecycle::deprecated(),
  secondary_text = lifecycle::deprecated(),
  main_link = NULL,
  secondary_link = NULL,
  logo_alt_text = "Departmental logo",
  main_alt_text = NULL,
  secondary_alt_text = NULL,
  logo_width = 66,
  logo_height = 34
) {
  if (lifecycle::is_present(main_text)) {
    lifecycle::deprecate_warn(
      when = "0.2.0",
      what = "header(main_text)",
      with = "header(org_name)"
    )
    org_name <- main_text
  }

  if (lifecycle::is_present(secondary_text)) {
    lifecycle::deprecate_warn(
      when = "0.2.0",
      what = "header(secondary_text)",
      with = "header(service_name)"
    )
    service_name <- secondary_text
  }

  # checks for alt text
  if (!is.null(logo) && is.null(logo_alt_text)) {
    warning(
      paste(
        "Please use logo_alt_text to provide alternative",
        "text for the logo you used."
      )
    )
  }

  if (lifecycle::is_present(main_link)) {
    lifecycle::deprecate_warn(
      when = "0.2.0",
      what = "header(main_link)",
      details = "main_link will be dropped in v1.0.0"
    )
  }

  if (lifecycle::is_present(main_alt_text)) {
    lifecycle::deprecate_warn(
      when = "0.2.0",
      what = "header(main_alt_text)",
      details = "main_alt_text will be dropped in v1.0.0"
    )
  }

  if (lifecycle::is_present(secondary_alt_text)) {
    lifecycle::deprecate_warn(
      when = "0.2.0",
      what = "header(secondary_alt_text)",
      details = "secondary_alt_text will be dropped in v1.0.0"
    )
  }

  if (lifecycle::is_present(secondary_link)) {
    lifecycle::deprecate_warn(
      when = "0.2.0",
      what = "header(secondary_link)",
      details = "secondary_link will be dropped in v1.0.0"
    )
  }

  gov_header <- shiny::tags$header(
    class = "govuk-header",
    role = "banner",
    shinyjs::inlineCSS(paste0(
      ".govuk-header__logotype-crown-fallback-image {width: ",
      logo_width,
      "px;
      height: ",
      logo_height,
      "px;}",
      ".govuk-header__link:focus .govuk-header__logotype-crown-fallback-image",
      ", .govuk-header__link:active .govuk-header__logotype-crown-fallback-",
      "image {filter: invert(1);}"
    )),
    shiny::tags$div(
      class = "govuk-header__container govuk-width-container",
      shiny::tags$div(
        class = "govuk-header__logo",
        shiny::tags$span(
          shiny::tags$span(
            class = "govuk-header__logotype",
            if (!is.null(logo) && logo == "crown") {
              shiny::tags$svg(
                `aria-hidden` = "true",
                focusable = "false",
                class = "govuk-header__logotype-crown",
                xmlns = "http://www.w3.org/2000/svg",
                viewBox = "0 0 132 97",
                height = "30",
                width = "36",
                shiny::tags$path(
                  fill = "currentColor",
                  `fill-rule` = "evenodd",
                  d = paste0(
                    "M25 30.2c3.5 1.5 7.7-.2 9.1-3.7 1.5-3.6-.2-7.8-3.9-9.2-3",
                    ".6-1.4-7.6.3-9.1 3.9-1.4 3.5.3 7.5 3.9 9zM9 39.5c3.6 1.5",
                    " 7.8-.2 9.2-3.7 1.5-3.6-.2-7.8-3.9-9.1-3.6-1.5-7.6.2-9.1",
                    " 3.8-1.4 3.5.3 7.5 3.8 9zM4.4 57.2c3.5 1.5 7.7-.2 9.1-3.",
                    "8 1.5-3.6-.2-7.7-3.9-9.1-3.5-1.5-7.6.3-9.1 3.8-1.4 3.5.3",
                    " 7.6 3.9 9.1zm38.3-21.4c3.5 1.5 7.7-.2 9.1-3.8 1.5-3.6-.",
                    "2-7.7-3.9-9.1-3.6-1.5-7.6.3-9.1 3.8-1.3 3.6.4 7.7 3.9 9.",
                    "1zm64.4-5.6c-3.6 1.5-7.8-.2-9.1-3.7-1.5-3.6.2-7.8 3.8-9.",
                    "2 3.6-1.4 7.7.3 9.2 3.9 1.3 3.5-.4 7.5-3.9 9zm15.9 9.3c-",
                    "3.6 1.5-7.7-.2-9.1-3.7-1.5-3.6.2-7.8 3.7-9.1 3.6-1.5 7.7",
                    ".2 9.2 3.8 1.5 3.5-.3 7.5-3.8 9zm4.7 17.7c-3.6 1.5-7.8-.",
                    "2-9.2-3.8-1.5-3.6.2-7.7 3.9-9.1 3.6-1.5 7.7.3 9.2 3.8 1.",
                    "3 3.5-.4 7.6-3.9 9.1zM89.3 35.8c-3.6 1.5-7.8-.2-9.2-3.8-",
                    "1.4-3.6.2-7.7 3.9-9.1 3.6-1.5 7.7.3 9.2 3.8 1.4 3.6-.3 7",
                    ".7-3.9 9.1zM69.7 17.7l8.9 4.7V9.3l-8.9 2.8c-.2-.3-.5-.6-",
                    ".9-.9L72.4 0H59.6l3.5 11.2c-.3.3-.6.5-.9.9l-8.8-2.8v13.1",
                    "l8.8-4.7c.3.3.6.7.9.9l-5 15.4v.1c-.2.8-.4 1.6-.4 2.4 0 4",
                    ".1 3.1 7.5 7 8.1h.2c.3 0 .7.1 1 .1.4 0 .7 0 1-.1h.2c4-.6",
                    " 7.1-4.1 7.1-8.1 0-.8-.1-1.7-.4-2.4V34l-5.1-15.4c.4-.2.7",
                    "-.6 1-.9zM66 92.8c16.9 0 32.8 1.1 47.1 3.2 4-16.9 8.9-26",
                    ".7 14-33.5l-9.6-3.4c1 4.9 1.1 7.2 0 10.2-1.5-1.4-3-4.3-4",
                    ".2-8.7L108.6 76c2.8-2 5-3.2 7.5-3.3-4.4 9.4-10 11.9-13.6",
                    " 11.2-4.3-.8-6.3-4.6-5.6-7.9 1-4.7 5.7-5.9 8-.5 4.3-8.7-",
                    "3-11.4-7.6-8.8 7.1-7.2 7.9-13.5 2.1-21.1-8 6.1-8.1 12.3-",
                    "4.5 20.8-4.7-5.4-12.1-2.5-9.5 6.2 3.4-5.2 7.9-2 7.2 3.1-",
                    ".6 4.3-6.4 7.8-13.5 7.2-10.3-.9-10.9-8-11.2-13.8 2.5-.5 ",
                    "7.1 1.8 11 7.3L80.2 60c-4.1 4.4-8 5.3-12.3 5.4 1.4-4.4 8",
                    "-11.6 8-11.6H55.5s6.4 7.2 7.9 11.6c-4.2-.1-8-1-12.3-5.4l",
                    "1.4 16.4c3.9-5.5 8.5-7.7 10.9-7.3-.3 5.8-.9 12.8-11.1 13",
                    ".8-7.2.6-12.9-2.9-13.5-7.2-.7-5 3.8-8.3 7.1-3.1 2.7-8.7-",
                    "4.6-11.6-9.4-6.2 3.7-8.5 3.6-14.7-4.6-20.8-5.8 7.6-5 13.",
                    "9 2.2 21.1-4.7-2.6-11.9.1-7.7 8.8 2.3-5.5 7.1-4.2 8.1.5.",
                    "7 3.3-1.3 7.1-5.7 7.9-3.5.7-9-1.8-13.5-11.2 2.5.1 4.7 1.",
                    "3 7.5 3.3l-4.7-15.4c-1.2 4.4-2.7 7.2-4.3 8.7-1.1-3-.9-5.",
                    "3 0-10.2l-9.5 3.4c5 6.9 9.9 16.7 14 33.5 14.8-2.1 30.8-3",
                    ".2 47.7-3.2z"
                  )
                )
              )
            } else if (!is.null(logo)) {
              shiny::tags$img(
                src = logo,
                class = "govuk-header__logotype-crown-fallback-image",
                alt = logo_alt_text
              )
            },
            shiny::tags$span(org_name, class = "govuk-header__product-name")
          )
        )
      ),
      if (!is.null(service_name)) {
        shiny::tags$div(
          class = "govuk-header__content",
          shiny::tags$span(service_name, class = "govuk-header__service-name")
        )
      }
    )
  )

  header_css <- htmltools::htmlDependency(
    name = "header",
    version = as.character(utils::packageVersion("shinyGovstyle")[[1]]),
    src = c(href = "shinyGovstyle/css"),
    stylesheet = "header-overrides.css"
  )

  htmltools::attachDependencies(
    attachDependency(gov_header), # nolint
    header_css,
    append = TRUE
  )
}

#' Notification Banner Function
#'
#' This function creates a notification banner.
#' @param inputId The input Id for the banner
#' @param title_txt The wording that appears in the title
#' @param body_txt The wording that appears in the banner body
#' @param type The type of banner. Options are standard and success.
#' Standard is default
#' @return a notification HTML shiny tag object
#' @family Govstyle feedback types
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::noti_banner(
#'     inputId = "banner", title_txt = "Important", body_txt = "Example text"
#'   )
#' )
#'
#' server <- function(input, output, session) {}
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
noti_banner <- function(
  inputId, # nolint
  title_txt = "Important",
  body_txt = NULL,
  type = "standard"
) {
  if (type == "success") {
    type_class <- "govuk-notification-banner govuk-notification-banner--success"
    role_type <- "alert"
  } else {
    type_class <- "govuk-notification-banner"
    role_type <- "region"
  }

  gov_noti_banner <- shiny::tags$div(
    class = type_class,
    role = role_type,
    `aria-labelledby` = inputId,
    `data-module` = "govuk-notification-banner",
    shiny::tags$div(
      class = "govuk-notification-banner__header",
      shiny::tags$h2(
        class = "govuk-notification-banner__title",
        id = inputId,
        title_txt
      )
    ),
    shiny::tags$div(
      class = "govuk-notification-banner__content",
      shiny::tags$p(
        class = "govuk-notification-banner__heading",
        shiny::HTML(body_txt)
      )
    )
  )
  attachDependency(gov_noti_banner) # nolint
}

test_that("noti banner works", {
  noti_check <- shinyGovstyle::noti_banner(
    inputId = "banner",
    title_txt = "Important",
    body_txt = "Example text"
  )

  expect_identical(
    noti_check$children[[1]]$children[[1]]$children[[1]],
    "Important"
  )

  expect_identical(
    noti_check$children[[2]]$children[[1]]$children[[1]],
    shiny::HTML("Example text")
  )
})

test_that("noti success banner works", {
  noti_check <- shinyGovstyle::noti_banner(
    inputId = "banner",
    title_txt = "Important",
    body_txt = "Example text",
    type = "success"
  )

  expect_identical(
    noti_check$children[[1]]$children[[1]]$children[[1]],
    "Important"
  )

  expect_identical(
    noti_check$children[[2]]$children[[1]]$children[[1]],
    shiny::HTML("Example text")
  )
})

#' Interactive govTable
#'
#' This function inserts a government-styled table using `reactable`.
#' You can use this in R markdown or Quarto documents, or use
#' renderGovReactable({}) and govReactableOutput() for tables in R Shiny.
#' govReactableOutput() gives the ability to add a caption, for static
#' tables made using just govReactable(), use heading_text() to add
#' captions to tables.
#'
#' @description
#' This function is opinionated and sets table defaults that are in
#' keeping with the wider GOV.UK design system. Some defaults are overrideable,
#' such as `highlight=TRUE` and `borderless=TRUE`, however some are fixed, such
#' as `showSortIcon=FALSE` as the default sort icon is inaccessible. Additional
#' arguments from `reactable::reactable` can be passed to customise the table.
#'
#' @param df A dataframe used to generate the table
#' @param right_col A vector of column names that should be right-aligned.
#' By default, numeric data is right-aligned, and character data is
#' left-aligned
#' @param page_size The default number of rows displayed per page (default: 10)
#' @param highlight Highlight table rows on hover
#' @param borderless Remove inner borders from table
#' @param min_widths Customise minimum column width using a list of columns and
#' minimum width in pixels
#' @param ... Additional arguments passed to `reactable::reactable`
#' @return A `reactable` HTML widget styled with GOV.UK classes
#' @family Govstyle tables tabs and accordions
#' @export
#' @examples
#' # Example static table using govReactable
#' if (interactive()) {
#'   govReactable(
#'     iris,
#'     right_col = c(
#'       "Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width"
#'     )
#'   )
#'
#'   govReactable(
#'     iris,
#'     right_col = c(
#'       "Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width"
#'     ),
#'     highlight = FALSE,
#'     page_size = 5,
#'     min_widths = list(
#'       Sepal.Length = 75,
#'       Sepal.Width = 75,
#'       Petal.Length = 75,
#'       Petal.Width = 75
#'     )
#'   )
#' }
govReactable <- # nolint
  function(
    df,
    right_col = NULL,
    page_size = 10,
    highlight = TRUE,
    borderless = TRUE,
    min_widths = list(),
    ...
  ) {
    # Generate column definitions
    col_defs <- stats::setNames(
      lapply(seq_along(names(df)), function(index) {
        col <- names(df)[index]

        reactable::colDef(
          name = col,
          sortable = TRUE,
          headerClass = "bar-sort-header",
          html = TRUE,
          na = "NA",
          align = if (!is.null(right_col) && col %in% right_col) {
            "right"
          } else {
            "left"
          },
          minWidth = if (!is.null(min_widths[[col]])) {
            min_widths[[col]]
          } else {
            NULL
          }
        )
      }),
      names(df)
    )

    # Create the reactable table
    table <- reactable::reactable(
      df,
      columns = col_defs,
      defaultPageSize = page_size,
      highlight = highlight,
      borderless = borderless,
      showSortIcon = FALSE,
      fullWidth = TRUE,
      wrap = TRUE,
      class = "gov-table govuk-table",
      ...
    )
    attachDependency(table)
  }

#' Shiny bindings for govReactable
#' Output and render functions for using govReactable within shiny apps
#'
#' @param output_table_name Output variable to read from
#' @param caption Adds a caption to the table as a header
#' @param caption_size Adjust the size of caption
#' Options are s, m, l, xl, with l as the default
#' @param heading_level The HTML heading level for
#' the caption (e.g., "h2", "h3", "h4", "h5"). Default is "h2"
#' @param expr An expression that generates a `reactable` widget
#' @param env The environment in which to evaluate `expr`
#' @param quoted Is `expr` a quoted expression (with [quote()])?
#' This is useful if you want to save an expression in a variable
#' @return `govReactableOutput()` returns a `reactable` output element
#' that can be included in a Shiny UI
#'
#' `renderGovReactable()` returns a `reactable` render function that
#' can be assigned to a Shiny output slot
#'
#' @name govReactable-shiny
#' @family Govstyle tables tabs and accordions
#' @examples
#' ui <- shiny::fluidPage(
#'   govReactableOutput(
#'     "table",
#'     caption = "Example table"
#'   )
#' )
#'
#' server <- function(input, output, session) {
#'   output$table <- renderGovReactable({
#'     govReactable(iris)
#'   })
#' }
#'
#' if (interactive()) shinyApp(ui, server)
#' @export
govReactableOutput <- # nolint
  function(
    output_table_name,
    caption,
    caption_size = "l",
    heading_level = "h2"
  ) {
    # Validate heading_level input
    allowed_levels <- c("h2", "h3", "h4", "h5")
    if (!heading_level %in% allowed_levels) {
      stop(
        "heading_level must be one of: ",
        paste(allowed_levels, collapse = ", ")
      )
    }

    heading_tag <- do.call(
      shiny::tags[[heading_level]],
      list(
        class = paste0("govuk-heading-", caption_size),
        caption
      )
    )

    htmltools::div(
      heading_tag,
      reactable::reactableOutput(output_table_name)
    )
  }

# use renderReactable to render the govTables - naming just for convention
# This function wraps reactable::renderReactable.
#' @rdname govReactable-shiny
#' @export
renderGovReactable <- # nolint
  function(
    expr,
    env = parent.frame(),
    quoted = FALSE
  ) {
    if (!quoted) {
      expr <- substitute(expr)
    }
    reactable::renderReactable(expr, env = env, quoted = TRUE)
  }

#' Banner Function
#'
#' This function create a detail component that you can click for further
#' details.
#' @param inputId The input slot that will be used to access the value
#' @param type Main type of label e.g. alpha or beta. Can be any word
#' @param label text to display
#' @return a banner HTML shiny tag object
#' @family Govstyle page structure
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "Beta", 'This is a new service'
#'   )
#' )
#'
#' server <- function(input, output, session) {}
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
banner <- function(
  inputId, # nolint
  type,
  label
) {
  gov_banner <- shiny::tags$div(
    class = "govuk-phase-banner",
    id = inputId,
    shiny::tags$div(
      class = "govuk-width-container",
      shiny::tags$p(
        class = "govuk-phase-banner__content",
        shiny::tags$strong(
          class = "govuk-tag govuk-phase-banner__content__tag",
          type
        ),
        shiny::tags$span(class = "govuk-phase-banner__text", shiny::HTML(label))
      )
    )
  )
  attachDependency(gov_banner)
}

test_that("banner check", {
  banner_check <- banner("bannerId", "alpha", "Banner test")

  expect_identical(
    banner_check$children[[1]]$children[[1]]$children[[1]]$children[[1]],
    "alpha"
  )

  expect_identical(
    banner_check$children[[1]]$children[[1]]$children[[2]]$children[[1]],
    shiny::HTML("Banner test")
  )
})

#' Styling overrides for to give full width
#'
#' `r lifecycle::badge("experimental")`
#' This is an experimental function that will likely be removed in future
#' releases, as we work through updating the core package styling and
#' adding full width options into the components themselves. This is not
#' well tested and may cause unexpected styling issues when combined with
#' components from other packages, use at your own risk.
#'
#' @return HTML containing CSS styling overrides
#' @family Govstyle styling
#' @export
#' @examples
#' shinyGovstyle::full_width_overrides()
full_width_overrides <- function() {
  shiny::tags$head(
    shiny::tags$style(
      shiny::HTML(
        # Overall overrides
        ".container-fluid { padding: 0; }",
        # Match GOV.UK Frontend's own gutter values: 15px (mobile) and 30px
        # (desktop). Without this, text would start flush against the viewport
        # edge in a full-width layout, which GOV.UK Frontend normally avoids
        # via its max-width container and auto margins.
        ".govuk-width-container { max-width: 100%; padding-left: 15px; }",
        paste0(
          "@media (min-width: 641px) {",
          " .govuk-width-container { padding-left: 30px; } }"
        ),
        ".govuk-grid-row { margin-left: 0; margin-right: 0; }",
        "[class*='govuk-grid-column-'] { padding: 0; }",
        ".govuk-main-wrapper { padding-top: 20px; }",

        # Cookie banner overrides
        ".govuk-button-group { margin-right: 0px; }",

        # Footer overrides
        ".govuk-footer { padding: 2rem; }",
        "html { background-color: #f3f2f1; }",

        # Left content overrides
        ".govuk-contents-box { margin-left: 0; margin-right: 0; }",
        ".govuk-contents-box { padding: 10px; width: fit-content !important; }"
      )
    )
  )
}

#' Error on Function
#'
#' This function turns on the error for the component. Can be used to
#' validate inputs.
#' @param inputId The input id that you to to turn the error on for
#' @param error_message if you want to add an additional error message
#' Defaults to NULL, showing the original designed error message
#' @return no return value.  This toggles on error CSS
#' @family Govstyle errors
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   # Required for error handling function
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", 'This is a new service'
#'   ),
#'   shinyGovstyle::gov_layout(size = "two-thirds",
#'     # Error text box
#'     shinyGovstyle::text_Input(
#'       inputId = "eventId",
#'       label = "Event Name",
#'       error = TRUE),
#'     # Button to trigger error
#'     shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   # Trigger error on blank submit of eventId2
#'   observeEvent(input$submit, {
#'     if (input$eventId != ""){
#'       shinyGovstyle::error_off(inputId = "eventId")
#'     } else {
#'       shinyGovstyle::error_on(
#'         inputId = "eventId",
#'         error_message = "Please complete"
#'       )
#'     }
#'   })
#' }
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
error_on <- function(
  inputId, # nolint
  error_message = NULL
) {
  shinyjs::addClass(paste0(inputId, "div"), "govuk-form-group--error")
  if (!is.null(error_message)) {
    shinyjs::html(paste0(inputId, "error"), error_message)
  }
  shinyjs::show(paste0(inputId, "error"))
  shinyjs::addClass(
    selector = paste0("#", inputId, "div :input"),
    class = "govuk-input--error"
  )
  shinyjs::addClass(
    selector = paste0("#", inputId, "file_div"),
    class = "govuk-input--error"
  )
}

#' Footer Function
#'
#' This function create a gov style footer for your page
#'
#' You can add links in the footer to content either within the dashboard or
#' content external to the dashboard using the links_list argument.
#'
#' Links in the footer should be used sparingly and are usually for supporting
#' information pages such as the accessibility statement, privacy notice,
#' cookies information or link to a statement of voluntary adoption of the
#' statistics code of practice.
#'
#' If adding a link to an internal page, generally you will be controlling a
#' hidden tabset so to the end user it looks like it is a new page.
#'
#' @param full Whenever you want to have blank footer or official gov version.
#' Defaults to `FALSE`
#' @param links A vector of actionLinks to be added to the footer, inputIDs
#' are auto-generated and are the snake case version of the link text, e.g.
#' "Accessibility Statement" will have an inputID of accessibility_statement
#' @return a footer HTML shiny tag object
#' @family Govstyle page structure
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png",
#'     logo_alt_text = "Ministry of Justice logo"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", "This is a new service"
#'   ),
#'   shiny::tags$br(),
#'   shiny::tags$br(),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {}
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
#'
#' # Add links
#' footer(links = c("Accessibility statement", "Cookies"))
#'
#' # Add links, internal and external
#' footer(
#'   links = c(
#'     "Accessibility statement",
#'     "Cookies",
#'     `Government Digital Service` =
#'     "https://www.gov.uk/government/organisations/government-digital-service"
#'   )
#' )
#'
#' # Full app with link controlling a hidden tab and a link to an external page
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png",
#'     logo_alt_text = "Ministry of Justice logo"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", "This is a new service"
#'   ),
#'   shiny::tabsetPanel(
#'     type = "hidden",
#'     id = "tabs",
#'     shiny::tabPanel(
#'       "Main content",
#'       value = "main",
#'       heading_text("Hello world!")
#'     ),
#'     shiny::tabPanel(
#'       "Accessibility statement",
#'       value = "accessibility-panel",
#'       heading_text("Accessibility statement")
#'     ),
#'     shiny::tabPanel(
#'       "Cookies",
#'       value = "cookies-panel",
#'       heading_text("Cookies")
#'     ),
#'   ),
#'   shinyGovstyle::footer(
#'     full = TRUE,
#'     links = c(
#'       `Accessibility statement` = "accessibility_footer_link",
#'       `Cookies` = "cookies_footer_link",
#'       `Government Digital Service` =
#'        paste0(
#'          "https://www.gov.uk/government/",
#'          "organisations/government-digital-service"
#'        )
#'     )
#'   )
#' )
#'
#' server <- function(input, output, session) {
#'   shiny::observeEvent(input$accessibility_footer_link, {
#'     shiny::updateTabsetPanel(
#'       session,
#'       "tabs",
#'       selected = "accessibility-panel"
#'     )
#'   })
#'   shiny::observeEvent(input$cookies_footer_link, {
#'     shiny::updateTabsetPanel(
#'       session,
#'       "tabs",
#'       selected = "cookies-panel"
#'     )
#'   })
#' }
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
footer <- function(full = FALSE, links = NULL) {
  if (is.null(names(links))) {
    link_names <- links
  } else {
    link_names <- names(links)
    link_names[link_names == ""] <- links[link_names == ""]
  }

  # The HTML div to be returned
  gov_footer <- shiny::tags$footer(
    class = "govuk-footer ",
    role = "contentinfo",
    shiny::div(
      class = "govuk-width-container ",
      shiny::div(
        class = "govuk-footer__meta",
        if (full == FALSE) {
          shiny::div(
            class = "govuk-footer__meta-item govuk-footer__meta-item--grow",
            if (!is.null(links)) {
              shiny::div(
                # Set a visually hidden title for accessibility
                shiny::h2(class = "govuk-visually-hidden", "Support links"),
                shiny::tags$ul(
                  class = "govuk-footer__inline-list",

                  # Generate as many links as needed
                  mapply(
                    footer_link,
                    links,
                    link_names,
                    SIMPLIFY = FALSE,
                    USE.NAMES = FALSE
                  )
                )
              )
            }
          )
        } else {
          shiny::tagList(
            shiny::div(
              class = "govuk-footer__meta-item govuk-footer__meta-item--grow",
              if (!is.null(links)) {
                shiny::div(
                  # Set a visually hidden title for accessibility
                  shiny::h2(class = "govuk-visually-hidden", "Support links"),
                  shiny::tags$ul(
                    class = "govuk-footer__inline-list",

                    # Generate as many links as needed
                    mapply(
                      footer_link,
                      links,
                      link_names,
                      SIMPLIFY = FALSE,
                      USE.NAMES = FALSE
                    )
                  )
                )
              },
              shiny::tag(
                "svg",
                list(
                  role = "presentation",
                  focusable = "false",
                  class = "govuk-footer__licence-logo",
                  xmlns = "http://www.w3.org/2000/svg",
                  viewbox = "0 0 483.2 195.7",
                  height = "17",
                  width = "41",
                  shiny::tag(
                    "path",
                    list(
                      fill = "currentColor",
                      d = paste0(
                        "M421.5 142.8V.1l-50.7 32.3v161.1h112.4v-50.7",
                        "zm-122.3-9.6A47.12 47.12 0 0 1 221 97.8c0-26 21",
                        ".1-47.1 47.1-47.1 16.7 0 31.4 8.7 39.7 21.8l42.7",
                        "-27.2A97.63 97.63 0 0 0 268.1 0c-36.5 0-68.3 20.1",
                        "-85.1 49.7A98 98 0 0 0 97.8 0C43.9 0 0 43.9 0 97",
                        ".8s43.9 97.8 97.8 97.8c36.5 0 68.3-20.1 85.1-49.",
                        "7a97.76 97.76 0 0 0 149.6 25.4l19.4 22.2h3v-87.8",
                        "h-80l24.3 27.5zM97.8 145c-26 0-47.1-21.1-47.1-47",
                        ".1s21.1-47.1 47.1-47.1 47.2 21 47.2 47S123.8 145",
                        " 97.8 145"
                      )
                    )
                  )
                )
              ),
              shiny::tags$span(
                class = "govuk-footer__licence-description",
                "All content is available under the",
                shiny::tags$a(
                  class = "govuk-footer__link",
                  href = paste0(
                    "https://www.nationalarchives.gov.uk/doc/",
                    "open-government-licence/version/3/"
                  ),
                  rel = "license",
                  "Open Government Licence v3.0"
                ),
                ", except where otherwise stated"
              )
            ),
            shiny::tags$div(
              class = "govuk-footer__meta-item",
              shiny::tags$a(
                class = "govuk-footer__link govuk-footer__copyright-logo",
                href = paste0(
                  "https://www.nationalarchives.gov.uk/information-management",
                  "/re-using-public-sector-information/uk-government-",
                  "licensing-framework/crown-copyright/"
                ),
                "\u00A9 Crown copyright"
              )
            )
          )
        }
      )
    )
  )
  attachDependency(gov_footer)
}

#' Create a footer link for use in `footer()` function
#'
#' @param link Character string containing either link text or url
#' @param link_name Name of a link where a URL has been provided in link_text
#'
#' @returns HTML tag list item
#'
#' @keywords internal
#' @examples
#' # Internal (i.e. within dashboard) link
#' shinyGovstyle:::footer_link("Cookie statement")
#' # Named internal link
#' shinyGovstyle:::footer_link("cookie_statement", "Cookies")
#' # External link
#' shinyGovstyle:::footer_link(
#'   "https://www.gov.uk/government/organisations/government-digital-service",
#'   "Government Digital Service"
#' )
footer_link <- function(link, link_name = NULL) {
  if (grepl("^http|^www", link)) {
    if (is.null(link_name)) {
      warning("Link name provided is NULL for ", link)
      link_name <- link
    }
    shiny::tags$li(
      class = "govuk-footer__inline-list-item",
      shinyGovstyle::external_link(
        link,
        link_name,
        add_warning = FALSE,
        footer = TRUE
      )
    )
  } else {
    if (is.null(link_name)) {
      link_name <- link
    }
    shiny::tags$li(
      class = "govuk-footer__inline-list-item",
      shiny::actionLink(
        class = "govuk-link govuk-footer__link",
        inputId = tolower(gsub(" ", "_", link)),
        label = link_name
      )
    )
  }
}

test_that("test default footer", {
  footer_check <- footer()

  expect_identical(
    footer_check$attribs$class,
    "govuk-footer "
  )
})

test_that("test default footer", {
  footer_check <- footer(TRUE)

  expect_identical(
    footer_check$attribs$class,
    "govuk-footer "
  )
})

test_that("footer links add correctly", {
  footer_with_links <- footer(
    links = c("Accessibility Statement", "Cookies")
  )

  expect_snapshot(footer_with_links)

  full_footer_with_links <- footer(
    TRUE,
    c("Privacy Notice", "Cookies")
  )

  expect_snapshot(full_footer_with_links)

  full_with_mixed_links <- footer(
    TRUE,
    c(
      `Privacy Notice` = "privacy_notice_link",
      `GitHub repository` = paste0(
        "https://github.com/dfe-analytical-services/shinyGovstyle"
      )
    )
  )

  expect_snapshot(full_with_mixed_links)

  full_with_ext_links <- footer(
    TRUE,
    c(
      `Privacy notice` = paste0(
        "https://github.com/dfe-analytical-services/shinyGovstyle"
      ),
      `GitHub repository` = paste0(
        "https://github.com/dfe-analytical-services/shinyGovstyle"
      )
    )
  )

  expect_snapshot(full_with_ext_links)
})

#' Attach shinyGovstyle dependencies
#'
#' @param tag An object which has (or should have) HTML dependencies
#' @param widget Name of a widget for particular dependencies
#'
#' @noRd
#' @importFrom utils packageVersion
#' @importFrom htmltools htmlDependency attachDependencies findDependencies
attachDependency <- # nolint
  function(tag, widget = NULL) {
    version <- as.character(packageVersion("shinyGovstyle")[[1]])

    dep <- htmltools::htmlDependency(
      name = "stylecss",
      version = version,
      src = c(href = "shinyGovstyle/css"),
      stylesheet = "govuk-frontend-6.1.0.min.css"
    )

    if (!is.null(widget)) {
      if (widget == "radio") {
        dep <- list(
          dep,
          htmltools::htmlDependency(
            name = "radio_button_Input",
            version = version,
            src = c(href = "shinyGovstyle/js"),
            script = "radio_button_input_binding.js"
          )
        )
      } else if (widget == "date") {
        dep <- list(
          dep,
          htmltools::htmlDependency(
            name = "date_Input",
            version = version,
            src = c(href = "shinyGovstyle/js"),
            script = "date_input_binding.js"
          )
        )
      } else if (widget == "accordion") {
        dep <- list(
          dep,
          htmltools::htmlDependency(
            name = "accordion",
            version = version,
            src = c(href = "shinyGovstyle/js"),
            script = "accordion.js"
          )
        )
      } else if (widget == "govTab") {
        dep <- list(
          dep,
          htmltools::htmlDependency(
            name = "govTab",
            version = version,
            src = c(href = "shinyGovstyle/js"),
            script = "govTab.js"
          )
        )
      } else if (widget == "contents_link") {
        dep <- list(
          dep,
          htmltools::htmlDependency(
            name = "contents_link",
            version = version,
            src = c(href = "shinyGovstyle/js"),
            script = "contents_link.js"
          )
        )
      } else if (widget == "service_navigation") {
        dep <- list(
          dep,
          htmltools::htmlDependency(
            name = "service_navigation",
            version = version,
            src = c(href = "shinyGovstyle/js"),
            script = "service_navigation.js"
          )
        )
      }
    }

    htmltools::attachDependencies(tag, dep, append = TRUE)
  }

#' Tabs Function
#'
#' This function creates a tabs based table. It requires a single dataframe
#' with a grouping variable.
#' @param inputId The Id to access the tag
#' @param df A single dataframe with all data. See example for structure
#' @param group_col The column name with the groups to be used as tabs
#' @return a tab table HTML shiny tag object
#' @family Govstyle tables tabs and accordions
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png",
#'     logo_alt_text = "Ministry of Justice logo"
#'   ),
#'   shinyGovstyle::gov_main_layout(
#'     size = "two-thirds",
#'     shinyGovstyle::govTabs("tabs", shinyGovstyle::case_data, "tabs")
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {}
#' if (interactive()) shinyApp(ui = ui, server = server)
govTabs <- # nolint
  function(
    inputId, # nolint
    df,
    group_col
  ) {
    tabs <- unique(df[[group_col]])
    tab_headers <- create_tabs(tabs, inputId)

    # Select the first tab as the selected one
    tab_headers$children[[1]][[1]]$attribs$class <-
      "govuk-tabs__list-item govuk-tabs__list-item--selected"

    main_temp_hold <- NULL
    # This needs to make up the tables on each tab
    for (i in tabs) {
      # filter through tabs and drop the grouping value
      temp_table <- df[df[[group_col]] == i, ]
      temp_table[group_col] <- NULL
      # Create row by row the main bulk of table to insert later
      main_row_store <- NULL
      for (j in seq_len(nrow(temp_table))) {
        temp_row_store <- create_tab_row(temp_table[j, ])
        main_row_store <- shiny::tagList(temp_row_store, main_row_store)
      }

      temp_hold <- create_tab_table(temp_table, main_row_store, i, inputId)
      main_temp_hold <- shiny::tagList(main_temp_hold, temp_hold)
    }

    # Put the lime with coconut and create the final thing
    main_tab_div <- shiny::tags$div(
      class = "govuk-frontend-supported",
      shiny::tags$h2(class = "govuk-tabs__title", "Contents"),
      tab_headers,
      main_temp_hold
    )

    # unhide first tab
    main_tab_div$children[[3]][[1]][[1]][[1]][[2]][[2]]$class <-
      "govuk-tabs__panel"

    attachDependency(main_tab_div, "govTab")
  }

create_tab_table <- function(
  df,
  rows,
  tab,
  inputId # nolint
) {
  shiny::tags$div(
    class = "govuk-tabs__panel govuk-tabs__panel--hidden",
    id = tolower(gsub(" ", "-", tab)),
    name = paste0(tolower(gsub(" ", "-", tab)), "-", inputId, "-table"),
    shiny::tags$h2(class = "govuk-heading-l", tab),
    shiny::tags$table(
      class = "govuk-table",
      shiny::tags$thead(
        class = "govuk-table__head",
        shiny::tags$tr(
          class = "govuk-table__row",
          Map(
            function(x) {
              shiny::tags$th(
                scope = "col",
                class = "govuk-table__header",
                x
              )
            },
            x = colnames(df)
          )
        )
      ),
      shiny::tags$tbody(
        class = "govuk-table__body",
        rows
      )
    )
  )
}

create_tab_row <- function(df_row) {
  rowHTML <- # nolint
    shiny::tags$tr(
      class = "govuk-table__row",
      Map(
        function(x) {
          shiny::tags$td(class = "govuk-table__cell", x)
        },
        x = df_row
      )
    )
}

create_tabs <- function(
  tabs_names,
  inputId # nolint
) {
  shiny::tags$ul(
    class = "govuk-tabs__list",
    id = paste0(inputId, "tab"),
    Map(
      function(x) {
        shiny::tags$li(
          name = paste0(tolower(gsub(" ", "-", x)), "-t"),
          id = paste0(tolower(gsub(" ", "-", x)), "-test"),
          class = "govuk-tabs__list-item",
          shiny::tags$a(
            class = "govuk-tabs__tab",
            name = paste0(tolower(gsub(" ", "-", x)), "-l"),
            id = paste0(tolower(gsub(" ", "-", x)), "-l-test"),
            href = paste0("#", tolower(gsub(" ", "-", x))),
            x
          )
        )
      },
      x = tabs_names
    )
  )
}

#' Error off Function
#'
#' This function turns off the the error o the component, once issues have
#' been sorted.
#' @param inputId The input Id to turn error handling on for
#' @return no return value. This toggles off error CSS
#' @family Govstyle errors
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   # Required for error handling function
#'   shinyjs::useShinyjs(),
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::banner(
#'     inputId = "banner", type = "beta", 'This is a new service'
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     # Error text box
#'     shinyGovstyle::text_Input(
#'       inputId = "eventId",
#'       label = "Event Name",
#'       error = TRUE
#'     ),
#'     # Button to trigger error
#'     shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {
#'   # Trigger error on blank submit of eventId2
#'   observeEvent(input$submit, {
#'     if (input$eventId != "") {
#'       shinyGovstyle::error_off(inputId = "eventId")
#'     } else {
#'       shinyGovstyle::error_on(
#'         inputId = "eventId",
#'         error_message = "Please complete"
#'       )
#'     }
#'   })
#' }
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
error_off <- function(
  inputId # nolint
) {
  shinyjs::removeClass(paste0(inputId, "div"), "govuk-form-group--error")
  shinyjs::hide(paste0(inputId, "error"))
  shinyjs::removeClass(
    selector = paste0("#", inputId, "div :input"),
    class = "govuk-input--error"
  )
  shinyjs::removeClass(
    selector = paste0("#", inputId, "file_div"),
    class = "govuk-input--error"
  )
}

#' Subcontents links function
#' This function is used internally within contents_link to create links
#' to headings within pages.
#' @param subcontents_text_list vector of link text for subcontents
#' @param subcontents_id_list vector of link ids for subcontents. Pass NAs
#' for automatic matching to Id in `heading_text()`
#' @return an ordered list HTML shiny tag object
#' @family Govstyle navigation
#' @keywords internal
#' @noRd
subcontents_links <- function(subcontents_text_list, subcontents_id_list) {
  if (!missing(subcontents_id_list)) {
    # check if custom link_id_list is of equal length to links list
    if (length(subcontents_text_list) != length(subcontents_id_list)) {
      message(
        "`subcontents_id_list` must be equal ",
        "length to `subcontents_text_list`"
      )
    }
  }

  # create sidelink
  create_sidelink <- function(link_text, link_id) {
    # match id created in shinygovstyle::heading_text
    # if custom id not specified
    if (is.na(link_id)) {
      link_id <- clean_heading_text(link_text)
    }

    shiny::tags$li(
      "\u2014 ",
      shiny::tags$a(
        class = "govuk-link--no-visited-state",
        link_text,
        href = stringr::str_c("\u0023", link_id)
      )
    )
  }

  # create <li></li> tags
  list_tags <- purrr::map2(
    subcontents_text_list,
    subcontents_id_list,
    create_sidelink
  )

  # return <ol> output
  shiny::tags$ol(class = "govuk-subcontents", list_tags)
}

#' Contents link function
#'
#' `r lifecycle::badge('deprecated')`
#'
#' This function creates an action link to nav between tabs and optionally
#' link to subcontents headers.
#'
#' @param link_text vector of link text for contents
#' @param input_id contents button Id
#' @param subcontents_text_list vector of link text for subcontents
#' @param subcontents_id_list vector of link Ids for subcontents. If missing
#' automatically matches to Id in `heading_text()`
#' @return an action button HTML shiny tag object
#' @export
#' @family Govstyle navigation
#' @examples
#' ui <- shiny::fluidPage(
#'   gov_row(
#'     # Nav columns
#'     shiny::column(
#'       width = 3,
#'       id = "nav", # DO NOT REMOVE ID
#'
#'       # Contents box
#'       shiny::tags$div(
#'         id = "govuk-contents-box", # DO NOT REMOVE ID
#'         class = "govuk-contents-box", # DO NOT REMOVE CLASS
#'
#'         shiny::tags$h2("Contents"),
#'
#'         # Text types tab
#'         contents_link(
#'           "Text Types",
#'           "text_types_button",
#'           subcontents_text_list = c(
#'             "date_Input",
#'             "text_Input",
#'             "text_area_Input",
#'             "button_Input"
#'           ),
#'           subcontents_id_list = c(NA, NA, NA, "button_input_text_types")
#'         ),
#'
#'         # Tables tabs and accordions tab
#'         contents_link(
#'           "Tables, tabs and accordions",
#'           "tables_tabs_and_accordions_button",
#'           subcontents_text_list = c(
#'             "govTable",
#'             "govTabs",
#'             "accordions",
#'             "button_Input"
#'           ),
#'           subcontents_id_list = c(
#'             NA,
#'             NA,
#'             NA,
#'             "button_input_tables_tabs_accordions"
#'           )
#'         ),
#'
#'         contents_link(
#'           "Cookies",
#'           "cookies_button"
#'         ),
#'       )
#'     ),
#'
#'     shiny::column(
#'       width = 9,
#'       id = "main_col", # DO NOT REMOVE ID
#'
#'       # Set up a nav panel so everything not on single page
#'       shiny::tabsetPanel(
#'         type = "hidden",
#'         id = "tab-container", # DO NOT REMOVE ID
#'
#'         shiny::tabPanel(
#'           "Text Types",
#'           value = "text_types",
#'           gov_layout(
#'             size = "two-thirds",
#'             backlink_Input("back1"),
#'             heading_text("Page 2", size = "l"),
#'             label_hint(
#'               "label2",
#'               paste(
#'                 "These are some examples of the",
#'                 "types of user text inputs that you can use"
#'               )
#'             ),
#'             heading_text("date_Input", size = "s"),
#'             date_Input(
#'               inputId = "date1",
#'               label = "What is your date of birth?",
#'               hint_label = "For example, 31 3 1980"
#'             ),
#'             heading_text("text_Input", size = "s"),
#'             text_Input(
#'               inputId = "txt1",
#'               label = "Event name"
#'             ),
#'             heading_text("text_area_Input", size = "s"),
#'             text_area_Input(
#'               inputId = "text_area1",
#'               label = "Can you provide more detail?",
#'               hint_label = paste(
#'                 "Do not include personal or financial",
#'                 "information, like your National Insurance",
#'                 "number or credit card details."
#'               )
#'             ),
#'             text_area_Input(
#'               inputId = "text_area2",
#'               label = "How are you today?",
#'               hint_label = "Leave blank to trigger error",
#'               error = TRUE,
#'               error_message = "Please do not leave blank",
#'               word_limit = 300
#'             ),
#'             heading_text(
#'               "button_Input",
#'               size = "s",
#'               id = "button_input_text_types"
#'             ),
#'             button_Input("btn2", "Go to next page"),
#'             button_Input(
#'               "btn3",
#'               "Check for errors",
#'               type = "warning"
#'             )
#'           )
#'         ),
#'
#'         shiny::tabPanel(
#'           "Tables, tabs and accordions",
#'           value = "tables_tabs_and_accordions",
#'           gov_layout(
#'             size = "two-thirds",
#'             backlink_Input("back2"),
#'             heading_text("Page 3", size = "l"),
#'             label_hint(
#'               "label3",
#'               paste(
#'                 "These are some examples of using tabs",
#'                 "type tables"
#'               )
#'             ),
#'             heading_text("govTable", size = "s"),
#'             heading_text("govTabs", size = "s"),
#'             heading_text("accordions", size = "s"),
#'             shinyGovstyle::accordion(
#'               "acc1",
#'               c(
#'                 "Writing well for the web",
#'                 "Writing well for specialists",
#'                 "Know your audience",
#'                 "How people read"
#'               ),
#'               c(
#'                 paste(
#'                   "This is the content for Writing well",
#'                   "for the web."
#'                 ),
#'                 paste(
#'                   "This is the content for Writing well",
#'                   "for specialists."
#'                 ),
#'                 paste(
#'                   "This is the content for",
#'                   "Know your audience."
#'                 ),
#'                 "This is the content for How people read."
#'               )
#'             ),
#'
#'             heading_text(
#'               "button_Input",
#'               size = "s",
#'               id = "button_input_tables_tabs_accordions"
#'             ),
#'             button_Input("btn4", "Go to next page"),
#'           )
#'         ),
#'
#'         #################### Create cookie panel #########
#'         shiny::tabPanel(
#'           "Cookies",
#'           value = "panel-cookies",
#'           gov_layout(
#'             size = "two-thirds",
#'             heading_text("Cookie page", size = "l"),
#'             label_hint(
#'               "label-cookies",
#'               "This an example cookie page"
#'             )
#'           )
#'         )
#'       )
#'     )
#'   ), # end of main_col
#'   footer(TRUE)
#' ) # end of gov_row
#'
#' server <- function(input, output, session) {
#'   # Tab nav
#'   shiny::observeEvent(input$back2, {
#'     shiny::updateTabsetPanel(
#'       session,
#'       "tab-container",
#'       selected = "text_types"
#'     )
#'   })
#'
#'   shiny::observeEvent(input$tables_tabs_and_accordions_button, {
#'     shiny::updateTabsetPanel(
#'       session,
#'       "tab-container",
#'       selected = "tables_tabs_and_accordions"
#'     )
#'   })
#'
#'   shiny::observeEvent(input$cookies_button, {
#'     shiny::updateTabsetPanel(
#'       session,
#'       "tab-container",
#'       selected = "panel-cookies"
#'     )
#'   })
#' } # end of server
#'
#' if (interactive()) shiny::shinyApp(ui = ui, server = server)
contents_link <- function(
  link_text,
  input_id,
  subcontents_text_list,
  subcontents_id_list
) {
  lifecycle::deprecate_warn("0.2.0", "contents_link()")

  if (
    missing(subcontents_id_list) &&
      !missing(subcontents_text_list)
  ) {
    subcontents_id_list <- rep(NA, length(subcontents_text_list))
  }

  contents_div <- shiny::tags$div(
    class = "govuk-contents",
    shiny::actionLink(
      class = "govuk-contents__link govuk-link--no-visited-state",
      inputId = input_id,
      label = link_text
    ),

    # add subcontents links if required
    if (!missing(subcontents_text_list)) {
      subcontents_links(subcontents_text_list, subcontents_id_list)
    }
  )

  attachDependency(contents_div, "contents_link")
}

test_that("contents_link", {
  contents_check <- contents_link("Test content link", "test_content_link")

  expect_snapshot(contents_check)
})

test_that("subcontents in contents_link", {
  contents_check <- contents_link(
    "Test content link",
    "test_content_link",
    c("My test", "Subcontents", "Links")
  )

  expect_snapshot(contents_check)
})

test_that("custom subcontents in contents_link", {
  contents_check <- contents_link(
    "Test content link",
    "test_content_link",
    c("My test", "Subcontents", "Links"),
    c(NA, NA, "my_custom_link")
  )

  expect_snapshot(contents_check)
})

#Test custom links
test_that("subcontents_links", {
  subcontents_check <-
    subcontents_links(c("Here are my", "Subcontents Links!"), c(NA, NA))

  expect_identical(
    subcontents_check$children[[1]][[2]]$children[[2]]$attribs$href,
    "#subcontents_links"
  )
})


# Test custom subcontents links
test_that("custom subcontents_links", {
  custom_subcontents_check <-
    subcontents_links(
      c("Here are my", "Subcontents Links!"),
      c(NA, "custom_link")
    )

  expect_identical(
    custom_subcontents_check$children[[1]][[2]]$children[[2]]$attribs$href,
    "#custom_link"
  )
})

#' Details Function
#'
#' This function create a detail component that you can click for further
#' details.
#' @param inputId The input slot that will be used to access the value
#' @param label Main label text
#' @param help_text Additional help information in the component
#' @return a details box HTML shiny tag object
#' @family Govstyle feedback types
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo = "shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     shinyGovstyle::details(
#'       inputId = "help_div",
#'       label = "Help with form",
#'       help_text = "To complete the form you need to fill it in..."
#'     )
#'   ),
#'   shinyGovstyle::footer(full = TRUE)
#' )
#'
#' server <- function(input, output, session) {}
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
details <- function(
  inputId, # nolint
  label,
  help_text
) {
  gov_details <- shiny::tags$details(
    class = "govuk-details",
    id = inputId,
    shiny::tags$summary(
      class = "govuk-details__summary",
      shiny::tags$span(
        class = "govuk-details__summary-text",
        shiny::HTML(label)
      )
    ),
    shiny::tags$div(class = "govuk-details__text", help_text)
  )
  attachDependency(gov_details)
}

test_that("details work", {
  check_details <- details("detailId", "Test Main", "Test Second")

  expect_identical(
    check_details$children[[1]]$children[[1]]$children[[1]],
    shiny::HTML("Test Main")
  )

  expect_identical(
    check_details$children[[2]]$children[[1]],
    "Test Second"
  )
})

#' Page Layout Function
#'
#' This function loads the page layout, This doesn't work as well as
#' the `gov_main_layout` and associated functions. This is being kept for now
#' as a simpler version where grids are not needed.
#' @param inputID ID of the main div. Defaults to "main"
#' @param size Layout of the page. Optional are full, one-half, two-thirds,
#' one-third and one-quarter. Defaults to "full"
#' @param ... include the components of the UI that you want within the
#' main page.
#' @return a HTML shiny layout div
#' @family Govstyle page structure
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::header(
#'     org_name = "Example",
#'     service_name = "User Examples",
#'     logo="shinyGovstyle/images/moj_logo.png"
#'   ),
#'   shinyGovstyle::gov_layout(
#'     size = "full",
#'     shinyGovstyle::panel_output(
#'       inputId = "panel1",
#'       main_text = "Application Complete",
#'       sub_text = paste(
#'         "Thank you for submitting your application.",
#'         "Your reference is xvsiq"
#'       )
#'     ),
#'     shinyGovstyle::footer(full = TRUE)
#'   )
#' )
#'
#' server <- function(input, output, session) {}
#' if (interactive()) shinyApp(ui = ui, server = server)
gov_layout <- function(
  ...,
  inputID = "main", # nolint
  size = "full"
) {
  gov_layout <- shiny::tags$div(
    id = inputID,
    class = "govuk-width-container  govuk-main-wrapper",
    shiny::tags$div(
      id = paste0(inputID, "_sub"),
      class = paste0("govuk-grid-column-", size),
      ...
    )
  )
  attachDependency(gov_layout)
}

test_that("default works", {
  layout_test <- gov_layout()

  expect_identical(
    layout_test$attribs$id,
    "main"
  )

  expect_identical(
    layout_test$attribs$class,
    "govuk-width-container  govuk-main-wrapper"
  )

  expect_identical(
    layout_test$children[[1]]$attribs$class,
    "govuk-grid-column-full"
  )
})

#' Run examples
#'
#' This function runs an example R Shiny app showcasing different parts of the
#' package. Code for the app can be found in the inst/example_app folder in the
#' source code.
#'
#' The app uses the [bslib package](https://rstudio.github.io/bslib/) as is
#' generally recommended for Shiny apps.
#'
#' The app is also deployed at the following URL:
#' https://department-for-education.shinyapps.io/shinygovstyle-example-app/
#'
#' @return runs an R Shiny app with examples in
#' @export
#' @examples
#' if (interactive()) run_example()
run_example <- function() {
  appdir <- system.file(package = "shinyGovstyle", "example_app")
  shiny::runApp(appdir)
}

#' Table Function
#'
#' This function inserts a gov styled table. Format is with header looking
#' rows and columns
#' @param inputId Input Id for the table
#' @param df expects a dataframe to create a table
#' @param caption adds a caption to the table as a header
#' @param caption_size adjust the size of caption. Options are s, m, l, xl,
#' with l as the default
#' @param num_col adds numeric class format to these columns
#' @param width_overwrite change width. Need to include width for every column.
#' Columns must add up to 1.
#' Options are three-quarters, two-thirds, one-half, one-third, one-quarter.
#' Default is `NULL`
#' @return a table HTML shiny tag object
#' @family Govstyle tables tabs and accordions
#' @export
#' @examples
#' ui <- shiny::fluidPage(
#'   shinyGovstyle::gov_layout(
#'     size = "two-thirds",
#'     shinyGovstyle::govTable(
#'       "tab1",
#'       shinyGovstyle::transport_data_small,
#'       "Test",
#'       "l",
#'       num_col = c(2,3),
#'       width_overwrite = c("one-half", "one-quarter", "one-quarter")
#'     )
#'   )
#' )
#'
#' server <- function(input, output, session) {}
#'
#' if (interactive()) shinyApp(ui = ui, server = server)
govTable <- # nolint
  function(
    inputId, # nolint
    df,
    caption,
    caption_size = "l",
    num_col = NULL,
    width_overwrite = NULL
  ) {
    # Create row by row the main bulk of table to insert later
    main_row_store <- NULL
    for (i in seq_len(nrow(df))) {
      temp_row_store <- create_rows(df[i, ], num_col)
      main_row_store <- shiny::tagList(temp_row_store, main_row_store)
    }

    # Create the actual table
    gov_table <- shiny::tags$table(
      id = inputId,
      class = "govuk-table",
      shiny::tags$caption(
        class = paste0(
          "govuk-table__caption govuk-table__caption--",
          caption_size
        ),
        style = "caption-side: top;",
        caption
      ),
      shiny::tags$thead(
        class = "govuk-table__head",
        shiny::tags$tr(
          class = "govuk-table__row",
          Map(
            function(x) {
              shiny::tags$th(scope = "col", class = "govuk-table__header", x)
            },
            x = colnames(df)
          )
        )
      ),
      shiny::tags$tbody(
        class = "govuk-table__body",
        main_row_store
      )
    )

    # Change class of headers to numeric if requested
    for (i in num_col) {
      if (i != 1) {
        gov_table$children[[2]]$children[[1]][[3]][[1]][[i]]$attribs$class <-
          "govuk-table__header govuk-table__header--numeric"
      }
    }

    # Change width of columns if requested
    if (!is.null(width_overwrite)) {
      for (i in seq_along(width_overwrite)) {
        gov_table$children[[2]]$children[[1]][[3]][[1]][[i]]$attribs$class <-
          paste0(
            gov_table$children[[2]]$children[[1]][[3]][[1]][[i]]$attribs$class,
            " govuk-!-width-",
            width_overwrite[i]
          )
      }
    }

    attachDependency(gov_table)
  }

create_rows <- function(df_row, num_col = NULL) {
  row_html <- shiny::tags$tr(
    class = "govuk-table__row",
    shiny::tags$th(scope = "row", class = "govuk-table__header", df_row[1, 1]),
    Map(
      function(x) {
        shiny::tags$td(class = "govuk-table__cell", x)
      },
      df_row[1, -1]
    )
  )
  #Not sure I can think of better way to add numeric class then do it post
  #creating the table rows
  for (i in num_col) {
    if (i != 1) {
      row_html$children[[2]][[i - 1]]$attribs$class <-
        "govuk-table__cell govuk-table__cell--numeric"
    }
  }
  row_html
}

#' Text Area Input Function
#'
#' This function create a text area input.
#' @param inputId The input slot that will be used to access the value
#' @param label Display label for the control, or `NULL` for no label
#' @param hint_label Display hint label for the control, or `NULL` for no
#' hint label
#' @param row_no Size of the text entry box. Defaults to 5
#' @param error Whenever to include error handling. Defaults to `FALSE`
#' @param error_message Message to display on error. Defaults to `NULL`
#' @param word_limit Add a word limit to the display. Defaults to `NULL`
#' @return a text area box HTML shiny tag object
#' @family Govstyle text types
#' @export
#' @examples
#' text_area_Input(
#'   "taId",
#'   "Can you provide more detail?",
#'   paste(
#'     "Do not include personal or financial information, like your",
#'     "National Insurance number or credit card details."
#'   )
#' )
text_area_Input <- # nolint
  function(
    inputId, # nolint
    label,
    hint_label = NULL,
    row_no = 5,
    error = FALSE,
    error_message = NULL,
    word_limit = NULL
  ) {
    gov_textarea <- shiny::tags$div(
      class = "govuk-form-group govuk-character-count",
      id = paste0(inputId, "div"),
      shiny::tags$label(shiny::HTML(label), class = "govuk-label"),
      shiny::tags$div(hint_label, class = "govuk-hint"),
      if (error == TRUE) {
        shinyjs::hidden(
          shiny::tags$p(
            error_message,
            class = "govuk-error-message",
            id = paste0(inputId, "error"),
            shiny::tags$span("Error:", class = "govuk-visually-hidden")
          )
        )
      },
      shiny::tags$textarea(
        id = inputId,
        class = "govuk-textarea",
        rows = row_no
      ),
      if (!is.null(word_limit)) {
        shiny::tags$div(
          class = "govuk-hint govuk-character-count__message",
          shiny::tags$span(
            "You have used"
          ),
          shiny::tags$span(
            id = paste0(inputId, "wc"),
            "0"
          ),
          shiny::tags$span(
            id = paste0(inputId, "wl"),
            paste("of the", word_limit, "allowed")
          )
        )
      }
    )
    attachDependency(gov_textarea)
  }

test_that("App loads and title of app appears as expected", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()

  # skip appdir <- system.file(package = "shinyGovstyle", "example_app")
  # skip expect_no_error(shinytest2::test_app(appdir))
  expect_equal(40 + 2, 42)
})

test_that("file input works", {
  file_check <- file_Input("inputId", "Test")

  expect_identical(
    file_check$children[[1]]$children[[1]],
    "Test"
  )
})

test_that("file input HTML is as expected", {
  local_edition(3)
  expect_snapshot(file_Input(
    "inputId",
    "Test",
    multiple = TRUE,
    accept = c(".xls")
  ))
})

test_that("file input error works", {
  file_check <- file_Input(
    "inputId",
    "Test",
    error = TRUE,
    error_message = "Error test"
  )

  expect_identical(
    file_check$children[[2]]$children[[1]],
    "Error test"
  )

  expect_identical(
    paste(
      file_check$children[[2]]$attribs$class,
      file_check$children[[2]]$attribs[3]$class
    ),
    "govuk-error-message shinyjs-hide"
  )
})

test_that("text area works", {
  text_area_check <- text_area_Input("input1", "Test area")

  expect_equal(
    text_area_check$children[[4]]$attribs$rows,
    5
  )

  expect_identical(
    text_area_check$children[[1]]$children[[1]],
    shiny::HTML("Test area")
  )
})


test_that("text area error works", {
  text_area_check <- text_area_Input(
    "input1",
    "Test area",
    error = TRUE,
    error_message = "Test error",
    row_no = 10
  )

  expect_equal(
    text_area_check$children[[4]]$attribs$rows,
    10
  )

  expect_identical(
    paste(
      text_area_check$children[[3]]$attribs$class,
      text_area_check$children[[3]]$attribs[3]$class
    ),
    "govuk-error-message shinyjs-hide"
  )

  expect_identical(
    text_area_check$children[[3]]$children[[1]],
    "Test error"
  )
})

test_that("text area word works", {
  text_area_check <- text_area_Input("input1", "Test area", word_limit = 300)

  expect_identical(
    paste(
      text_area_check$children[[5]]$children[[1]]$children[[1]],
      text_area_check$children[[5]]$children[[2]]$children[[1]],
      text_area_check$children[[5]]$children[[3]]$children[[1]]
    ),
    "You have used 0 of the 300 allowed"
  )

  expect_identical(
    text_area_check$children[[5]]$attribs$class,
    "govuk-hint govuk-character-count__message"
  )
})

test_that("cookie banner works", {
  cookie_banner_check <- shinyGovstyle::cookieBanner("The best thing")

  cookie_check_child <- cookie_banner_check$children[[1]]

  expect_equal(
    length(cookie_check_child$children[[2]]),
    3
  )

  expect_identical(
    cookie_check_child$children[[1]]$children[[1]]$children[[1]]$children[[1]],
    "Cookies on The best thing"
  )

  expect_identical(
    cookie_banner_check$children[[2]]$attribs[3]$class,
    "shinyjs-hide"
  )

  expect_identical(
    cookie_banner_check$children[[3]]$attribs[3]$class,
    "shinyjs-hide"
  )
})

test_that("tabs works", {
  tab_check <- govTabs("tabsID", shinyGovstyle::case_data, "tabs")

  expect_equal(length(tab_check$children[[2]]$children[[1]]), 4)

  expect_identical(
    tab_check$children[[3]][[1]][[1]][[1]][[2]][[2]]$class,
    "govuk-tabs__panel"
  )

  expect_identical(
    tab_check$children[[3]][[1]][[1]][[2]][[2]]$class,
    "govuk-tabs__panel govuk-tabs__panel--hidden"
  )
})

test_that("table works", {
  # test table with specified widths
  table_check <- govTable(
    "tab1",
    shinyGovstyle::transport_data_small,
    "Test",
    "l",
    num_col = c(2, 3),
    width_overwrite = c("one-half", "one-quarter", "one-quarter")
  )

  expect_identical(
    table_check$children[[2]]$children[[1]][[3]][[1]][[1]]$attribs$class,
    "govuk-table__header govuk-!-width-one-half"
  )

  expect_identical(
    table_check$children[[2]]$children[[1]][[3]][[1]][[2]]$attribs$class,
    paste(
      "govuk-table__header govuk-table__header--numeric",
      "govuk-!-width-one-quarter"
    )
  )

  expect_equal(
    length(table_check$children[[3]]),
    3
  )

  # test table with unspecified widths
  table_check2 <- govTable(
    "tab2",
    shinyGovstyle::transport_data_small,
    "Test",
    "l",
    num_col = c(2, 3),
    width_overwrite = NULL
  )

  expect_identical(
    table_check2$children[[2]]$children[[1]][[3]][[1]][[1]]$attribs$class,
    "govuk-table__header"
  )

  expect_identical(
    table_check2$children[[2]]$children[[1]][[3]][[1]][[2]]$attribs$class,
    "govuk-table__header govuk-table__header--numeric"
  )

  expect_equal(
    length(table_check2$children[[3]]),
    3
  )

  # and if the argument isn't mentioned at all
  table_check3 <- govTable(
    "tab2",
    shinyGovstyle::transport_data_small,
    "Test",
    "l",
    num_col = c(2, 3)
  )

  expect_identical(
    table_check3$children[[2]]$children[[1]][[3]][[1]][[1]]$attribs$class,
    "govuk-table__header"
  )

  expect_identical(
    table_check3$children[[2]]$children[[1]][[3]][[1]][[2]]$attribs$class,
    "govuk-table__header govuk-table__header--numeric"
  )

  expect_equal(
    length(table_check3$children[[3]]),
    3
  )
})

test_that("Default", {
  choices <- c("A", "B", "C")
  cbtag <- checkbox_Input(
    inputId = "Id029",
    label = "Label",
    cb_labels = choices,
    checkboxIds = choices
  )
  choicestag <- cbtag$children[[1]]$children[[1]]$children[[4]]$children[[1]]
  expect_length(choicestag, length(choices))

  checked <- lapply(
    choicestag,
    function(x) grepl(pattern = "checked", x = as.character(x))
  )
  checked <- unlist(checked)
  expect_true(all(!checked))
})


test_that("Error", {
  choices <- c("A", "B", "C")
  cbtag <- checkbox_Input(
    inputId = "Id029",
    label = "Label",
    cb_labels = choices,
    checkboxIds = choices,
    error = TRUE,
    error_message = "Error Test"
  )
  choicestag <- cbtag$children[[1]]$children[[1]]$children[[4]]$children[[1]]
  expect_length(choicestag, length(choices))

  checked <- lapply(
    choicestag,
    function(x) grepl(pattern = "checked", x = as.character(x))
  )
  checked <- unlist(checked)
  expect_true(all(!checked))

  err_msg <- cbtag$children[[1]]$children[[1]]$children[[3]]$children[[1]]
  expect_identical(err_msg, "Error Test")

  err_class <- paste(
    cbtag$children[[1]]$children[[1]]$children[[3]]$attribs[1]$class,
    cbtag$children[[1]]$children[[1]]$children[[3]]$attribs[3]$class
  )
  expect_identical(err_class, "govuk-error-message shinyjs-hide")
})


test_that("Small", {
  choices <- c("A", "B", "C")
  cbtag <- checkbox_Input(
    inputId = "Id029",
    label = "Label",
    cb_labels = choices,
    checkboxIds = choices,
    small = TRUE
  )
  choicestag <- cbtag$children[[1]]$children[[1]]$children[[4]]$children[[1]]
  expect_length(choicestag, length(choices))

  checked <- lapply(
    choicestag,
    function(x) grepl(pattern = "checked", x = as.character(x))
  )
  checked <- unlist(checked)
  expect_true(all(!checked))

  small_check <- cbtag$children[[1]]$children[[1]]$children[[4]]$attribs$class
  expect_identical(small_check, "govuk-checkboxes govuk-checkboxes--small")
})

test_that("throws error", {
  expect_error(button_Input("foo", button_type = "bar"))
})

test_that("button works", {
  button <- button_Input("btn1", "Click me")

  expect_identical(
    button$attribs$id,
    "btn1"
  )

  expect_identical(
    button$attribs$class,
    "govuk-button action-button"
  )

  expect_identical(
    "Click me",
    button$children[[1]]
  )
})

test_that("start_button works", {
  button <- button_Input("btn1", "Click me", type = "start")

  expect_identical(
    button$attribs$id,
    "btn1"
  )

  expect_identical(
    button$attribs$class,
    "govuk-button govuk-button--start action-button"
  )

  expect_identical(
    "Click me",
    button$children[[1]]
  )
})

test_that("secondary_button works", {
  button <- button_Input("btn1", "Click me", type = "secondary")

  expect_identical(
    button$attribs$id,
    "btn1"
  )

  expect_identical(
    button$attribs$class,
    "govuk-button govuk-button--secondary action-button"
  )

  expect_identical(
    "Click me",
    button$children[[1]]
  )
})

test_that("warning_button works", {
  button <- button_Input("btn1", "Click me", type = "warning")

  expect_identical(
    button$attribs$id,
    "btn1"
  )

  expect_identical(
    button$attribs$class,
    "govuk-button govuk-button--warning action-button"
  )

  expect_identical(
    "Click me",
    button$children[[1]]
  )
})

test_that("select works", {
  choices <- c("A", "B", "C")
  select_test <- select_Input("slt1", "Select test", choices, choices)

  expect_equal(
    length(select_test$children[[2]]$children[[1]]),
    length(choices)
  )

  expect_identical(
    select_test$children[[2]]$attribs$class,
    "govuk-select"
  )
})

test_that("table works", {
  # Test table with sorting and pagination enabled
  table_check <- govReactable(
    df = shinyGovstyle::transport_data,
    right_col = c("colours", "bikes", "cars", "vans", "buses"),
    page_size = 5
  )

  # Ensure the function runs without errors
  expect_no_error(table_check)

  # Take snapshot of table HTML
  output_html <- htmltools::renderTags(table_check)$html

  # Prevent unnecessary changes due to random IDs
  stripped_ids <- gsub('"htmlwidget-[^"]*"', "", output_html)

  expect_snapshot(stripped_ids)
})

test_that("multiplication works", {
  insert_text <- insert_text("insert1", "Test insert")

  expect_identical(
    insert_text$attribs$id,
    "insert1"
  )

  expect_identical(
    insert_text$attribs$class,
    "govuk-inset-text"
  )

  expect_identical(
    shiny::HTML("Test insert"),
    insert_text$children[[1]]
  )
})

test_that("tag works", {
  tag_check <- shinyGovstyle::tag_Input("tag1", "Complete")
  expect_identical(
    tag_check$children[[1]],
    "Complete"
  )

  expect_identical(
    tag_check$attribs$class,
    "govuk-tag"
  )
})


test_that("tag colour works", {
  tag_check2 <- shinyGovstyle::tag_Input("tag1", "Complete", "yellow")

  expect_identical(
    tag_check2$attribs$class,
    "govuk-tag govuk-tag--yellow"
  )
})


test_that("deprecated colours are warned against", {
  expect_warning(
    tag_Input("tag2", "Complete", "light-blue"),
    paste(
      "'light-blue' is no longer a supported colour.
        Please select an alternative from:
       'navy', 'grey', 'purple', 'teal', 'blue', 'yellow',
        'orange', 'red', 'magenta', or 'green'."
    )
  )
})

test_that("text box works", {
  text_check <- text_Input("txtId", "Text test")

  expect_identical(
    text_check$children[[1]]$children[[1]],
    shiny::HTML("Text test")
  )
})

test_that("text width change", {
  text_check <- text_Input("txtId", "Text test", width = 30)

  expect_identical(
    text_check$children[[4]]$attribs$class,
    "govuk-input govuk-input--width-30"
  )
})


test_that("text box error works", {
  text_check <- text_Input(
    "txtId",
    "Text test",
    error = TRUE,
    error_message = "Error test"
  )

  expect_identical(text_check$children[[3]]$children[[1]], "Error test")

  expect_identical(
    paste(
      text_check$children[[3]]$attribs$class,
      text_check$children[[3]]$attribs[3]$class
    ),
    "govuk-error-message shinyjs-hide"
  )
})

test_that("text box prefix works", {
  text_check <- text_Input("txtId", "Text test", prefix = "£")

  expect_identical(text_check$children[[4]]$children[[1]]$children[[1]], "£")
})

test_that("text box suffix works", {
  text_check <- text_Input("txtId", "Text test", suffix = ".00")

  expect_identical(text_check$children[[4]]$children[[2]]$children[[1]], ".00")
})

test_that("text box prefix suffix works", {
  text_check <- text_Input("txtId", "Text test", prefix = "£", suffix = ".00")

  expect_identical(text_check$children[[4]]$children[[1]]$children[[1]], "£")
  expect_identical(text_check$children[[4]]$children[[3]]$children[[1]], ".00")
})

test_that("date default works", {
  date_check <- date_Input("dateid", "Test Date")

  expect_equal(length(date_check$children[[1]]$children[[4]]), 3)

  expect_identical(
    date_check$children[[1]]$children[[4]]$attribs$class,
    "govuk-date-input"
  )
})

test_that("date error works", {
  date_check <- date_Input(
    "dateid",
    "Test Date",
    error = TRUE,
    error_message = "Error test"
  )

  expect_equal(length(date_check$children[[1]]$children[[4]]), 3)

  expect_identical(
    paste(
      date_check$children[[1]]$children[[2]]$attribs$class,
      date_check$children[[1]]$children[[2]]$attribs[3]$class
    ),
    "govuk-error-message shinyjs-hide"
  )

  expect_identical(
    date_check$children[[1]]$children[[2]]$children[[1]],
    "Error test"
  )
})


test_that("date defaults values works", {
  date_check <- date_Input(
    "dateid",
    "Test Date",
    day = 1,
    month = 2,
    year = 2020
  )

  expect_equal(length(date_check$children[[1]]$children[[4]]), 3)

  date_child <- date_check$children[[1]]$children[[4]]
  expect_equal(
    date_child$children[[1]]$children[[1]]$children[[2]]$attribs$value,
    1
  )

  expect_equal(
    date_child$children[[2]]$children[[1]]$children[[2]]$attribs$value,
    2
  )

  expect_equal(
    date_child$children[[3]]$children[[1]]$children[[2]]$attribs$value,
    2020
  )
})

---
title: "Layout options"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Layout options}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(shinyGovstyle)
```

## Overview

shinyGovstyle provides a set of layout functions that produce the HTML structure GOV.UK Frontend CSS expects. This vignette covers all of the layout functions available and explains how they fit together to build a complete app.

The layout functions fall into two groups:

- **Page-level components** — `header()`, `footer()`, `banner()`, `cookieBanner()`, `skip_to_main()`, and `service_navigation()` — form the frame of the page that sits outside the main content area.
- **Content layout functions** — `gov_main_layout()`, `gov_row()`, `gov_box()`, and `gov_layout()` — structure content within the main content area.

---

## Page-level components

These components form the outer frame of every page. They sit outside the main content area and are consistent across all pages of your app.

```
+-------------------------------------------------------+
|  skip_to_main()   [visually hidden, keyboard only]    |
+-------------------------------------------------------+
|  cookieBanner()   [optional]                          |
+-------------------------------------------------------+
|  header()                                             |
+-------------------------------------------------------+
|  service_navigation()   [optional, multi-page apps]   |
+-------------------------------------------------------+
|  banner()   [optional, e.g. Beta or Alpha]            |
+-------------------------------------------------------+
|                                                       |
|  gov_main_layout()   ← id = "main"                    |
|  +--------------------------------------------------+  |
|  |  your content goes here                         |  |
|  +--------------------------------------------------+  |
|                                                       |
+-------------------------------------------------------+
|  footer()                                             |
+-------------------------------------------------------+
```

### `skip_to_main()`

Provides a visually hidden "Skip to main content" link that becomes visible when focused by a keyboard user. This is an accessibility requirement and should always be the first element in your UI, before the header.

```{r, eval = FALSE}
skip_to_main()
```

By default it links to `#main`, which matches the `id` applied by `gov_main_layout()`. If you change the `inputID` argument of `gov_main_layout()`, pass the same value to `skip_to_main()`.

For more information, read the documentation for the [GOV.UK Skip link component](https://design-system.service.gov.uk/components/skip-link/).

### `cookieBanner()`

Displays a GOV.UK-styled cookie consent banner. It requires `shinyjs::useShinyjs()` to be present in the UI. All element IDs within the banner are preset — see `?cookieBanner` for the server-side `observeEvent` pattern needed to handle accept and reject interactions.

```{r, eval = FALSE}
shinyjs::useShinyjs()
cookieBanner("My service name")
```

For more information, including when this should be used, read the documentation for the [GOV.UK Cookie banner component](https://design-system.service.gov.uk/components/cookie-banner/).

### `header()`

Creates a GOV.UK styled header bar, optionally containing your department logo, name and service name. This is not the official [GOV.UK header](https://design-system.service.gov.uk/components/header/), as that should only be used on GOV.UK domains. If you believe you have an R Shiny app on a GOV.UK domain, please [raise an issue](https://github.com/dfe-analytical-services/shinyGovstyle/issues/new/choose)
to request this as an addition to the package.

```{r, eval = FALSE}
header(
  org_name = "Department for Education",
  service_name = "My dashboard"
)
```

### `banner()`

Displays a phase banner immediately below the header, used to indicate the maturity of your service and give a clear route for users to provide feedback.

```{r, eval = FALSE}
banner(
  inputId = "phase-banner",
  type = "Beta",
  label = paste0(
    "This is a new service \u2014 your ",
    '<a class="govuk-link" href="#">feedback</a> will help us to improve it.'
  )
)
```

For more information on when and how to use this, read the documentation for the [GOV.UK Phase banner component](https://design-system.service.gov.uk/components/phase-banner/). 


### `footer()`

Creates a GOV.UK styled footer, though like the header, this is not an offical version as that should only be used on a GOV.UK domain. Use `full = TRUE` to include the OGL licence logo and Crown copyright statement. You can add support links that point either to internal hidden tab panels or to external URLs.

```{r, eval = FALSE}
# Minimal footer
footer()

# Footer with support links
footer(
  links = c(
    `Accessibility statement` = "accessibility_footer_link",
    `Cookies` = "cookies_footer_link"
  )
)
```

Internal links use auto-generated inputIDs — the link text lowercased with non-alphanumeric characters replaced by underscores — that you handle with `observeEvent()` in your server to switch the active tab panel.

---

## The main content area

`gov_main_layout()` produces a `<div class="govuk-width-container">` wrapping a `<main class="govuk-main-wrapper">`. The outer `<div>` constrains content width; the `<main>` element carries the responsive vertical padding. Everything between the page-level components and the footer lives inside it.

```{r, eval = FALSE}
gov_main_layout(
  # your content here
)
```

The `id` (default `"main"`) is applied directly to the `<main>` element, which is the correct target for `skip_to_main()`. The `<main>` element also carries `role="main"` and `tabindex="-1"`, so keyboard focus moves to it when the skip link is activated.

---

## The primary layout system

Inside `gov_main_layout()`, content is structured using a three-function grid system: `gov_row()`, `gov_box()`, and optionally `gov_text()`.

```
gov_main_layout()
└── gov_row()
    ├── gov_box(size = "two-thirds")
    │   └── [your content]
    └── gov_box(size = "one-third")
        └── [your content]
```

### `gov_row()`

Creates a GOV.UK grid row. You can have multiple rows inside `gov_main_layout()`, each stacked vertically.

```{r, eval = FALSE}
gov_main_layout(
  gov_row(
    # columns go here
  ),
  gov_row(
    # another row
  )
)
```

### `gov_box()`

Creates a column within a row. The `size` argument controls the column width using GOV.UK Frontend's grid classes:

| `size` | Width |
|---|---|
| `"full"` | 100% |
| `"one-half"` | 50% |
| `"two-thirds"` | 66% |
| `"one-third"` | 33% |
| `"three-quarters"` | 75% |
| `"one-quarter"` | 25% |

Sizes within a row should add up to a full width. For example, `"two-thirds"` and `"one-third"` sit side by side:

```{r, eval = FALSE}
gov_main_layout(
  gov_row(
    gov_box(
      size = "two-thirds",
      heading_text("Main content", size = "l"),
      # inputs, text, etc.
    ),
    gov_box(
      size = "one-third",
      heading_text("Sidebar", size = "m"),
      # supporting content
    )
  )
)
```

For a simple single-column layout, use `size = "full"`:

```{r, eval = FALSE}
gov_main_layout(
  gov_row(
    gov_box(
      size = "full",
      heading_text("Page title", size = "l")
    )
  )
)
```

### `gov_text()`

A wrapper that produces a `<p class="govuk-body">` paragraph element. For full guidance on `gov_text()` and all other text functions, see the [Headings and text](headings-and-text.html) vignette.

---

## `gov_layout()` — legacy alternative

> **Warning:** `gov_layout()` is not recommended for new development and may be removed in a future release. Use `gov_main_layout()` with `gov_row()` and `gov_box()` instead.

`gov_layout()` is a single-function alternative that combines a width container and a column in one call:

```{r, eval = FALSE}
gov_layout(
  size = "two-thirds",
  heading_text("Page title", size = "l"),
  # content
)
```

It is well suited to simple, single-column apps where you want a width constraint without setting up the full `gov_main_layout()` / `gov_row()` / `gov_box()` hierarchy.

**As soon as your app needs more than one column, multiple rows, or a combination of widths, switch to the full system.** Nesting `gov_layout()` inside `gov_main_layout()` will produce doubled-up width container HTML and cause the content to appear visually inset from the page-level components.

---

## Multi-page dashboards

For apps with multiple sections, use `service_navigation()` in combination with a hidden tab panel. The navigation bar renders as a row of links below the header; clicking a link fires a Shiny input that you use in your server to switch the visible panel.

### Setting up navigation links

Pass a named character vector to `service_navigation()`. The names are displayed as link text; the values become the inputIDs:

```{r, eval = FALSE}
service_navigation(
  c(
    "Summary" = "nav_summary",
    "Detailed data" = "nav_detail",
    "User guide" = "nav_guide"
  )
)
```

If you pass an unnamed vector, inputIDs are auto-generated by lowercasing the text and replacing non-alphanumeric characters with underscores (e.g. `"Detailed data"` becomes `detailed_data`).

### Wiring navigation to panels

Use a hidden tab panel for the content area and `observeEvent()` in your server to switch panels when a navigation link is clicked. When the user clicks a service navigation link, the JavaScript binding updates the active state automatically — you only need to switch the panel:

```{r, eval = FALSE}
# ui.R — shiny tabsetPanel
shiny::tabsetPanel(
  type = "hidden",
  id = "main_panels",
  shiny::tabPanel("Summary",       value = "nav_summary",  "Content"),
  shiny::tabPanel("Detailed data", value = "nav_detail",   "Content"),
  shiny::tabPanel("User guide",    value = "nav_guide",    "Content")
)

# server.R — nav link click: JS handles the active state, just switch the panel
shiny::observeEvent(input$nav_summary, {
  shiny::updateTabsetPanel(session, "main_panels", selected = "nav_summary")
})
```

If you prefer bslib tab panels, use `bslib::navset_hidden()` and `bslib::nav_select()` instead:

```{r, eval = FALSE}
# ui.R — bslib navset_hidden
bslib::navset_hidden(
  id = "main_panels",
  bslib::nav_panel("Summary",       value = "nav_summary",  "Content"),
  bslib::nav_panel("Detailed data", value = "nav_detail",   "Content"),
  bslib::nav_panel("User guide",    value = "nav_guide",    "Content")
)

# server.R
shiny::observeEvent(input$nav_summary, {
  bslib::nav_select("main_panels", "nav_summary")
})
```

Repeat the `observeEvent` block for each navigation link.

`update_service_navigation()` is only needed when navigation is triggered **programmatically** — for example, via a next / back button — because in that case the nav link itself is not clicked and the active state does not update automatically. See `?update_service_navigation` for full details and examples.

```{r, eval = FALSE}
# server.R — programmatic navigation: must update both the panel and the nav
shiny::observeEvent(input$next_btn, {
  shiny::updateTabsetPanel(session, "main_panels", selected = "nav_detail")
  shinyGovstyle::update_service_navigation(session, "nav_detail")
})
```

### Footer-only pages

Some pages — such as an accessibility statement, privacy notice, or cookies information page — should not appear in the service navigation but still need to be reachable. The standard pattern is to add a link in `footer()` and a corresponding hidden tab panel, but to omit the link from `service_navigation()`.

Because the user navigates to these pages outside of the service navigation, there is no active nav item to highlight. You do not need to call `update_service_navigation()` for these transitions. However, you should call it when navigating *back* to a main page from a footer-linked page, so the correct nav item becomes active again.

```{r, eval = FALSE}
# ui.R — footer link, no entry in service_navigation()
footer(
  full = TRUE,
  links = c(`Accessibility statement` = "accessibility_footer_link")
)

# ui.R — tab panel exists in the hidden tabset but not in service_navigation()
shiny::tabsetPanel(
  type = "hidden",
  id = "main_panels",
  shiny::tabPanel("Summary", value = "nav_summary", "Content"),
  shiny::tabPanel("Accessibility statement", value = "accessibility_panel",
                  "Content")
)

# server.R — navigate to the footer page (no update_service_navigation needed)
shiny::observeEvent(input$accessibility_footer_link, {
  shiny::updateTabsetPanel(session, "main_panels",
                           selected = "accessibility_panel")
})
```

### Modularising the code

Once an app has multiple pages, it is strongly recommended to use Shiny modules to keep each page's UI and server logic self-contained. The `inst/example_app` bundled with this package demonstrates this pattern: each page is a module in `inst/example_app/modules/`, with `mod_<name>_ui()` and `mod_<name>_server()` functions called from the top-level `ui.R` and `server.R`. This keeps individual files focused and makes it straightforward to add or remove pages without touching the overall app structure.

---

## Complete example

The following is a minimal but complete multi-page app that uses all of the layout components covered in this vignette:

```{r, eval = FALSE}
library(shiny)
library(shinyGovstyle)

ui <- bslib::page_fluid(
  skip_to_main(),
  header(
    org_name = "My department",
    service_name = "My dashboard"
  ),
  service_navigation(
    c(
      "Summary"  = "nav_summary",
      "About"    = "nav_about"
    )
  ),
  banner(
    inputId = "phase",
    type = "Beta",
    label = "This is a new service."
  ),

  gov_main_layout(
    shiny::tabsetPanel(
      type = "hidden",
      id = "main_panels",

      shiny::tabPanel(
        "Summary", value = "nav_summary",
        gov_row(
          gov_box(
            size = "two-thirds",
            heading_text("Summary", size = "l"),
            gov_text("Welcome to the summary page.")
          ),
          gov_box(
            size = "one-third",
            heading_text("Quick facts", size = "m"),
            gov_text("Supporting information goes here.")
          )
        )
      ),

      shiny::tabPanel(
        "About", value = "nav_about",
        gov_row(
          gov_box(
            size = "full",
            heading_text("About this dashboard", size = "l"),
            gov_text("This page describes the dashboard.")
          )
        )
      ),

      shiny::tabPanel(
        "Accessibility statement", value = "accessibility_panel",
        gov_row(
          gov_box(
            size = "full",
            heading_text("Accessibility statement", size = "l"),
            gov_text("This page describes the accessibility of the dashboard.")
          )
        )
      )
    )
  ),

  footer(
    links = c(`Accessibility statement` = "accessibility_footer_link")
  )
)

server <- function(input, output, session) {
  shiny::observeEvent(input$nav_summary, {
    shiny::updateTabsetPanel(session, "main_panels", selected = "nav_summary")
  })
  shiny::observeEvent(input$nav_about, {
    shiny::updateTabsetPanel(session, "main_panels", selected = "nav_about")
  })
  shiny::observeEvent(input$accessibility_footer_link, {
    shiny::updateTabsetPanel(session, "main_panels",
                             selected = "accessibility_panel")
  })
}

shiny::shinyApp(ui, server)
```

---
title: "Headings and text"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Headings and text}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(shinyGovstyle)
```

## Overview

This vignette covers all functions in shinyGovstyle that produce styled text content: headings, body text, lists, callout components, links, and typography. For page structure and layout, see the [Layout options](layout-options.html) vignette.

---

## `heading_text()`

Creates a semantic HTML heading element with a GOV.UK heading CSS class.

```{r, eval = FALSE}
heading_text("Summary", size = "l", level = 2)
```

### Arguments

| Argument | Default | Description |
|---|---|---|
| `text_input` | — | The heading text to display |
| `size` | `"xl"` | Visual size: `"xl"`, `"l"`, `"m"`, or `"s"` |
| `level` | `1` | HTML heading level: integer 1–6 |
| `id` | auto | Element ID — auto-generated from `text_input` if omitted |

### Visual size and semantic level are independent

`size` controls the CSS class applied (`govuk-heading-xl`, `govuk-heading-l`, etc.), which determines how the heading looks. `level` controls the HTML element used (`<h1>`, `<h2>`, etc.), which determines its position in the document outline read by screen readers and assistive technologies.

These are independent because it is occasionally necessary to use a smaller visual heading at a higher semantic level, or vice versa. For example, a page where the first visible heading needs to be modest in size but is still structurally the `<h1>`:

```{r, eval = FALSE}
heading_text("User guide", size = "m", level = 1)
```

Always set both arguments deliberately. Do not leave `level` at its default of `1` for every heading on a page.

### Recommended size–level pairings

**Standard pages** (most dashboards and data tools):

| Heading role | `size` | `level` |
|---|---|---|
| Page title | `"l"` | `1` |
| Section heading | `"m"` | `2` |
| Sub-section heading | `"s"` | `3` |

**Long-form content** (user guides, methodology, accessibility statements):

| Heading role | `size` | `level` |
|---|---|---|
| Page title | `"xl"` | `1` |
| Section heading | `"l"` | `2` |
| Sub-section heading | `"m"` | `3` |
| Sub-sub-section heading | `"s"` | `4` |

### Accessibility

**Do not skip heading levels.** Moving from an `<h1>` straight to an `<h3>` breaks the document outline and makes it harder for screen reader users to navigate, as they commonly jump between headings to scan a page. This is a requirement under [WCAG 2.2 success criterion 1.3.1: Info and Relationships](https://www.w3.org/WAI/WCAG22/Understanding/info-and-relationships).

**Write headings in sentence case.** Only capitalise the first word and proper nouns.

```{r, eval = FALSE}
# Correct — sentence case
heading_text("Summary of findings", size = "l", level = 1)

# Incorrect — title case
heading_text("Summary Of Findings", size = "l", level = 1)
```

For more information, read the [GOV.UK headings guidance](https://design-system.service.gov.uk/styles/headings/).

### Heading IDs

If `id` is not supplied it is auto-generated by lowercasing `text_input` and replacing non-alphanumeric characters with underscores. Supply an explicit `id` when you need a stable anchor to link to, or when two headings would otherwise generate the same ID:

```{r, eval = FALSE}
heading_text("Methodology", size = "l", level = 2, id = "methodology")
```

---

## `gov_text()`

A wrapper that produces a `<p class="govuk-body">` paragraph element. Use it when you want to add body text with correct GOV.UK styling without writing raw tag calls.

```{r, eval = FALSE}
gov_box(
  size = "two-thirds",
  gov_text("This is a paragraph of body text.")
)
```

---

## `gov_list()`

Creates a GOV.UK-styled list. The `style` argument controls both the HTML element used and the visual presentation:

| `style` | HTML element | Appearance |
|---|---|---|
| `"none"` (default) | `<ul>` | Plain, no markers |
| `"bullet"` | `<ul>` | Bulleted list |
| `"number"` | `<ol>` | Numbered list |

```{r, eval = FALSE}
gov_list(c("First item", "Second item", "Third item"), style = "bullet")

gov_list(c("First step", "Second step", "Third step"), style = "number")
```

Use `style = "number"` when order matters — steps in a process, ranked results, or sequential instructions. Use `style = "bullet"` for unordered items. The plain style (`"none"`) is useful when you want list semantics for screen readers without a visual marker.

For more information, read the [GOV.UK lists guidance](https://design-system.service.gov.uk/styles/lists/).

---

## `insert_text()`

Displays a GOV.UK inset text box — a bordered callout for supplementary information that is related to, but not the main focus of, the surrounding content.

```{r, eval = FALSE}
insert_text(
  inputId = "processing-note",
  text = "It can take up to 8 weeks to process your application."
)
```

Use inset text for information the user needs to know but that is not the primary action or decision on the page — for example, a processing time, an exception to a rule, or a clarification. Do not use it for warnings about consequences; use `warning_text()` instead.

For more information, read the [GOV.UK inset text guidance](https://design-system.service.gov.uk/components/inset-text/).

---

## `warning_text()`

Displays a GOV.UK warning text component: a bold statement with a prominent "!" icon, used to warn users about something with serious consequences.

```{r, eval = FALSE}
warning_text(
  inputId = "fine-warning",
  text = "You can be fined up to £5,000 if you do not register."
)
```

The "!" icon carries `aria-hidden = "true"` and the word "Warning" is prepended as visually hidden text, so screen readers announce "Warning: [your text]" without reading out the icon character. Given this, you should avoid starting the text with 'Warning', else you'll submit users to 'Warning Warning'.

Use `warning_text()` when the consequence of missing the information is serious. For less critical supplementary information, use `insert_text()` instead.

For more information, read the [GOV.UK warning text guidance](https://design-system.service.gov.uk/components/warning-text/).

---

## `noti_banner()`

Displays a GOV.UK notification banner for information that is not directly related to the current page content — such as a service-wide problem, an upcoming deadline, or the outcome of a previous action.

### Two types

**Standard** (default, blue): for neutral information such as service problems or upcoming events.

```{r, eval = FALSE}
noti_banner(
  inputId = "service-notice",
  title_txt = "Important",
  body_txt = paste0(
    "This service will be unavailable on Saturday ",
    "14 June from 8am to 6pm."
  )
)
```

**Success** (green): to confirm that a previous action completed successfully. Uses `role="alert"` so screen readers announce it automatically on page load.

```{r, eval = FALSE}
noti_banner(
  inputId = "submission-confirm",
  title_txt = "Success",
  body_txt = "Your report has been submitted.",
  type = "success"
)
```

### When to use each component

| Situation | Component |
|---|---|
| Information not related to the current page task | `noti_banner()` |
| Supplementary information related to the page | `insert_text()` |
| Serious consequences if the user misses information | `warning_text()` |
| Form validation errors | `error_summary()` / error messages |

Use notification banners sparingly — users often overlook them when they appear frequently. Show only one at a time, and never alongside an error summary.

For more information, read the [GOV.UK notification banner guidance](https://design-system.service.gov.uk/components/notification-banner/).

---

## `external_link()`

A wrapper for HTML anchor elements that produces safe, accessible external links with consistent behaviour.

```{r, eval = FALSE}
external_link("https://www.example.gov.uk/guidance", "Guidance for applicants")
```

### What the function does automatically

- Adds `target="_blank"` to open the link in a new tab
- Adds `rel="noopener noreferrer"` to prevent [reverse tabnabbing](https://owasp.org/www-community/attacks/Reverse_Tabnabbing)
- Appends "(opens in new tab)" to the visible link text by default
- Adds a visually hidden "(opens in new tab)" span for screen readers when `add_warning = FALSE`

### Descriptive link text

The function validates `link_text` and will error if you supply a raw URL as the link text, vague text such as "click here" or "here", or text ending with a full stop. It will also warn if the text is fewer than 7 characters.

This enforces [WCAG 2.2 success criterion 2.4.4: Link Purpose (In Context)](https://www.w3.org/WAI/WCAG22/Understanding/link-purpose-in-context), which requires link text to describe the destination without needing the surrounding context to make sense of it.

```{r, eval = FALSE}
# Correct — descriptive
external_link("https://www.example.gov.uk/apply", "Apply for a licence")

# Will error — vague text
external_link("https://www.example.gov.uk/apply", "click here")

# Will error — raw URL as text
external_link(
  "https://www.example.gov.uk/apply",
  "https://www.example.gov.uk/apply"
)
```

### Grouped links

When displaying several external links together, repeating "(opens in new tab)" on each is visually repetitive. Set `add_warning = FALSE` and add a single explanatory sentence above the group instead:

```{r, eval = FALSE}
gov_text("The following links open in a new tab.")
shiny::tags$ul(
  shiny::tags$li(
    external_link(
      "https://www.example.gov.uk/a",
      "Guidance document A",
      add_warning = FALSE
    )
  ),
  shiny::tags$li(
    external_link(
      "https://www.example.gov.uk/b",
      "Guidance document B",
      add_warning = FALSE
    )
  )
)
```

For more information, read the [GOV.UK links guidance](https://design-system.service.gov.uk/styles/links/).

---

## `font()`

GDS Transport is a restricted typeface and **must only be used on GOV.UK domains**. If your app is not hosted on a GOV.UK domain, do not call `font()`. You do not need to do anything and your app will default to Arial instead, which is the correct behaviour.

If you are on a GOV.UK domain and therefore want to use the `font()` function, putting it within your UI will load the GDS Transport typeface for use in your app. By default the GOV.UK Frontend CSS specifies `font-family: GDS Transport, arial, sans-serif` — if the font files are not loaded the browser falls back to Arial automatically.

```{r, eval = FALSE}
# Only include this if your app is on a GOV.UK domain
font()
```

For more information on when GDS Transport is permitted, read the [GOV.UK typeface guidance](https://design-system.service.gov.uk/styles/typeface/).

---
title: "Get started"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Get started}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(shinyGovstyle)
```

## Overview
**shinyGovstyle** integrates [GOV.UK Design System](https://design-system.service.gov.uk/) components into R Shiny. It aims to make it easier for government analysts to develop production-ready dashboards that users find intuitive and can trust. 

The [GOV.UK Design System component documentation](https://design-system.service.gov.uk/components/) remains the authoritative reference for all components, including detailed guidance of when, how, and when not(!) to use each of the components.

> **Note:** Using accessible components does not replace the need for comprehensive accessibility testing of the combination you put together in your own specific application.

We welcome feedback and contributions:

- **Suggestions, requests and bugs:** [Raise a GitHub issue](https://github.com/dfe-analytical-services/shinyGovstyle/issues/new) for bugs, requests or suggestions.
- **Search:** Check [existing issues](https://github.com/dfe-analytical-services/shinyGovstyle/issues) before posting.
- **Contribute:** See our [contributing guide](https://github.com/dfe-analytical-services/shinyGovstyle/blob/main/.github/CONTRIBUTING.md) to help improve the package.

---

## See it in action

The quickest way to get a feel for what is available is the example showcase app, which demonstrates the full range of components:

- **Online:** [department-for-education.shinyapps.io/shinygovstyle-example-app](https://department-for-education.shinyapps.io/shinygovstyle-example-app/)
- **Local** (after installing the package): `shinyGovstyle::run_example()`

The source code for the example app lives in [`inst/example_app/`](https://github.com/dfe-analytical-services/shinyGovstyle/tree/main/inst/example_app) and is a useful reference for how components fit together in a real multi-page app using [Shiny modules](https://mastering-shiny.org/scaling-modules.html).

---

## Installation

Install the latest stable release from CRAN:

```{r, eval = FALSE}
install.packages("shinyGovstyle")
```

To use the latest development version directly from GitHub:

```{r, eval = FALSE}
pak::pak("dfe-analytical-services/shinyGovstyle")
```

---

## shinyjs dependency

Some components require [`shinyjs`](https://deanattali.com/shinyjs/) to be loaded in your UI. Add `shinyjs::useShinyjs()` anywhere in your UI when using:

- `error_on()` / `error_off()` — inline field-level error toggling
- `word_count()` — live character and word counting on text inputs
- `cookieBanner()` — GOV.UK cookie consent banner

```{r, eval = FALSE}
ui <- bslib::page_fluid(
  shinyjs::useShinyjs(),
  # rest of your UI
)
```

---

## A minimal working app

The skeleton below shows the components that appear on every page of a shinyGovstyle app. It uses [`bslib`](https://rstudio.github.io/bslib/), which is generally [recommended for building responsive apps with R Shiny](https://shiny.posit.co/r/getstarted/shiny-basics/lesson1/).

```{r, eval = FALSE}
library(shiny)
library(shinyGovstyle)

ui <- bslib::page_fluid(
  skip_to_main(),
  header(
    org_name = "My department",
    service_name = "My dashboard"
  ),
  banner(
    inputId = "phase",
    type = "Beta",
    label = "This is a new service."
  ),
  gov_main_layout(
    gov_row(
      gov_box(
        size = "two-thirds",
        heading_text("Welcome", size = "l", level = 1),
        gov_text("Select a report from the navigation above.")
      )
    )
  ),
  footer()
)

server <- function(input, output, session) {}

shiny::shinyApp(ui, server)
```

For a more complete example - including multi-page navigation, footer links, and the server logic to wire it all together - see the [complete example](layout-options.html#complete-example) at the end of the Layout options vignette.

---

## Component overview

### Page structure and layout

All page-level components (`header()`, `footer()`, `banner()`, `skip_to_main()`, `service_navigation()`, `cookieBanner()`) and the content grid system (`gov_main_layout()`, `gov_row()`, `gov_box()`) are covered in the [Layout options](layout-options.html) vignette.

### Headings and text

Styled headings, body text, lists, callouts, external links, notification banners, and typography are covered in the [Headings and text](headings-and-text.html) vignette.

### Form inputs

`text_input()`, `text_area_input()`, `select_input()`, `checkbox_input()`, `radio_button_Input()`, `date_Input()`, `file_Input()`, `button_Input()`, and `download_button()` provide GOV.UK-styled equivalents of standard Shiny form controls; values are read from `input$<inputId>` in the server in the usual way.

### Interactive components

`accordion()` and `govTabs()` provide collapsible panels and tabbed interfaces with full keyboard navigation and ARIA support built in.

### Error handling

`error_summary()` produces a GOV.UK error summary panel that lists all validation errors at the top of the page; `error_on()` and `error_off()` toggle inline field-level error messages on any input. These require `shinyjs::useShinyjs()` in the UI, see [shinyjs dependency](#shinyjs-dependency) for more details.

### Data display

`govTable()` and `govReactable()` render data tables with GOV.UK styling; `value_box()` displays a headline figure with a label, suited to summary statistics at the top of a dashboard.

# footer links add correctly

    Code
      footer_with_links
    Output
      <footer class="govuk-footer " role="contentinfo">
        <div class="govuk-width-container ">
          <div class="govuk-footer__meta">
            <div class="govuk-footer__meta-item govuk-footer__meta-item--grow">
              <div>
                <h2 class="govuk-visually-hidden">Support links</h2>
                <ul class="govuk-footer__inline-list">
                  <li class="govuk-footer__inline-list-item">
                    <a class="action-button action-link govuk-link govuk-footer__link" href="#" id="accessibility_statement"><span class="action-label">Accessibility Statement</span></a>
                  </li>
                  <li class="govuk-footer__inline-list-item">
                    <a class="action-button action-link govuk-link govuk-footer__link" href="#" id="cookies"><span class="action-label">Cookies</span></a>
                  </li>
                </ul>
              </div>
            </div>
          </div>
        </div>
      </footer>

---

    Code
      full_footer_with_links
    Output
      <footer class="govuk-footer " role="contentinfo">
        <div class="govuk-width-container ">
          <div class="govuk-footer__meta">
            <div class="govuk-footer__meta-item govuk-footer__meta-item--grow">
              <div>
                <h2 class="govuk-visually-hidden">Support links</h2>
                <ul class="govuk-footer__inline-list">
                  <li class="govuk-footer__inline-list-item">
                    <a class="action-button action-link govuk-link govuk-footer__link" href="#" id="privacy_notice"><span class="action-label">Privacy Notice</span></a>
                  </li>
                  <li class="govuk-footer__inline-list-item">
                    <a class="action-button action-link govuk-link govuk-footer__link" href="#" id="cookies"><span class="action-label">Cookies</span></a>
                  </li>
                </ul>
              </div>
              <svg role="presentation" focusable="false" class="govuk-footer__licence-logo" xmlns="http://www.w3.org/2000/svg" viewbox="0 0 483.2 195.7" height="17" width="41">
                <path fill="currentColor" d="M421.5 142.8V.1l-50.7 32.3v161.1h112.4v-50.7zm-122.3-9.6A47.12 47.12 0 0 1 221 97.8c0-26 21.1-47.1 47.1-47.1 16.7 0 31.4 8.7 39.7 21.8l42.7-27.2A97.63 97.63 0 0 0 268.1 0c-36.5 0-68.3 20.1-85.1 49.7A98 98 0 0 0 97.8 0C43.9 0 0 43.9 0 97.8s43.9 97.8 97.8 97.8c36.5 0 68.3-20.1 85.1-49.7a97.76 97.76 0 0 0 149.6 25.4l19.4 22.2h3v-87.8h-80l24.3 27.5zM97.8 145c-26 0-47.1-21.1-47.1-47.1s21.1-47.1 47.1-47.1 47.2 21 47.2 47S123.8 145 97.8 145"></path>
              </svg>
              <span class="govuk-footer__licence-description">
                All content is available under the
                <a class="govuk-footer__link" href="https://www.nationalarchives.gov.uk/doc/open-government-licence/version/3/" rel="license">Open Government Licence v3.0</a>
                , except where otherwise stated
              </span>
            </div>
            <div class="govuk-footer__meta-item">
              <a class="govuk-footer__link govuk-footer__copyright-logo" href="https://www.nationalarchives.gov.uk/information-management/re-using-public-sector-information/uk-government-licensing-framework/crown-copyright/">© Crown copyright</a>
            </div>
          </div>
        </div>
      </footer>

---

    Code
      full_with_mixed_links
    Output
      <footer class="govuk-footer " role="contentinfo">
        <div class="govuk-width-container ">
          <div class="govuk-footer__meta">
            <div class="govuk-footer__meta-item govuk-footer__meta-item--grow">
              <div>
                <h2 class="govuk-visually-hidden">Support links</h2>
                <ul class="govuk-footer__inline-list">
                  <li class="govuk-footer__inline-list-item">
                    <a class="action-button action-link govuk-link govuk-footer__link" href="#" id="privacy_notice_link"><span class="action-label">Privacy Notice</span></a>
                  </li>
                  <li class="govuk-footer__inline-list-item"><a href="https://github.com/dfe-analytical-services/shinyGovstyle" class="govuk-link govuk-footer__link" target="_blank" rel="noopener noreferrer">GitHub repository<span class="sr-only"> (opens in new tab)</span></a></li>
                </ul>
              </div>
              <svg role="presentation" focusable="false" class="govuk-footer__licence-logo" xmlns="http://www.w3.org/2000/svg" viewbox="0 0 483.2 195.7" height="17" width="41">
                <path fill="currentColor" d="M421.5 142.8V.1l-50.7 32.3v161.1h112.4v-50.7zm-122.3-9.6A47.12 47.12 0 0 1 221 97.8c0-26 21.1-47.1 47.1-47.1 16.7 0 31.4 8.7 39.7 21.8l42.7-27.2A97.63 97.63 0 0 0 268.1 0c-36.5 0-68.3 20.1-85.1 49.7A98 98 0 0 0 97.8 0C43.9 0 0 43.9 0 97.8s43.9 97.8 97.8 97.8c36.5 0 68.3-20.1 85.1-49.7a97.76 97.76 0 0 0 149.6 25.4l19.4 22.2h3v-87.8h-80l24.3 27.5zM97.8 145c-26 0-47.1-21.1-47.1-47.1s21.1-47.1 47.1-47.1 47.2 21 47.2 47S123.8 145 97.8 145"></path>
              </svg>
              <span class="govuk-footer__licence-description">
                All content is available under the
                <a class="govuk-footer__link" href="https://www.nationalarchives.gov.uk/doc/open-government-licence/version/3/" rel="license">Open Government Licence v3.0</a>
                , except where otherwise stated
              </span>
            </div>
            <div class="govuk-footer__meta-item">
              <a class="govuk-footer__link govuk-footer__copyright-logo" href="https://www.nationalarchives.gov.uk/information-management/re-using-public-sector-information/uk-government-licensing-framework/crown-copyright/">© Crown copyright</a>
            </div>
          </div>
        </div>
      </footer>

---

    Code
      full_with_ext_links
    Output
      <footer class="govuk-footer " role="contentinfo">
        <div class="govuk-width-container ">
          <div class="govuk-footer__meta">
            <div class="govuk-footer__meta-item govuk-footer__meta-item--grow">
              <div>
                <h2 class="govuk-visually-hidden">Support links</h2>
                <ul class="govuk-footer__inline-list">
                  <li class="govuk-footer__inline-list-item"><a href="https://github.com/dfe-analytical-services/shinyGovstyle" class="govuk-link govuk-footer__link" target="_blank" rel="noopener noreferrer">Privacy notice<span class="sr-only"> (opens in new tab)</span></a></li>
                  <li class="govuk-footer__inline-list-item"><a href="https://github.com/dfe-analytical-services/shinyGovstyle" class="govuk-link govuk-footer__link" target="_blank" rel="noopener noreferrer">GitHub repository<span class="sr-only"> (opens in new tab)</span></a></li>
                </ul>
              </div>
              <svg role="presentation" focusable="false" class="govuk-footer__licence-logo" xmlns="http://www.w3.org/2000/svg" viewbox="0 0 483.2 195.7" height="17" width="41">
                <path fill="currentColor" d="M421.5 142.8V.1l-50.7 32.3v161.1h112.4v-50.7zm-122.3-9.6A47.12 47.12 0 0 1 221 97.8c0-26 21.1-47.1 47.1-47.1 16.7 0 31.4 8.7 39.7 21.8l42.7-27.2A97.63 97.63 0 0 0 268.1 0c-36.5 0-68.3 20.1-85.1 49.7A98 98 0 0 0 97.8 0C43.9 0 0 43.9 0 97.8s43.9 97.8 97.8 97.8c36.5 0 68.3-20.1 85.1-49.7a97.76 97.76 0 0 0 149.6 25.4l19.4 22.2h3v-87.8h-80l24.3 27.5zM97.8 145c-26 0-47.1-21.1-47.1-47.1s21.1-47.1 47.1-47.1 47.2 21 47.2 47S123.8 145 97.8 145"></path>
              </svg>
              <span class="govuk-footer__licence-description">
                All content is available under the
                <a class="govuk-footer__link" href="https://www.nationalarchives.gov.uk/doc/open-government-licence/version/3/" rel="license">Open Government Licence v3.0</a>
                , except where otherwise stated
              </span>
            </div>
            <div class="govuk-footer__meta-item">
              <a class="govuk-footer__link govuk-footer__copyright-logo" href="https://www.nationalarchives.gov.uk/information-management/re-using-public-sector-information/uk-government-licensing-framework/crown-copyright/">© Crown copyright</a>
            </div>
          </div>
        </div>
      </footer>

# content and URL are correctly formatted

    Code
      test_link
    Output
      <a href="https://shiny.posit.co/" class="govuk-link" target="_blank" rel="noopener noreferrer">R Shiny (opens in new tab)</a>

# New tab warning always stays for non-visual users

    Code
      external_link("https://shiny.posit.co/", "R Shiny", add_warning = FALSE)
    Output
      <a href="https://shiny.posit.co/" class="govuk-link" target="_blank" rel="noopener noreferrer">R Shiny<span class="sr-only"> (opens in new tab)</span></a>

# Surrounding whitespace shrubbery is trimmed

    Code
      external_link("https://shiny.posit.co/", "   R Shiny")
    Output
      <a href="https://shiny.posit.co/" class="govuk-link" target="_blank" rel="noopener noreferrer">R Shiny (opens in new tab)</a>

---

    Code
      external_link("https://shiny.posit.co/", "R Shiny      ")
    Output
      <a href="https://shiny.posit.co/" class="govuk-link" target="_blank" rel="noopener noreferrer">R Shiny (opens in new tab)</a>

---

    Code
      external_link("https://shiny.posit.co/", "   R Shiny   ")
    Output
      <a href="https://shiny.posit.co/" class="govuk-link" target="_blank" rel="noopener noreferrer">R Shiny (opens in new tab)</a>

---

    Code
      external_link("https://shiny.posit.co/", "   R Shiny", add_warning = FALSE)
    Output
      <a href="https://shiny.posit.co/" class="govuk-link" target="_blank" rel="noopener noreferrer">R Shiny<span class="sr-only"> (opens in new tab)</span></a>

---

    Code
      external_link("https://shiny.posit.co/", "R Shiny   ", add_warning = FALSE)
    Output
      <a href="https://shiny.posit.co/" class="govuk-link" target="_blank" rel="noopener noreferrer">R Shiny<span class="sr-only"> (opens in new tab)</span></a>

---

    Code
      external_link("https://shiny.posit.co/", "   R Shiny   ", add_warning = FALSE)
    Output
      <a href="https://shiny.posit.co/" class="govuk-link" target="_blank" rel="noopener noreferrer">R Shiny<span class="sr-only"> (opens in new tab)</span></a>

# service_navigation HTML is as expected

    Code
      service_navigation(links = c("Page 1", "Page 2", "Page 3"))
    Output
      <section aria-label="Service information" class="govuk-service-navigation" data-module="govuk-service-navigation">
        <div class="govuk-width-container">
          <div class="govuk-service-navigation__container">
            <nav aria-label="Menu" class="govuk-service-navigation__wrapper">
              <button type="button" class="govuk-service-navigation__toggle govuk-js-service-navigation-toggle" aria-controls="navigation" hidden="TRUE">Menu</button>
              <ul class="govuk-service-navigation__list" id="navigation">
                <li class="govuk-service-navigation__item">
                  <a class="action-button action-link govuk-service-navigation__link" href="#" id="page_1"><span class="action-label">Page 1</span></a>
                </li>
                <li class="govuk-service-navigation__item">
                  <a class="action-button action-link govuk-service-navigation__link" href="#" id="page_2"><span class="action-label">Page 2</span></a>
                </li>
                <li class="govuk-service-navigation__item">
                  <a class="action-button action-link govuk-service-navigation__link" href="#" id="page_3"><span class="action-label">Page 3</span></a>
                </li>
              </ul>
            </nav>
          </div>
        </div>
      </section>

---

    Code
      service_navigation(service_name = "My app", c(`Page 1` = "p1", `Page 2` = "p2",
        `Page 3` = "p3"))
    Output
      <section aria-label="Service information" class="govuk-service-navigation" data-module="govuk-service-navigation">
        <div class="govuk-width-container">
          <div class="govuk-service-navigation__container">
            <span class="govuk-service-navigation__service-name">
              <a href="#" class="govuk-service-navigation__link">My app</a>
            </span>
            <nav aria-label="Menu" class="govuk-service-navigation__wrapper">
              <button type="button" class="govuk-service-navigation__toggle govuk-js-service-navigation-toggle" aria-controls="navigation" hidden="TRUE">Menu</button>
              <ul class="govuk-service-navigation__list" id="navigation">
                <li class="govuk-service-navigation__item">
                  <a class="action-button action-link govuk-service-navigation__link" href="#" id="p1"><span class="action-label">Page 1</span></a>
                </li>
                <li class="govuk-service-navigation__item">
                  <a class="action-button action-link govuk-service-navigation__link" href="#" id="p2"><span class="action-label">Page 2</span></a>
                </li>
                <li class="govuk-service-navigation__item">
                  <a class="action-button action-link govuk-service-navigation__link" href="#" id="p3"><span class="action-label">Page 3</span></a>
                </li>
              </ul>
            </nav>
          </div>
        </div>
      </section>

# service_navigation works with a single link

    Code
      service_navigation(links = "Solo page")
    Output
      <section aria-label="Service information" class="govuk-service-navigation" data-module="govuk-service-navigation">
        <div class="govuk-width-container">
          <div class="govuk-service-navigation__container">
            <nav aria-label="Menu" class="govuk-service-navigation__wrapper">
              <button type="button" class="govuk-service-navigation__toggle govuk-js-service-navigation-toggle" aria-controls="navigation" hidden="TRUE">Menu</button>
              <ul class="govuk-service-navigation__list" id="navigation">
                <li class="govuk-service-navigation__item">
                  <a class="action-button action-link govuk-service-navigation__link" href="#" id="solo_page"><span class="action-label">Solo page</span></a>
                </li>
              </ul>
            </nav>
          </div>
        </div>
      </section>

# error_summary works

    Code
      error_summary
    Output
      <div id="error1" class="govuk-error-summary">
        <h2 class="govuk-error-summary__title">Error Title</h2>
        <div id="error1list" class="govuk-error-summary__body">
          <ul class="govuk-list govuk-error-summary__list">
            <li>error entry 1</li>
            <li>error entry 2</li>
          </ul>
        </div>
      </div>

# file input HTML is as expected

    Code
      file_Input("inputId", "Test", multiple = TRUE, accept = c(".xls"))
    Output
      <div id="inputIddiv" class="govuk-form-group">
        <label class="govuk-label" tabindex="-1">Test</label>
        <div id="inputIdfile_div" class="input-group govuk-file-upload" style="display: flex; align-items: center;">
          <label class="govuk-button govuk-button--secondary govuk-file-upload-button__pseudo-button" style="margin-bottom: 0; position: relative; overflow: hidden;">
            Choose file
            <input id="inputId" name="inputId" type="file" style="position: absolute; width: 1px; height: 1px; padding: 0; margin: -1px; overflow: hidden; clip: rect(0,0,0,0); border: 0;" multiple="multiple" accept=".xls"/>
          </label>
          <input type="text" class="govuk-body" style="margin: 0; border: 0; outline: none; width: 98%; flex: 1 1 auto;" placeholder="No file chosen" readonly="readonly" tabindex="-1"/>
        </div>
      </div>

# backlink HTML is as expected

    Code
      backlink_Input("backId")
    Output
      <a class="action-button action-link govuk-back-link" href="#" id="backId"><span class="action-label">Back</span></a>

# table works

    Code
      stripped_ids
    Output
      <div class="reactable html-widget html-fill-item" id= style="width:auto;height:auto;"></div>
      <script type="application/json" data-for=>{"x":{"tag":{"name":"Reactable","attribs":{"data":{"months":["January","February","March","April","May","January","February","March","April","May"],"colours":["Red","Blue","Red","Blue","Red","Blue","Red","Blue","Red","Blue"],"bikes":[85,75,165,90,80,95,85,175,100,95],"vans":[150,130,180,160,140,175,135,185,155,145],"buses":[200,180,220,210,190,215,185,225,205,195]},"columns":[{"id":"months","name":"months","type":"character","sortable":true,"html":true,"na":"NA","align":"left","headerClassName":"bar-sort-header"},{"id":"colours","name":"colours","type":"character","sortable":true,"html":true,"na":"NA","align":"right","headerClassName":"bar-sort-header"},{"id":"bikes","name":"bikes","type":"numeric","sortable":true,"html":true,"na":"NA","align":"right","headerClassName":"bar-sort-header"},{"id":"vans","name":"vans","type":"numeric","sortable":true,"html":true,"na":"NA","align":"right","headerClassName":"bar-sort-header"},{"id":"buses","name":"buses","type":"numeric","sortable":true,"html":true,"na":"NA","align":"right","headerClassName":"bar-sort-header"}],"defaultPageSize":5,"highlight":true,"borderless":true,"showSortIcon":false,"className":"gov-table govuk-table","dataKey":"eccc8e8cf6736e3758d28b3971dbaeed"},"children":[]},"class":"reactR_markup"},"evals":[],"jsHooks":[]}</script>

# contents_link

    Code
      contents_check
    Output
      <div class="govuk-contents">
        <a class="action-button action-link govuk-contents__link govuk-link--no-visited-state" href="#" id="test_content_link"><span class="action-label">Test content link</span></a>
      </div>

# subcontents in contents_link

    Code
      contents_check
    Output
      <div class="govuk-contents">
        <a class="action-button action-link govuk-contents__link govuk-link--no-visited-state" href="#" id="test_content_link"><span class="action-label">Test content link</span></a>
        <ol class="govuk-subcontents">
          <li>
            — 
            <a class="govuk-link--no-visited-state" href="#my_test">My test</a>
          </li>
          <li>
            — 
            <a class="govuk-link--no-visited-state" href="#subcontents">Subcontents</a>
          </li>
          <li>
            — 
            <a class="govuk-link--no-visited-state" href="#links">Links</a>
          </li>
        </ol>
      </div>

# custom subcontents in contents_link

    Code
      contents_check
    Output
      <div class="govuk-contents">
        <a class="action-button action-link govuk-contents__link govuk-link--no-visited-state" href="#" id="test_content_link"><span class="action-label">Test content link</span></a>
        <ol class="govuk-subcontents">
          <li>
            — 
            <a class="govuk-link--no-visited-state" href="#my_test">My test</a>
          </li>
          <li>
            — 
            <a class="govuk-link--no-visited-state" href="#subcontents">Subcontents</a>
          </li>
          <li>
            — 
            <a class="govuk-link--no-visited-state" href="#my_custom_link">Links</a>
          </li>
        </ol>
      </div>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accordion.R
\name{accordion}
\alias{accordion}
\title{Accordion Function}
\usage{
accordion(inputId, titles, descriptions)
}
\arguments{
\item{inputId}{Input Id for the accordion}

\item{titles}{Add the titles for the accordion}

\item{descriptions}{Add the main text for the accordion}
}
\value{
an accordion HTML shiny tag object
}
\description{
This function inserts a accordion
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png",
    logo_alt_text = "Ministry of Justice logo"
  ),
  shinyGovstyle::banner(
    inputId = "banner", type = "beta", 'This is a new service'
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    accordion(
      "acc1",
      c(
        "Writing well for the web",
        "Writing well for specialists",
        "Know your audience",
        "How people read"
      ),
      c(
        "This is the content for Writing well for the web.",
        "This is the content for Writing well for specialists.",
        "This is the content for Know your audience.",
        "This is the content for How people read."
      )
    )
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {}

if (interactive()) shiny::shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle tables tabs and accordions: 
\code{\link{govReactable}()},
\code{\link{govReactable-shiny}},
\code{\link{govTable}()},
\code{\link{govTabs}()}
}
\concept{Govstyle tables tabs and accordions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/footer.R
\name{footer}
\alias{footer}
\title{Footer Function}
\usage{
footer(full = FALSE, links = NULL)
}
\arguments{
\item{full}{Whenever you want to have blank footer or official gov version.
Defaults to \code{FALSE}}

\item{links}{A vector of actionLinks to be added to the footer, inputIDs
are auto-generated and are the snake case version of the link text, e.g.
"Accessibility Statement" will have an inputID of accessibility_statement}
}
\value{
a footer HTML shiny tag object
}
\description{
This function create a gov style footer for your page
}
\details{
You can add links in the footer to content either within the dashboard or
content external to the dashboard using the links_list argument.

Links in the footer should be used sparingly and are usually for supporting
information pages such as the accessibility statement, privacy notice,
cookies information or link to a statement of voluntary adoption of the
statistics code of practice.

If adding a link to an internal page, generally you will be controlling a
hidden tabset so to the end user it looks like it is a new page.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png",
    logo_alt_text = "Ministry of Justice logo"
  ),
  shinyGovstyle::banner(
    inputId = "banner", type = "beta", "This is a new service"
  ),
  shiny::tags$br(),
  shiny::tags$br(),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {}

if (interactive()) shinyApp(ui = ui, server = server)

# Add links
footer(links = c("Accessibility statement", "Cookies"))

# Add links, internal and external
footer(
  links = c(
    "Accessibility statement",
    "Cookies",
    `Government Digital Service` =
    "https://www.gov.uk/government/organisations/government-digital-service"
  )
)

# Full app with link controlling a hidden tab and a link to an external page
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png",
    logo_alt_text = "Ministry of Justice logo"
  ),
  shinyGovstyle::banner(
    inputId = "banner", type = "beta", "This is a new service"
  ),
  shiny::tabsetPanel(
    type = "hidden",
    id = "tabs",
    shiny::tabPanel(
      "Main content",
      value = "main",
      heading_text("Hello world!")
    ),
    shiny::tabPanel(
      "Accessibility statement",
      value = "accessibility-panel",
      heading_text("Accessibility statement")
    ),
    shiny::tabPanel(
      "Cookies",
      value = "cookies-panel",
      heading_text("Cookies")
    ),
  ),
  shinyGovstyle::footer(
    full = TRUE,
    links = c(
      `Accessibility statement` = "accessibility_footer_link",
      `Cookies` = "cookies_footer_link",
      `Government Digital Service` =
       paste0(
         "https://www.gov.uk/government/",
         "organisations/government-digital-service"
       )
    )
  )
)

server <- function(input, output, session) {
  shiny::observeEvent(input$accessibility_footer_link, {
    shiny::updateTabsetPanel(
      session,
      "tabs",
      selected = "accessibility-panel"
    )
  })
  shiny::observeEvent(input$cookies_footer_link, {
    shiny::updateTabsetPanel(
      session,
      "tabs",
      selected = "cookies-panel"
    )
  })
}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle page structure: 
\code{\link{banner}()},
\code{\link{cookieBanner}()},
\code{\link{gov_layout}()},
\code{\link{header}()},
\code{\link{layouts}},
\code{\link{skip_to_main}()}
}
\concept{Govstyle page structure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/full_width_overrides.R
\name{full_width_overrides}
\alias{full_width_overrides}
\title{Styling overrides for to give full width}
\usage{
full_width_overrides()
}
\value{
HTML containing CSS styling overrides
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}
This is an experimental function that will likely be removed in future
releases, as we work through updating the core package styling and
adding full width options into the components themselves. This is not
well tested and may cause unexpected styling issues when combined with
components from other packages, use at your own risk.
}
\examples{
shinyGovstyle::full_width_overrides()
}
\seealso{
Other Govstyle styling: 
\code{\link{font}()}
}
\concept{Govstyle styling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insert_text.R
\name{insert_text}
\alias{insert_text}
\title{Insert Text Function}
\usage{
insert_text(inputId, text)
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{text}{Text that you want to display on the insert}
}
\value{
a insert text HTML shiny tag object
}
\description{
This function loads the insert text component to display additional
information in a special format.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    shinyGovstyle::insert_text(
      inputId = "note",
      text = paste(
        "It can take up to 8 weeks to register a lasting power of",
        "attorney if there are no mistakes in the application."
      )
    )
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle feedback types: 
\code{\link{details}()},
\code{\link{gov_summary}()},
\code{\link{label_hint}()},
\code{\link{noti_banner}()},
\code{\link{panel_output}()},
\code{\link{tag_Input}()},
\code{\link{value_box}()},
\code{\link{warning_text}()}
}
\concept{Govstyle feedback types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/header.R
\name{header}
\alias{header}
\title{Header Function}
\usage{
header(
  org_name = "Shiny example app",
  service_name = NULL,
  logo = "shinyGovstyle/images/Dept_logo.svg",
  main_text = lifecycle::deprecated(),
  secondary_text = lifecycle::deprecated(),
  main_link = NULL,
  secondary_link = NULL,
  logo_alt_text = "Departmental logo",
  main_alt_text = NULL,
  secondary_alt_text = NULL,
  logo_width = 66,
  logo_height = 34
)
}
\arguments{
\item{org_name}{Organisation name that goes in the header}

\item{service_name}{Service name to supplement the organisation name}

\item{logo}{Add a link to a logo which will apply in the header. Use crown to
use the crown SVG version on GOV UK}

\item{main_text}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Use \code{org_name} instead}

\item{secondary_text}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Use \code{service_name}
instead}

\item{main_link}{Add a link for clicking on main text
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}}

\item{secondary_link}{Add a link for clicking on secondary
header \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}}

\item{logo_alt_text}{Add alternative text for the logo. Should be used when a
logo is used}

\item{main_alt_text}{Add alternative text for the main link. Should be used
when a main link is used \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}}

\item{secondary_alt_text}{Add alternative text for the secondary link. Should
be used when a secondary link is used \ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}}

\item{logo_width}{Change the logo size width CSS to improve fit}

\item{logo_height}{Change the logo size height CSS to improve fit}
}
\value{
a header HTML shiny tag object
}
\description{
This function create a header banner. For use at top of the screen
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png",
    logo_alt_text = "Ministry of Justice Logo"
  )
)

server <- function(input, output, session) {}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle page structure: 
\code{\link{banner}()},
\code{\link{cookieBanner}()},
\code{\link{footer}()},
\code{\link{gov_layout}()},
\code{\link{layouts}},
\code{\link{skip_to_main}()}
}
\concept{Govstyle page structure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/text_input.R
\name{text_Input}
\alias{text_Input}
\title{Text Input Function}
\usage{
text_Input(
  inputId,
  label,
  hint_label = NULL,
  type = "text",
  width = NULL,
  error = FALSE,
  error_message = NULL,
  prefix = NULL,
  suffix = NULL
)
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{label}{Display label for the control, or \code{NULL} for no label}

\item{hint_label}{Display hint label for the control, or \code{NULL} for
no hint label}

\item{type}{Type of text input to accept. Defaults to text}

\item{width}{control the size of the box based on number of characters
required.  Options are 30, 20, 10, 5, 4, 3, 2.  NULL will not limit the size}

\item{error}{Whenever to include error handling. Defaults to FALSE}

\item{error_message}{Message to display on error. Defaults to NULL}

\item{prefix}{Add a prefix to the box. Defaults to NULL}

\item{suffix}{Add a suffix to the box. Defaults to NULL}
}
\value{
a text input HTML shiny tag object
}
\description{
This function create a text input.
}
\examples{
ui <- shiny::fluidPage(
  # Required for error handling function
  shinyjs::useShinyjs(),
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::banner(
    inputId = "banner", type = "beta", 'This is a new service'
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    # Simple text box
    shinyGovstyle::text_Input(inputId = "eventId", label = "Event Name"),
    # Error text box
    shinyGovstyle::text_Input(
      inputId = "eventId2",
      label = "Event Name",
      hint_label = "This can be found on the letter",
      error = TRUE
    ),
    # Button to trigger error
    shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {
  # Trigger error on blank submit of eventId2
  observeEvent(input$submit, {
    if (input$eventId2 != "") {
      shinyGovstyle::error_off(inputId = "eventId2")
    } else {
      shinyGovstyle::error_on(
        inputId = "eventId2",
        error_message = "Please complete"
      )
    }
  })
}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle text types: 
\code{\link{date_Input}()},
\code{\link{gov_list}()},
\code{\link{heading_text}()},
\code{\link{input_field}()},
\code{\link{text_area_Input}()},
\code{\link{word_count}()}
}
\concept{Govstyle text types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input_field.R
\name{input_field}
\alias{input_field}
\title{Input Field Function}
\usage{
input_field(
  legend,
  labels,
  inputIds,
  widths = NULL,
  types = "text",
  error = FALSE,
  error_message = NULL
)
}
\arguments{
\item{legend}{Legend that goes above the fieldset}

\item{labels}{A list of labels for the text inputs}

\item{inputIds}{A list input slots that will be used to access the value}

\item{widths}{control the size of the box based on number of characters
required.  Options are 30, 20, 10, 5, 4, 3, 2. NULL will not limit the size}

\item{types}{text box types. Will default to text}

\item{error}{Whenever to icnlud error handling. Defaults to FALSE}

\item{error_message}{Message to display on error. Defaults to NULL}
}
\value{
a input field of HTML as a shiny tag object
}
\description{
This function inserts number of text inputs. Useful for addresses.
}
\examples{
ui <- shiny::fluidPage(
  # Required for error handling function
  shinyjs::useShinyjs(),
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::banner(
    inputId = "banner", type = "beta", 'This is a new service'
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    shinyGovstyle::input_field(
      legend ="List of three text boxes in a field",
      labels = c("Field 1", "Field 2", "Field 3"),
      inputIds = c("field1", "field2", "field3"),
      widths = c(30,20,10),
      error = TRUE
    ),
   # Button to trigger error
   shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {
  # Trigger error on blank submit of field2
  observeEvent(input$submit, {
    if (input$field2 == ""){
      shinyGovstyle::error_on(
        inputId = "field2",
        error_message = "Please complete"
      )
    } else {
      shinyGovstyle::error_off(
        inputId = "field2"
      )
    }
  })
}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle text types: 
\code{\link{date_Input}()},
\code{\link{gov_list}()},
\code{\link{heading_text}()},
\code{\link{text_Input}()},
\code{\link{text_area_Input}()},
\code{\link{word_count}()}
}
\concept{Govstyle text types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/service_navigation.R
\name{service_navigation}
\alias{service_navigation}
\title{Service navigation}
\usage{
service_navigation(links, service_name = NULL)
}
\arguments{
\item{links}{A vector of actionLinks to be added to the
service navigation. inputIDs are auto-generated by
lowercasing the link text and replacing all
non-alphanumeric characters with underscores, e.g.
"Overview page" becomes \code{overview_page}. For precise
control over inputIDs, supply a named vector where names
are the page titles and values are the inputIDs.}

\item{service_name}{An optional character string
containing the service name to be displayed in the
navigation bar}
}
\value{
Shiny tag object
}
\description{
Service navigation component consistent with the
\href{https://design-system.service.gov.uk/components/service-navigation/}{GDS service navigation}. # nolint

\strong{Note:} This component uses a hardcoded element ID
for the navigation list and mobile menu toggle. Using
multiple \code{service_navigation()} instances on the same
page will cause ID conflicts and the mobile menu toggle
may not work correctly.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header("Title", "Secondary heading"),
  shinyGovstyle::service_navigation(
     c("Summary data", "Detailed stats 1", "User guide")
  ),
  bslib::navset_hidden(
     id = "main_panels",
     bslib::nav_panel(
       "summary_data",
       shiny::tags$h2("Summary data")
     ),
     bslib::nav_panel(
       "detailed_stats_1",
       shiny::tags$h2("Detailed stats 1")
     ),
     bslib::nav_panel(
       "user_guide",
       shiny::tags$h2("User guide")
     )
   ),
   shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {
  observeEvent(
    input$summary_data,
    bslib::nav_select("main_panels", "summary_data")
  )
  observeEvent(
    input$detailed_stats_1,
    bslib::nav_select("main_panels", "detailed_stats_1")
  )
  observeEvent(
    input$user_guide,
    bslib::nav_select("main_panels", "user_guide")
  )
}

if (interactive()) shiny::shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle navigation: 
\code{\link{backlink_Input}()},
\code{\link{contents_link}()},
\code{\link{update_service_navigation}()}
}
\concept{Govstyle navigation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-examples.R
\docType{data}
\name{transport_data_small}
\alias{transport_data_small}
\title{Transport data small}
\format{
A data frame with 3 rows and 3 variables:
\describe{
\item{months}{The month the data belongs to}
\item{bikes}{Cost of bikes}
\item{cars}{Cost of cars}
}
}
\source{
Random data generated for the shinyGovstyle package
}
\usage{
transport_data_small
}
\description{
Smaller example dataset containing transport data, used to demonstrate
various table styles in the example app.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{gov_summary}
\alias{gov_summary}
\title{Tabs Function}
\usage{
gov_summary(inputId, headers, info, action = FALSE, border = TRUE)
}
\arguments{
\item{inputId}{The Id to access the summary list}

\item{headers}{input for the row headers value}

\item{info}{summary information values for the table}

\item{action}{whenever a change link is needed. Sets input to the value of
the headers using lowercase and with underscore to replace gaps. Default
set to \code{FALSE}}

\item{border}{set if the table should have borders. Default set
to \code{TRUE}}
}
\value{
a summary list table HTML shiny tag object
}
\description{
This function creates a tabs based table. It requires a single dataframe
with a grouping variable.
}
\examples{
# Create an example dataset
headers <- c(
  "Name",
  "Date of birth",
  "Contact information",
  "Contact details"
)
info <- c(
  "Sarah Philips",
  "5 January 1978",
  "72 Guild Street <br> London <br> SE23 6FH",
  "07700 900457 <br> sarah.phillips@example.com"
)

ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    shinyGovstyle::gov_summary("sumID", headers, info, action = FALSE)
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle feedback types: 
\code{\link{details}()},
\code{\link{insert_text}()},
\code{\link{label_hint}()},
\code{\link{noti_banner}()},
\code{\link{panel_output}()},
\code{\link{tag_Input}()},
\code{\link{value_box}()},
\code{\link{warning_text}()}
}
\concept{Govstyle feedback types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/file_Input.R
\name{file_Input}
\alias{file_Input}
\title{File Input Function}
\usage{
file_Input(
  inputId,
  label,
  multiple = FALSE,
  accept = NULL,
  width = NULL,
  buttonLabel = "Choose file",
  placeholder = "No file chosen",
  error = FALSE,
  error_message = NULL
)
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{label}{Display label for the control, or \code{NULL} for no label}

\item{multiple}{Whether the user should be allowed to select and upload
multiple files at once. Does not work on older browsers, including Internet
Explorer 9 and earlier}

\item{accept}{A character vector of MIME types; gives the browser a hint of
what kind of files the server is expecting}

\item{width}{The width of the input,  e.g. \code{'400px'}, or \code{'100\\\%'}}

\item{buttonLabel}{The label used on the button. Can be text or an HTML tag
object}

\item{placeholder}{The text to show before a file has been uploaded}

\item{error}{Whenever to icnlud error handling. Defaults to FALSE}

\item{error_message}{Message to display on error. Defaults to NULL}
}
\value{
a file input HTML shiny tag object
}
\description{
This function create a file upload component. It uses the basis of the
shiny fileInput function, but restyles the label and adds error onto it.
}
\examples{
ui <- shiny::fluidPage(
  # Required for error handling function
  shinyjs::useShinyjs(),
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::banner(
    inputId = "banner", type = "beta", 'This is a new service'
  ),
  shinyGovstyle::gov_layout(size = "two-thirds",
    # Simple file input
    shinyGovstyle::file_Input(inputId = "file1", label = "Upload a file"),
    # Error file
    shinyGovstyle::file_Input(
      inputId = "file2",
      label = "Upload a file",
      error = TRUE
    ),
    # Button to trigger error
    shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {
  # Trigger error on blank submit of file2
  observeEvent(input$submit, {
    if (is.null(input$file2)){
      shinyGovstyle::error_on(inputId = "file2")
    } else {
      shinyGovstyle::error_off(
        inputId = "file2"
      )
    }
  })
}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle select inputs: 
\code{\link{button_Input}()},
\code{\link{checkbox_Input}()},
\code{\link{radio_button_Input}()},
\code{\link{select_Input}()}
}
\concept{Govstyle select inputs}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_hint.R
\name{label_hint}
\alias{label_hint}
\title{Label with Hint Function}
\usage{
label_hint(inputId, label, hint_input = NULL)
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{label}{Display label for the control, or \code{NULL} for no label}

\item{hint_input}{Display hint label for the control, or \code{NULL} for
no hint label}
}
\value{
a label hint HTML shiny tag object
}
\description{
This function inserts a label and optional hint.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    label_hint(
      inputId = "label1",
      label = "This is a label",
      hint_input = "This is a hint"
    )
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle feedback types: 
\code{\link{details}()},
\code{\link{gov_summary}()},
\code{\link{insert_text}()},
\code{\link{noti_banner}()},
\code{\link{panel_output}()},
\code{\link{tag_Input}()},
\code{\link{value_box}()},
\code{\link{warning_text}()}
}
\concept{Govstyle feedback types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/service_navigation.R
\name{update_service_navigation}
\alias{update_service_navigation}
\title{Update the active item in a service navigation component}
\usage{
update_service_navigation(session, inputId)
}
\arguments{
\item{session}{The Shiny session object}

\item{inputId}{The inputId of the service navigation link to set as active}
}
\value{
NULL, called for side effects
}
\description{
Sends a message to the browser to update the highlighted active item in the
service navigation bar.

\strong{When you need this function:} when navigation is triggered
programmatically — for example, via a next / back button or a footer link
that points to a main page. In those cases the nav link itself is not
clicked, so the JavaScript binding does not fire and the active state does
not update automatically. Call \code{update_service_navigation()} alongside
your tab-switching call to keep them in sync.

\strong{When you don't need this function:} when the user clicks a service
navigation link directly. The JavaScript binding updates the active state
automatically, so you only need to switch the tab panel in your
\code{observeEvent()}.
}
\examples{
# Nav link clicked — JS handles active state, just switch the panel.
# Works the same whether you use shiny or bslib tab panels.
if (interactive()) {
  server <- function(input, output, session) {
    # shiny tabsetPanel
    shiny::observeEvent(input$page_two, {
      shiny::updateTabsetPanel(session, "tabs", selected = "page_two")
    })

    # bslib navset
    shiny::observeEvent(input$page_two, {
      bslib::nav_select("tabs", "page_two")
    })
  }
}

# Programmatic navigation (e.g. a next / back button) — the nav link is not
# clicked, so you must also call update_service_navigation() explicitly.
if (interactive()) {
  server <- function(input, output, session) {
    # shiny tabsetPanel
    shiny::observeEvent(input$next_btn, {
      shiny::updateTabsetPanel(session, "tabs", selected = "page_two")
      shinyGovstyle::update_service_navigation(session, "page_two")
    })

    # bslib navset
    shiny::observeEvent(input$next_btn, {
      bslib::nav_select("tabs", "page_two")
      shinyGovstyle::update_service_navigation(session, "page_two")
    })
  }
}
}
\seealso{
Other Govstyle navigation: 
\code{\link{backlink_Input}()},
\code{\link{contents_link}()},
\code{\link{service_navigation}()}
}
\concept{Govstyle navigation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/external_link.R
\name{external_link}
\alias{external_link}
\title{External link}
\usage{
external_link(href, link_text, add_warning = TRUE, footer = FALSE)
}
\arguments{
\item{href}{URL that you want the link to point to}

\item{link_text}{Text that will appear describing your link, must be
descriptive of the page you are linking to. Vague text like 'click here' or
'here' will cause an error, as will ending in a full stop. Leading and
trailing white space will be automatically trimmed. If the string is shorter
than 7 characters a console warning will be thrown. There is no way to hush
this other than providing more detail}

\item{add_warning}{Boolean for adding "(opens in new tab)" at the end of the
link text to warn users of the behaviour. Be careful and consider
accessibility before removing the visual warning}

\item{footer}{Apply standard GDS footer CSS styling. Logical,
default = FALSE}
}
\value{
shiny tag object
}
\description{
It is commonplace for external links to open in a new tab, and when we do
this we should be careful...

This function automatically adds the following to your link:
\itemize{
\item \code{target="_blank"} to open in new tab
\item \code{rel="noopener noreferrer"} to prevent \href{https://owasp.org/www-community/attacks/Reverse_Tabnabbing}{reverse tabnabbing}
}

By default this function also adds "(opens in new tab)" to your link text
to warn users of the behaviour.

This also adds "This link opens in a new tab" as a visually hidden span
element within the HTML outputted to warn non-visual users of the behaviour.

The function will error if you end with a full stop, give a warning for
particularly short link text and will automatically trim any leading or
trailing white space inputted into link_text.

If you are displaying lots of links together and want to save space by
avoiding repeating (opens in new tab), then you can set add_warning = FALSE
and add a line of text above all of the links saying something like 'The
following links open in a new tab'.

Related links and guidance:
\itemize{
\item \href{https://design-system.service.gov.uk/styles/links/}{Government digital services guidelines on the use of links}
\item \href{https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a}{Anchor tag HTML element and its properties}
\item \href{https://www.w3.org/WAI/WCAG22/Understanding/link-purpose-in-context}{WCAG 2.2 success criteria 2.4.4: Link Purpose (In Context)}
\item \href{https://www.w3.org/TR/WCAG20-TECHS/G200.html}{Web Accessibility standards link text behaviour}
}
}
\details{
Intentionally basic wrapper for HTML anchor elements making it easier to
create safe external links with standard and accessible behaviour. For more
information on how the tag is generated, see \code{\link[htmltools:builder]{htmltools::tags()}}.
}
\examples{
external_link("https://shiny.posit.co/", "R Shiny")

external_link(
  "https://shiny.posit.co/",
  "R Shiny",
  add_warning = FALSE
)

# This will trim and show as 'R Shiny'
external_link("https://shiny.posit.co/", "  R Shiny")

# Example of within text
shiny::tags$p(
  "Oi, ", external_link("https://shiny.posit.co/", "R Shiny"), " is great."
)

# Example of multiple links together
shiny::tags$h2("Related resources")
shiny::tags$p("The following links open in a new tab.")
shiny::tags$ul(
  shiny::tags$li(
    external_link(
      "https://shiny.posit.co/",
      "R Shiny documentation",
      add_warning = FALSE
    )
  ),
  shiny::tags$li(
    external_link(
      "https://www.python.org/",
      "Python documentation",
      add_warning = FALSE
    )
  ),
  shiny::tags$li(
    external_link(
      "https://nextjs.org/",
      "Next.js documentation",
      add_warning = FALSE
    )
  )
)
}
\seealso{
Other Govstyle actions: 
\code{\link{download_button}()},
\code{\link{download_link}()},
\code{\link{download_radios}()},
\code{\link{download_radios_handler}()}
}
\concept{Govstyle actions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/footer.R
\name{footer_link}
\alias{footer_link}
\title{Create a footer link for use in \code{footer()} function}
\usage{
footer_link(link, link_name = NULL)
}
\arguments{
\item{link}{Character string containing either link text or url}

\item{link_name}{Name of a link where a URL has been provided in link_text}
}
\value{
HTML tag list item
}
\description{
Create a footer link for use in \code{footer()} function
}
\examples{
# Internal (i.e. within dashboard) link
shinyGovstyle:::footer_link("Cookie statement")
# Named internal link
shinyGovstyle:::footer_link("cookie_statement", "Cookies")
# External link
shinyGovstyle:::footer_link(
  "https://www.gov.uk/government/organisations/government-digital-service",
  "Government Digital Service"
)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_heading_text.R
\name{clean_heading_text}
\alias{clean_heading_text}
\title{Clean heading text function
This internal function is used in \code{heading_text()}
to create an automated link between the two}
\usage{
clean_heading_text(text)
}
\arguments{
\item{text}{string}
}
\value{
a string
}
\description{
Clean heading text function
This internal function is used in \code{heading_text()}
to create an automated link between the two
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gov_layout.R
\name{gov_layout}
\alias{gov_layout}
\title{Page Layout Function}
\usage{
gov_layout(..., inputID = "main", size = "full")
}
\arguments{
\item{...}{include the components of the UI that you want within the
main page.}

\item{inputID}{ID of the main div. Defaults to "main"}

\item{size}{Layout of the page. Optional are full, one-half, two-thirds,
one-third and one-quarter. Defaults to "full"}
}
\value{
a HTML shiny layout div
}
\description{
This function loads the page layout, This doesn't work as well as
the \code{gov_main_layout} and associated functions. This is being kept for now
as a simpler version where grids are not needed.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::gov_layout(
    size = "full",
    shinyGovstyle::panel_output(
      inputId = "panel1",
      main_text = "Application Complete",
      sub_text = paste(
        "Thank you for submitting your application.",
        "Your reference is xvsiq"
      )
    ),
    shinyGovstyle::footer(full = TRUE)
  )
)

server <- function(input, output, session) {}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle page structure: 
\code{\link{banner}()},
\code{\link{cookieBanner}()},
\code{\link{footer}()},
\code{\link{header}()},
\code{\link{layouts}},
\code{\link{skip_to_main}()}
}
\concept{Govstyle page structure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/font.R
\name{font}
\alias{font}
\title{Font Function}
\usage{
font()
}
\value{
no value returned. This loads the font CSS file
}
\description{
Loads the GDS Transport font for use in your app. GDS Transport is a
restricted typeface that must only be used on GOV.UK domains. If your app
is not hosted on a GOV.UK domain, do not call this function — the GOV.UK
Frontend CSS will fall back to Helvetica or Arial automatically.
}
\details{
See the
\href{https://design-system.service.gov.uk/styles/typeface/}{GOV.UK
typeface guidance} for full details on when GDS Transport is permitted.
}
\examples{
ui <- shiny::fluidPage(
  font(),
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png")
)

server <- function(input, output, session) {}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle styling: 
\code{\link{full_width_overrides}()}
}
\concept{Govstyle styling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkbox_input.R
\name{checkbox_Input}
\alias{checkbox_Input}
\title{Checkbox Function}
\usage{
checkbox_Input(
  inputId,
  cb_labels,
  checkboxIds,
  label,
  hint_label = NULL,
  small = FALSE,
  error = FALSE,
  error_message = NULL
)
}
\arguments{
\item{inputId}{Input Id for the group of checkboxes}

\item{cb_labels}{Add the names of the options that will appear}

\item{checkboxIds}{Add the values for each checkbox}

\item{label}{Insert the text for the checkbox group}

\item{hint_label}{Insert optional hint/secondary text. Defaults to NULL}

\item{small}{change the sizing to a small version of the checkbox. Defaults
to \code{FALSE}}

\item{error}{Whenever you want to include error handle on the component}

\item{error_message}{If you want a default error message}
}
\value{
a checkbox HTML shiny tag object
}
\description{
This function inserts a checkbox group
}
\examples{
ui <- shiny::fluidPage(
  # Required for error handling function
  shinyjs::useShinyjs(),
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"),
  shinyGovstyle::banner(
    inputId = "banner", type = "beta", 'This is a new service'),
  shinyGovstyle::gov_layout(size = "two-thirds",
    # Simple checkbox
    shinyGovstyle::checkbox_Input(
      inputId = "check1",
      cb_labels = c("Option 1", "Option 2", "Option 3"),
      checkboxIds = c("op1", "op2", "op3"),
      label = "Choice option"
    ),
    # Error checkbox
    shinyGovstyle::checkbox_Input(
      inputId = "check2",
      cb_labels = c("Option 1", "Option 2", "Option 3"),
      checkboxIds = c("op1", "op2", "op3"),
      label = "Choice option",
      hint_label = "Select the best fit",
      error = TRUE,
      error_message = "Select one"
    ),
    # Button to trigger error
    shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {
  # Trigger error on blank submit of eventId2
  observeEvent(input$submit, {
    if (is.null(input$check2)){
      shinyGovstyle::error_on(inputId = "check2")
    } else {
      shinyGovstyle::error_off(inputId = "check2")
    }
  })
}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle select inputs: 
\code{\link{button_Input}()},
\code{\link{file_Input}()},
\code{\link{radio_button_Input}()},
\code{\link{select_Input}()}
}
\concept{Govstyle select inputs}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/error_off.R
\name{error_off}
\alias{error_off}
\title{Error off Function}
\usage{
error_off(inputId)
}
\arguments{
\item{inputId}{The input Id to turn error handling on for}
}
\value{
no return value. This toggles off error CSS
}
\description{
This function turns off the the error o the component, once issues have
been sorted.
}
\examples{
ui <- shiny::fluidPage(
  # Required for error handling function
  shinyjs::useShinyjs(),
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::banner(
    inputId = "banner", type = "beta", 'This is a new service'
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    # Error text box
    shinyGovstyle::text_Input(
      inputId = "eventId",
      label = "Event Name",
      error = TRUE
    ),
    # Button to trigger error
    shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {
  # Trigger error on blank submit of eventId2
  observeEvent(input$submit, {
    if (input$eventId != "") {
      shinyGovstyle::error_off(inputId = "eventId")
    } else {
      shinyGovstyle::error_on(
        inputId = "eventId",
        error_message = "Please complete"
      )
    }
  })
}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle errors: 
\code{\link{error_on}()},
\code{\link{error_summary}()},
\code{\link{error_summary_update}()}
}
\concept{Govstyle errors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/govTabs.R
\name{govTabs}
\alias{govTabs}
\title{Tabs Function}
\usage{
govTabs(inputId, df, group_col)
}
\arguments{
\item{inputId}{The Id to access the tag}

\item{df}{A single dataframe with all data. See example for structure}

\item{group_col}{The column name with the groups to be used as tabs}
}
\value{
a tab table HTML shiny tag object
}
\description{
This function creates a tabs based table. It requires a single dataframe
with a grouping variable.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png",
    logo_alt_text = "Ministry of Justice logo"
  ),
  shinyGovstyle::gov_main_layout(
    size = "two-thirds",
    shinyGovstyle::govTabs("tabs", shinyGovstyle::case_data, "tabs")
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle tables tabs and accordions: 
\code{\link{accordion}()},
\code{\link{govReactable}()},
\code{\link{govReactable-shiny}},
\code{\link{govTable}()}
}
\concept{Govstyle tables tabs and accordions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-examples.R
\docType{data}
\name{case_data}
\alias{case_data}
\title{Case data}
\format{
A data frame with 12 rows and 4 variables:
\describe{
\item{tabs}{The tab the data belongs to}
\item{case_manager}{The name of the case manager}
\item{cases_open}{Number of open cases}
\item{cases_closed}{Number of closed cases}
}
}
\source{
Random data generated for the shinyGovstyle package
}
\usage{
case_data
}
\description{
Simple example dataset containing case management data, used to demonstrate
tabs in the example app.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/warning_text.R
\name{warning_text}
\alias{warning_text}
\title{Warning Text Function}
\usage{
warning_text(inputId, text)
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{text}{Text that goes in the main}
}
\value{
a warning box HTML shiny tag object
}
\description{
This function create warning text.
}
\examples{
shinyGovstyle::warning_text(
  inputId = "warn1",
  text = "You can be fined up to £5,000 if you do not register."
)
}
\seealso{
Other Govstyle feedback types: 
\code{\link{details}()},
\code{\link{gov_summary}()},
\code{\link{insert_text}()},
\code{\link{label_hint}()},
\code{\link{noti_banner}()},
\code{\link{panel_output}()},
\code{\link{tag_Input}()},
\code{\link{value_box}()}
}
\concept{Govstyle feedback types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_radios.R
\name{download_radios}
\alias{download_radios}
\title{Download with extension radios}
\usage{
download_radios(
  id = "download_radios",
  download_type = "table",
  file_types = c("CSV", "ODS", "XLSX"),
  file_sizes = c("< 1 GB", "< 1 GB", "< 1 GB"),
  small = FALSE
)
}
\arguments{
\item{id}{Shiny element Id. Default is "download_radios", but must be
customised to be unique if multiple instances of this module are being
used in a single app. Must match up to the ID
of a \code{download_radios_helper()} instance in server code}

\item{download_type}{Element being downloaded. Expected to be along the
lines of "underlying data", "table", "current data view". Default is "table"}

\item{file_types}{File formats to offer, can be any combination of
"CSV", "XLSX", "ODS". Default is a vector of all 3}

\item{file_sizes}{Estimated file sizes for each file format. This needs to
be a vector of the same length as file_types}

\item{small}{Set radio buttons to small size (logical, default: FALSE)}
}
\value{
UI containing radio selection and download button
}
\description{
Download with extension radios
}
\examples{
ui <- shiny::fluidPage(
  download_radios("download_file",
    file_types = c("CSV", "ODS"),
    file_sizes = c("2 KB", "5 KB")
  )
)

server <- function(input, output, session) {
  output$download_file <- download_radios_handler(
    "download_file",
    file_name = "simple_data_frame",
    file_contents = mtcars
  )
}

if (interactive()) shinyApp(ui, server)
}
\seealso{
Other Govstyle actions: 
\code{\link{download_button}()},
\code{\link{download_link}()},
\code{\link{download_radios_handler}()},
\code{\link{external_link}()}
}
\concept{Govstyle actions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/govLayouts.R
\name{layouts}
\alias{layouts}
\alias{gov_main_layout}
\alias{gov_row}
\alias{gov_box}
\alias{gov_text}
\title{Page Layout Functions}
\usage{
gov_main_layout(..., inputID = "main")

gov_row(...)

gov_box(..., size = "full")

gov_text(...)
}
\arguments{
\item{...}{include the components of the UI that you want within the
main page. These components are made to flow through each other. See
example}

\item{inputID}{ID of the main div. Defaults to "main"}

\item{size}{size of the box in the row. Optional are full, one-half,
two-thirds, one-third and one-quarter. Defaults to "full"}
}
\value{
a HTML shiny layout div
}
\description{
These function loads the page layout in a gov layout. There is a selection
of components that can sit within each other. The gov_main_layout is the
overarching layout. The gov_row creates a each row and gov_box creates
a box within the row. The gov_text is a container for text bodies.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"),
  shinyGovstyle::gov_main_layout(
    shinyGovstyle::gov_row(
      shinyGovstyle::gov_box(
        size = "full",
        shinyGovstyle::gov_text("govuk-grid-column-full")
      )
    ),
    shinyGovstyle::gov_row(
      shinyGovstyle::gov_box(
        size = "one-half",
        shinyGovstyle::gov_text("govuk-grid-column-one-half")
      ),
      shinyGovstyle::gov_box(
        size = "one-half",
        shinyGovstyle::gov_text("govuk-grid-column-one-half")
      )
    ),
    shinyGovstyle::gov_row(
      shinyGovstyle::gov_box(
        size = "one-third",
        shinyGovstyle::gov_text("govuk-grid-column-one-third")
      ),
      shinyGovstyle::gov_box(
        size = "two-third",
        shinyGovstyle::gov_text("govuk-grid-column-two-third")
      )
    ),
    shinyGovstyle::gov_row(
      shinyGovstyle::gov_box(
        size = "one-quarter",
        shinyGovstyle::gov_text("govuk-grid-column-one-quarter")
      ),
      shinyGovstyle::gov_box(
        size = "three-quarters",
        shinyGovstyle::gov_text("govuk-grid-column-three-quarters")
      )
    )
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle page structure: 
\code{\link{banner}()},
\code{\link{cookieBanner}()},
\code{\link{footer}()},
\code{\link{gov_layout}()},
\code{\link{header}()},
\code{\link{skip_to_main}()}
}
\concept{Govstyle page structure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/button_Input.R
\name{button_Input}
\alias{button_Input}
\title{Button Function}
\usage{
button_Input(inputId, label, type = "default")
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{label}{Display label for the control, or \code{NULL} for no label}

\item{type}{The type of button. Options are default, start, secondary and
warning. Defaults to "default"}
}
\value{
a HTML button shiny tag object
}
\description{
This function create a gov style button
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    shinyGovstyle::button_Input(
      inputId = "btn1",
      label = "Continue",
      type = "default"
    )
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle select inputs: 
\code{\link{checkbox_Input}()},
\code{\link{file_Input}()},
\code{\link{radio_button_Input}()},
\code{\link{select_Input}()}
}
\concept{Govstyle select inputs}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cookieBanner.R
\name{cookieBanner}
\alias{cookieBanner}
\title{Cookie Banner Function}
\usage{
cookieBanner(service_name)
}
\arguments{
\item{service_name}{Name for this service to add to banner}
}
\value{
a cookie banner HTML shiny tag object
}
\description{
This function creates a cookie banner. You need to have
\code{shinyjs::useShinyjs()} enabled for this to work. All the Ids are preset.
See example for how to structure.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"),
  #Needs shinyjs to work
  shinyjs::useShinyjs(),
  shinyGovstyle::cookieBanner("The best thing"),
  shinyGovstyle::gov_layout(size = "two-thirds"),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {
  # Set of observeEvent to create a path through the cookie banner
  observeEvent(input$cookieAccept, {
    shinyjs::show(id = "cookieAcceptDiv")
    shinyjs::hide(id = "cookieMain")
  })

  observeEvent(input$cookieReject, {
    shinyjs::show(id = "cookieRejectDiv")
    shinyjs::hide(id = "cookieMain")
  })

  observeEvent(input$hideAccept, {
    shinyjs::toggle(id = "cookieDiv")
  })

  observeEvent(input$hideReject, {
    shinyjs::toggle(id = "cookieDiv")
  })

  observeEvent(input$cookieLink, {
    # Need to link here to where further info is located. You can use
    # updateTabsetPanel to have a cookie page for instance
  })

}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle page structure: 
\code{\link{banner}()},
\code{\link{footer}()},
\code{\link{gov_layout}()},
\code{\link{header}()},
\code{\link{layouts}},
\code{\link{skip_to_main}()}
}
\concept{Govstyle page structure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/noti_banner.R
\name{noti_banner}
\alias{noti_banner}
\title{Notification Banner Function}
\usage{
noti_banner(
  inputId,
  title_txt = "Important",
  body_txt = NULL,
  type = "standard"
)
}
\arguments{
\item{inputId}{The input Id for the banner}

\item{title_txt}{The wording that appears in the title}

\item{body_txt}{The wording that appears in the banner body}

\item{type}{The type of banner. Options are standard and success.
Standard is default}
}
\value{
a notification HTML shiny tag object
}
\description{
This function creates a notification banner.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::noti_banner(
    inputId = "banner", title_txt = "Important", body_txt = "Example text"
  )
)

server <- function(input, output, session) {}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle feedback types: 
\code{\link{details}()},
\code{\link{gov_summary}()},
\code{\link{insert_text}()},
\code{\link{label_hint}()},
\code{\link{panel_output}()},
\code{\link{tag_Input}()},
\code{\link{value_box}()},
\code{\link{warning_text}()}
}
\concept{Govstyle feedback types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/panel_output.R
\name{panel_output}
\alias{panel_output}
\title{Panel output}
\usage{
panel_output(inputId, main_text, sub_text)
}
\arguments{
\item{inputId}{The input slot that will be used to access the value.}

\item{main_text}{Add the header for the panel}

\item{sub_text}{Add the main body of text for the panel}
}
\value{
a panel HTML shiny tag object
}
\description{
This function inserts a panel.  Normally used for confirmation screens
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::gov_layout(size = "full",
    shinyGovstyle::panel_output(
      inputId = "panel1",
      main_text = "Application Complete",
      sub_text = paste(
        "Thank you for submitting your application.",
        "Your reference is xvsiq"
      )
    ),
    shinyGovstyle::footer(full = TRUE)
  )
)

server <- function(input, output, session) {}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle feedback types: 
\code{\link{details}()},
\code{\link{gov_summary}()},
\code{\link{insert_text}()},
\code{\link{label_hint}()},
\code{\link{noti_banner}()},
\code{\link{tag_Input}()},
\code{\link{value_box}()},
\code{\link{warning_text}()}
}
\concept{Govstyle feedback types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag.R
\name{tag_Input}
\alias{tag_Input}
\title{Tag Function}
\usage{
tag_Input(inputId, text, colour = "navy")
}
\arguments{
\item{inputId}{The Id to access the tag}

\item{text}{The text in the tag}

\item{colour}{The colour of the tag. Default is navy. Other options are
grey, green, teal, blue, purple, magenta, red, orange and yellow}
}
\value{
a tag HTML shiny tag object
}
\description{
This function creates a tag.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    shinyGovstyle::tag_Input("tag1", "Complete"),
    shinyGovstyle::tag_Input("tag2", "Incomplete", "red")
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle feedback types: 
\code{\link{details}()},
\code{\link{gov_summary}()},
\code{\link{insert_text}()},
\code{\link{label_hint}()},
\code{\link{noti_banner}()},
\code{\link{panel_output}()},
\code{\link{value_box}()},
\code{\link{warning_text}()}
}
\concept{Govstyle feedback types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contents_links.R
\name{contents_link}
\alias{contents_link}
\title{Contents link function}
\usage{
contents_link(link_text, input_id, subcontents_text_list, subcontents_id_list)
}
\arguments{
\item{link_text}{vector of link text for contents}

\item{input_id}{contents button Id}

\item{subcontents_text_list}{vector of link text for subcontents}

\item{subcontents_id_list}{vector of link Ids for subcontents. If missing
automatically matches to Id in \code{heading_text()}}
}
\value{
an action button HTML shiny tag object
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
}
\details{
This function creates an action link to nav between tabs and optionally
link to subcontents headers.
}
\examples{
ui <- shiny::fluidPage(
  gov_row(
    # Nav columns
    shiny::column(
      width = 3,
      id = "nav", # DO NOT REMOVE ID

      # Contents box
      shiny::tags$div(
        id = "govuk-contents-box", # DO NOT REMOVE ID
        class = "govuk-contents-box", # DO NOT REMOVE CLASS

        shiny::tags$h2("Contents"),

        # Text types tab
        contents_link(
          "Text Types",
          "text_types_button",
          subcontents_text_list = c(
            "date_Input",
            "text_Input",
            "text_area_Input",
            "button_Input"
          ),
          subcontents_id_list = c(NA, NA, NA, "button_input_text_types")
        ),

        # Tables tabs and accordions tab
        contents_link(
          "Tables, tabs and accordions",
          "tables_tabs_and_accordions_button",
          subcontents_text_list = c(
            "govTable",
            "govTabs",
            "accordions",
            "button_Input"
          ),
          subcontents_id_list = c(
            NA,
            NA,
            NA,
            "button_input_tables_tabs_accordions"
          )
        ),

        contents_link(
          "Cookies",
          "cookies_button"
        ),
      )
    ),

    shiny::column(
      width = 9,
      id = "main_col", # DO NOT REMOVE ID

      # Set up a nav panel so everything not on single page
      shiny::tabsetPanel(
        type = "hidden",
        id = "tab-container", # DO NOT REMOVE ID

        shiny::tabPanel(
          "Text Types",
          value = "text_types",
          gov_layout(
            size = "two-thirds",
            backlink_Input("back1"),
            heading_text("Page 2", size = "l"),
            label_hint(
              "label2",
              paste(
                "These are some examples of the",
                "types of user text inputs that you can use"
              )
            ),
            heading_text("date_Input", size = "s"),
            date_Input(
              inputId = "date1",
              label = "What is your date of birth?",
              hint_label = "For example, 31 3 1980"
            ),
            heading_text("text_Input", size = "s"),
            text_Input(
              inputId = "txt1",
              label = "Event name"
            ),
            heading_text("text_area_Input", size = "s"),
            text_area_Input(
              inputId = "text_area1",
              label = "Can you provide more detail?",
              hint_label = paste(
                "Do not include personal or financial",
                "information, like your National Insurance",
                "number or credit card details."
              )
            ),
            text_area_Input(
              inputId = "text_area2",
              label = "How are you today?",
              hint_label = "Leave blank to trigger error",
              error = TRUE,
              error_message = "Please do not leave blank",
              word_limit = 300
            ),
            heading_text(
              "button_Input",
              size = "s",
              id = "button_input_text_types"
            ),
            button_Input("btn2", "Go to next page"),
            button_Input(
              "btn3",
              "Check for errors",
              type = "warning"
            )
          )
        ),

        shiny::tabPanel(
          "Tables, tabs and accordions",
          value = "tables_tabs_and_accordions",
          gov_layout(
            size = "two-thirds",
            backlink_Input("back2"),
            heading_text("Page 3", size = "l"),
            label_hint(
              "label3",
              paste(
                "These are some examples of using tabs",
                "type tables"
              )
            ),
            heading_text("govTable", size = "s"),
            heading_text("govTabs", size = "s"),
            heading_text("accordions", size = "s"),
            shinyGovstyle::accordion(
              "acc1",
              c(
                "Writing well for the web",
                "Writing well for specialists",
                "Know your audience",
                "How people read"
              ),
              c(
                paste(
                  "This is the content for Writing well",
                  "for the web."
                ),
                paste(
                  "This is the content for Writing well",
                  "for specialists."
                ),
                paste(
                  "This is the content for",
                  "Know your audience."
                ),
                "This is the content for How people read."
              )
            ),

            heading_text(
              "button_Input",
              size = "s",
              id = "button_input_tables_tabs_accordions"
            ),
            button_Input("btn4", "Go to next page"),
          )
        ),

        #################### Create cookie panel #########
        shiny::tabPanel(
          "Cookies",
          value = "panel-cookies",
          gov_layout(
            size = "two-thirds",
            heading_text("Cookie page", size = "l"),
            label_hint(
              "label-cookies",
              "This an example cookie page"
            )
          )
        )
      )
    )
  ), # end of main_col
  footer(TRUE)
) # end of gov_row

server <- function(input, output, session) {
  # Tab nav
  shiny::observeEvent(input$back2, {
    shiny::updateTabsetPanel(
      session,
      "tab-container",
      selected = "text_types"
    )
  })

  shiny::observeEvent(input$tables_tabs_and_accordions_button, {
    shiny::updateTabsetPanel(
      session,
      "tab-container",
      selected = "tables_tabs_and_accordions"
    )
  })

  shiny::observeEvent(input$cookies_button, {
    shiny::updateTabsetPanel(
      session,
      "tab-container",
      selected = "panel-cookies"
    )
  })
} # end of server

if (interactive()) shiny::shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle navigation: 
\code{\link{backlink_Input}()},
\code{\link{service_navigation}()},
\code{\link{update_service_navigation}()}
}
\concept{Govstyle navigation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-bad_link_text.R
\docType{data}
\name{bad_link_text}
\alias{bad_link_text}
\title{Lookup for bad link text}
\format{
\subsection{\code{bad_link_text}}{

A data frame with 53 rows and 1 columns:
\describe{
\item{bad_link_text}{Lower cased examples of non-descriptive link text}
}
}
}
\source{
Curated by explore.statistics@education.gov.uk
}
\usage{
bad_link_text
}
\description{
A single column data frame, listing out known examples of bad link text that
check for in the \code{external_link()} function.
}
\details{
We've started curating this list so we can create automated checks to help
all link text to be as descriptive as possible in line with
\href{https://www.w3.org/WAI/WCAG22/Understanding/link-purpose-in-context}{WCAG 2.2 success criteria 2.4.4: Link Purpose (In Context)}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/banner.R
\name{banner}
\alias{banner}
\title{Banner Function}
\usage{
banner(inputId, type, label)
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{type}{Main type of label e.g. alpha or beta. Can be any word}

\item{label}{text to display}
}
\value{
a banner HTML shiny tag object
}
\description{
This function create a detail component that you can click for further
details.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::banner(
    inputId = "banner", type = "Beta", 'This is a new service'
  )
)

server <- function(input, output, session) {}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle page structure: 
\code{\link{cookieBanner}()},
\code{\link{footer}()},
\code{\link{gov_layout}()},
\code{\link{header}()},
\code{\link{layouts}},
\code{\link{skip_to_main}()}
}
\concept{Govstyle page structure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heading_text.R
\name{heading_text}
\alias{heading_text}
\title{Heading Text Function}
\usage{
heading_text(text_input, size = "xl", id, level = 1)
}
\arguments{
\item{text_input}{Text to display}

\item{size}{Text size using xl, l, m, s. Defaults to xl}

\item{id}{Custom header id}

\item{level}{Heading level, integer between 1 and 6. Defaults to 1}
}
\value{
a heading text HTML shiny tag object
}
\description{
This function createS heading text
}
\examples{
shinyGovstyle::heading_text("This is great text")
shinyGovstyle::heading_text("This is great text", size = "l", level = 2)
}
\seealso{
Other Govstyle text types: 
\code{\link{date_Input}()},
\code{\link{gov_list}()},
\code{\link{input_field}()},
\code{\link{text_Input}()},
\code{\link{text_area_Input}()},
\code{\link{word_count}()}
}
\concept{Govstyle text types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_radios.R
\name{download_radios_handler}
\alias{download_radios_handler}
\title{Download with extension radios handler}
\usage{
download_radios_handler(id = "download_radios", file_name, file_contents)
}
\arguments{
\item{id}{Shiny element Id. Default is "download_radios", but must be
customised to be unique if multiple instances of this module are being
used in a single app. Must match up to the ID of a \code{download_radios()}
instance in UI code}

\item{file_name}{Name of the file to be downloaded}

\item{file_contents}{Contents to write to the download file}
}
\value{
Output for use with \code{download_radios()}
}
\description{
Download with extension radios handler
}
\examples{
ui <- shiny::fluidPage(
  download_radios("download_file",
    file_types = c("CSV", "ODS"),
    file_sizes = c("2 KB", "5 KB")
  )
)

server <- function(input, output, session) {
  output$download_file <- download_radios_handler(
    "download_file",
    file_name = "simple_data_frame",
    file_contents = mtcars
  )
}

if (interactive()) shinyApp(ui, server)
}
\seealso{
Other Govstyle actions: 
\code{\link{download_button}()},
\code{\link{download_link}()},
\code{\link{download_radios}()},
\code{\link{external_link}()}
}
\concept{Govstyle actions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/details.R
\name{details}
\alias{details}
\title{Details Function}
\usage{
details(inputId, label, help_text)
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{label}{Main label text}

\item{help_text}{Additional help information in the component}
}
\value{
a details box HTML shiny tag object
}
\description{
This function create a detail component that you can click for further
details.
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    shinyGovstyle::details(
      inputId = "help_div",
      label = "Help with form",
      help_text = "To complete the form you need to fill it in..."
    )
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle feedback types: 
\code{\link{gov_summary}()},
\code{\link{insert_text}()},
\code{\link{label_hint}()},
\code{\link{noti_banner}()},
\code{\link{panel_output}()},
\code{\link{tag_Input}()},
\code{\link{value_box}()},
\code{\link{warning_text}()}
}
\concept{Govstyle feedback types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/radio_button_input.R
\name{radio_button_Input}
\alias{radio_button_Input}
\title{Radio Button Function}
\usage{
radio_button_Input(
  inputId,
  label,
  choices = NULL,
  selected = NULL,
  inline = FALSE,
  small = FALSE,
  choiceNames = NULL,
  choiceValues = NULL,
  hint_label = NULL,
  error = FALSE,
  error_message = NULL,
  custom_class = ""
)
}
\arguments{
\item{inputId}{The \code{input} slot that will be used to access the value}

\item{label}{Input label}

\item{choices}{List of values to select from (if elements of the list are
named then that name rather than the value is displayed to the user)}

\item{selected}{The initially selected value.}

\item{inline}{If you want the radio inline or not,  Default is FALSE}

\item{small}{If you want the smaller versions of radio buttons,  Default
is FALSE}

\item{choiceNames, choiceValues}{Same as in
\code{\link[shiny:checkboxGroupInput]{shiny::checkboxGroupInput()}}. List of names and values,
respectively, that are displayed to the user in the app and correspond to
the each choice (for this reason they must have the same length). If either
of these arguments is provided, then the other must be provided and choices
must not be provided. The advantage of using both of these over a named list
for choices is that choiceNames allows any type of UI object to be passed
through (tag objects, icons, HTML code, ...), instead of just simple text}

\item{hint_label}{Additional hint text you may want to display below the
label. Defaults to NULL}

\item{error}{Whenever you want to include error handle on the component}

\item{error_message}{If you want a default error message}

\item{custom_class}{If you want to add additional classes to the radio
buttons}
}
\value{
radio buttons HTML shiny tag object
}
\description{
This function create radio buttons
}
\examples{
ui <- shiny::fluidPage(
  # Required for error handling function
  shinyjs::useShinyjs(),
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::banner(
    inputId = "banner", type = "beta", "This is a new service"
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    # Simple radio
    shinyGovstyle::radio_button_Input(
      inputId = "radio1",
      choices = c("Yes", "No", "Maybe"),
      label = "Choice option"
    ),
    # Error radio
    shinyGovstyle::radio_button_Input(
      inputId = "radio2",
      choices = c("Yes", "No", "Maybe"),
      label = "Choice option",
      hint_label = "Select the best fit",
      inline = TRUE,
      error = TRUE,
      error_message = "Select one"
    ),
    # Button to trigger error
    shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {
  # Trigger error on blank submit of eventId2
  observeEvent(input$submit, {
    if (is.null(input$radio2)) {
      shinyGovstyle::error_on(inputId = "radio2")
    } else {
      shinyGovstyle::error_off(
        inputId = "radio2"
      )
    }
  })
}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle select inputs: 
\code{\link{button_Input}()},
\code{\link{checkbox_Input}()},
\code{\link{file_Input}()},
\code{\link{select_Input}()}
}
\concept{Govstyle select inputs}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/govReactable.R
\name{govReactable-shiny}
\alias{govReactable-shiny}
\alias{govReactableOutput}
\alias{renderGovReactable}
\title{Shiny bindings for govReactable
Output and render functions for using govReactable within shiny apps}
\usage{
govReactableOutput(
  output_table_name,
  caption,
  caption_size = "l",
  heading_level = "h2"
)

renderGovReactable(expr, env = parent.frame(), quoted = FALSE)
}
\arguments{
\item{output_table_name}{Output variable to read from}

\item{caption}{Adds a caption to the table as a header}

\item{caption_size}{Adjust the size of caption
Options are s, m, l, xl, with l as the default}

\item{heading_level}{The HTML heading level for
the caption (e.g., "h2", "h3", "h4", "h5"). Default is "h2"}

\item{expr}{An expression that generates a \code{reactable} widget}

\item{env}{The environment in which to evaluate \code{expr}}

\item{quoted}{Is \code{expr} a quoted expression (with \code{\link[=quote]{quote()}})?
This is useful if you want to save an expression in a variable}
}
\value{
\code{govReactableOutput()} returns a \code{reactable} output element
that can be included in a Shiny UI

\code{renderGovReactable()} returns a \code{reactable} render function that
can be assigned to a Shiny output slot
}
\description{
Shiny bindings for govReactable
Output and render functions for using govReactable within shiny apps
}
\examples{
ui <- shiny::fluidPage(
  govReactableOutput(
    "table",
    caption = "Example table"
  )
)

server <- function(input, output, session) {
  output$table <- renderGovReactable({
    govReactable(iris)
  })
}

if (interactive()) shinyApp(ui, server)
}
\seealso{
Other Govstyle tables tabs and accordions: 
\code{\link{accordion}()},
\code{\link{govReactable}()},
\code{\link{govTable}()},
\code{\link{govTabs}()}
}
\concept{Govstyle tables tabs and accordions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/skip_to_main.R
\name{skip_to_main}
\alias{skip_to_main}
\title{Skip to main content link}
\usage{
skip_to_main(id = "main")
}
\arguments{
\item{id}{An optional parameter to specify the Id of the main content
section, will be automatically preceeded by a hash '#'. Default is
"main" to match the "#main" Id within \code{gov_main()}.}
}
\value{
A Shiny tag representing the 'Skip to main content' link
}
\description{
This function generates a 'Skip to main content' link, which is
typically used by keyboard users to bypass content and
navigate directly to the main content of a page.
}
\examples{
ui <- shiny::fluidPage(
  skip_to_main(),
  header(
    org_name = "Example",
    service_name = "User Examples",
    logo="shinyGovstyle/images/moj_logo.png"
  ),
  gov_main_layout(
    heading_text("Example heading"),
  )
)

server <- function(input, output, session){}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle page structure: 
\code{\link{banner}()},
\code{\link{cookieBanner}()},
\code{\link{footer}()},
\code{\link{gov_layout}()},
\code{\link{header}()},
\code{\link{layouts}}
}
\concept{Govstyle page structure}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/select_input.R
\name{select_Input}
\alias{select_Input}
\title{Select Function}
\usage{
select_Input(inputId, label, select_text, select_value)
}
\arguments{
\item{inputId}{Input Id for the component}

\item{label}{Insert the text for the label}

\item{select_text}{Add the text that will apply in the drop down as a list}

\item{select_value}{Add the value that will be used for each selection}
}
\value{
a select input HTML shiny tag object
}
\description{
This function inserts a select box
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png",
    logo_alt_text = "Ministry of Justice logo"
  ),
  shinyGovstyle::gov_layout(
    size = "full",
    select_Input(
      inputId = "sorter",
      label = "Sort by",
      select_text = c(
        "Recently published",
        "Recently updated",
        "Most views",
        "Most comments"
      ),
      select_value = c("published", "updated", "view", "comments")
    ),
    shiny::tags$br()
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle select inputs: 
\code{\link{button_Input}()},
\code{\link{checkbox_Input}()},
\code{\link{file_Input}()},
\code{\link{radio_button_Input}()}
}
\concept{Govstyle select inputs}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_Input.R
\name{date_Input}
\alias{date_Input}
\title{Date Input Function}
\usage{
date_Input(
  inputId,
  label,
  hint_label = NULL,
  error = FALSE,
  error_message = NULL,
  day = NULL,
  month = NULL,
  year = NULL
)
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{label}{Display label for the control, or \code{NULL} for no label}

\item{hint_label}{Display hint label for the control, or \code{NULL} for no
hint label}

\item{error}{Whenever to include error components.Defaults to \code{FALSE}}

\item{error_message}{Error handling message? Defaults to \code{NULL}}

\item{day}{Select a default day on start up. Defaults to \code{NULL}}

\item{month}{Select a default month on start up. Defaults to \code{NULL}}

\item{year}{Select a default year on start up. Defaults to \code{NULL}}
}
\value{
a data input HTML shiny tag object
}
\description{
This function create a date input that follows GDS component
}
\examples{
ui <- shiny::fluidPage(
  # Required for error handling function.
  shinyjs::useShinyjs(),
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::banner(
    inputId = "banner", type = "beta", 'This is a new service'
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    # Simple date input
    shinyGovstyle::date_Input(
      inputId = "dob_input",
      label = "Please enter your birthday"
    ),
    # Error date input
    shinyGovstyle::date_Input(
      inputId = "dob_input2",
      label = "Please enter your birthday",
      hint_label = "For example, 12 11 2007",
      error = TRUE
    ),
    # Button to trigger error
    shinyGovstyle::button_Input(inputId = "submit", label = "Submit")
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {
  # Trigger error on blank submit of dob_input2
  observeEvent(input$submit, {
    if (input$dob_input2 == "//") {
      shinyGovstyle::error_on(inputId = "dob_input2")
    } else {
      shinyGovstyle::error_off(inputId = "dob_input2")
    }
  })
}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle text types: 
\code{\link{gov_list}()},
\code{\link{heading_text}()},
\code{\link{input_field}()},
\code{\link{text_Input}()},
\code{\link{text_area_Input}()},
\code{\link{word_count}()}
}
\concept{Govstyle text types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_button.R
\name{download_button}
\alias{download_button}
\title{Download button}
\usage{
download_button(outputId, button_label, file_type = "CSV", file_size = NULL)
}
\arguments{
\item{outputId}{The name of the output slot that the
\code{shiny::downloadHandler()} is assigned to}

\item{button_label}{Text that will appear describing the download action.
Vague text like 'click here' or 'here' will cause an error, as will ending
in a full stop. Leading and trailing white space will be automatically
trimmed. If the string is shorter than 7 characters a console warning will
be thrown. There is no way to hush this other than providing more detail}

\item{file_type}{The file type to be download (default: CSV)}

\item{file_size}{The file size if known. Needs to be a string ending in one
of KB, MB, GB or rows}
}
\value{
shiny tag object
}
\description{
The \code{download_button()} provides a standard way to provide a download
link, which facilitates important accessible / positive user experience
elements, namely:
\itemize{
\item file type
\item file size
}
These are necessary in order for users to understand what they are
downloading, both in terms of being able to decide if they are comfortable
with downloading the file over their current connection and if it's in a
form they're able to deal with once it is downloaded. If the exact file
size is not easily determined, then it may be acceptable to provide an
estimate or an upper limit.
}
\examples{
ui <- shiny::fluidPage(
  gov_text("Choose a data set to download."),
  select_Input(
    "dataset",
    "Data set",
    select_text = c("Car road tests", "New York air quality"),
    select_value = c("mtcars", "airquality")
  ),
  download_button(
    "download_data",
    "Download selected data set",
    file_size = "4 KB"
  )
)

server <- function(input, output) {
  # The requested data set
  data <- reactive({
    get(input$dataset)
  })

  output$download_data <- downloadHandler(
    filename = function() {
      # Use the selected dataset as the suggested file name
      paste0(input$dataset, ".csv")
    },
    content = function(file) {
      # Write the dataset to the `file` that will be downloaded
      write.csv(data(), file)
    }
  )
}

if (interactive()) shiny::shinyApp(ui, server)
}
\seealso{
Other Govstyle actions: 
\code{\link{download_link}()},
\code{\link{download_radios}()},
\code{\link{download_radios_handler}()},
\code{\link{external_link}()}
}
\concept{Govstyle actions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/word_count.R
\name{word_count}
\alias{word_count}
\title{Word Count Function}
\usage{
word_count(inputId, input, word_limit = NULL)
}
\arguments{
\item{inputId}{The input slot of the text area that you want to affect}

\item{input}{The text input that is associated with the box}

\item{word_limit}{Change the word limit if needed. Default will keep as
what was used in text area component}
}
\value{
no value returned. Updates the word count in a shiny app
}
\description{
This function create tracks the word count and should be used with the
text area function.
}
\examples{
ui <- shiny::fluidPage(
  shinyjs::useShinyjs(),
  shinyGovstyle::header(
    "Justice", "", logo = "shinyGovstyle/images/moj_logo.png"
  ),
  gov_layout(
    size = "full",
    text_area_Input(
      inputId = "text_area",
      label = "Can you provide more detail?",
      hint_label = paste(
        "Do not include personal or financial information,",
        "like your National Insurance number or credit card details."
      ),
      word_limit = 300
    )
  ),
  footer(TRUE)
)

server <- function(input, output, session) {
  shiny::observeEvent(input$text_area,
    word_count(
      inputId = "text_area",
      input = input$text_area
    )
  )
}
if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle text types: 
\code{\link{date_Input}()},
\code{\link{gov_list}()},
\code{\link{heading_text}()},
\code{\link{input_field}()},
\code{\link{text_Input}()},
\code{\link{text_area_Input}()}
}
\concept{Govstyle text types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/govReactable.R
\name{govReactable}
\alias{govReactable}
\title{Interactive govTable}
\usage{
govReactable(
  df,
  right_col = NULL,
  page_size = 10,
  highlight = TRUE,
  borderless = TRUE,
  min_widths = list(),
  ...
)
}
\arguments{
\item{df}{A dataframe used to generate the table}

\item{right_col}{A vector of column names that should be right-aligned.
By default, numeric data is right-aligned, and character data is
left-aligned}

\item{page_size}{The default number of rows displayed per page (default: 10)}

\item{highlight}{Highlight table rows on hover}

\item{borderless}{Remove inner borders from table}

\item{min_widths}{Customise minimum column width using a list of columns and
minimum width in pixels}

\item{...}{Additional arguments passed to \code{reactable::reactable}}
}
\value{
A \code{reactable} HTML widget styled with GOV.UK classes
}
\description{
This function is opinionated and sets table defaults that are in
keeping with the wider GOV.UK design system. Some defaults are overrideable,
such as \code{highlight=TRUE} and \code{borderless=TRUE}, however some are fixed, such
as \code{showSortIcon=FALSE} as the default sort icon is inaccessible. Additional
arguments from \code{reactable::reactable} can be passed to customise the table.
}
\details{
This function inserts a government-styled table using \code{reactable}.
You can use this in R markdown or Quarto documents, or use
renderGovReactable({}) and govReactableOutput() for tables in R Shiny.
govReactableOutput() gives the ability to add a caption, for static
tables made using just govReactable(), use heading_text() to add
captions to tables.
}
\examples{
# Example static table using govReactable
if (interactive()) {
  govReactable(
    iris,
    right_col = c(
      "Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width"
    )
  )

  govReactable(
    iris,
    right_col = c(
      "Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width"
    ),
    highlight = FALSE,
    page_size = 5,
    min_widths = list(
      Sepal.Length = 75,
      Sepal.Width = 75,
      Petal.Length = 75,
      Petal.Width = 75
    )
  )
}
}
\seealso{
Other Govstyle tables tabs and accordions: 
\code{\link{accordion}()},
\code{\link{govReactable-shiny}},
\code{\link{govTable}()},
\code{\link{govTabs}()}
}
\concept{Govstyle tables tabs and accordions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/backlink_Input.R
\name{backlink_Input}
\alias{backlink_Input}
\title{Back Link Function}
\usage{
backlink_Input(inputId, label = "Back")
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{label}{The link text for the backlink, default is "Back"}
}
\value{
a backlink HTML shiny tag object
}
\description{
This function adds a back link to the page
}
\examples{
ui <- shiny::fluidPage(
  header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png"
  ),
  shiny::navlistPanel(
    "",
    id = "nav",
    widths = c(2, 10),
    well = FALSE,
    # Create first panel
    shiny::tabPanel(
      "Select Types",
      value = "panel1",
      gov_layout(
        size = "two-thirds",
        backlink_Input("link1"),
        shiny::tags$br(),
        shiny::tags$br()
      )
    ),
    shiny::tabPanel(
      "Tab2",
      value = "panel2"
    )
  ),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {
  # Slightly confused in that it goes forward rather than back
  # but shows how to use
  observeEvent(input$link1, {
    updateTabsetPanel(session, "nav", selected = "panel2")
  })
}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle navigation: 
\code{\link{contents_link}()},
\code{\link{service_navigation}()},
\code{\link{update_service_navigation}()}
}
\concept{Govstyle navigation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_example.R
\name{run_example}
\alias{run_example}
\title{Run examples}
\usage{
run_example()
}
\value{
runs an R Shiny app with examples in
}
\description{
This function runs an example R Shiny app showcasing different parts of the
package. Code for the app can be found in the inst/example_app folder in the
source code.
}
\details{
The app uses the \href{https://rstudio.github.io/bslib/}{bslib package} as is
generally recommended for Shiny apps.

The app is also deployed at the following URL:
https://department-for-education.shinyapps.io/shinygovstyle-example-app/
}
\examples{
if (interactive()) run_example()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/error_summary.R
\name{error_summary}
\alias{error_summary}
\title{Error Summary Function}
\usage{
error_summary(inputId, error_title, error_list)
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{error_title}{The title for the error summary}

\item{error_list}{A list of text values to be displayed in the error body}
}
\value{
an error_summary HTML shiny tag object
}
\description{
This function loads the error summary component to display error text.
This replicates the gov style error boxes linked below:
https://design-system.service.gov.uk/components/error-summary/
}
\examples{
ui <- shiny::fluidPage(
  shinyjs::useShinyjs(),
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples",
    logo = "shinyGovstyle/images/moj_logo.png"
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    error_summary(
      inputId = "errorId",
      error_title = "Error title",
      error_list = c("error item1", "error item2")
    )
  ),
  shinyGovstyle::button_Input("btn1", "Change error summary"),
  shinyGovstyle::footer(full = TRUE)
)

server <- function(input, output, session) {
  shiny::observeEvent(input$btn1, {
    error_summary_update(
      "errorId",
      c("error item1", "error item2", "error item3")
    )},
    ignoreInit = TRUE
  )
}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle errors: 
\code{\link{error_off}()},
\code{\link{error_on}()},
\code{\link{error_summary_update}()}
}
\concept{Govstyle errors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/text_area_input.R
\name{text_area_Input}
\alias{text_area_Input}
\title{Text Area Input Function}
\usage{
text_area_Input(
  inputId,
  label,
  hint_label = NULL,
  row_no = 5,
  error = FALSE,
  error_message = NULL,
  word_limit = NULL
)
}
\arguments{
\item{inputId}{The input slot that will be used to access the value}

\item{label}{Display label for the control, or \code{NULL} for no label}

\item{hint_label}{Display hint label for the control, or \code{NULL} for no
hint label}

\item{row_no}{Size of the text entry box. Defaults to 5}

\item{error}{Whenever to include error handling. Defaults to \code{FALSE}}

\item{error_message}{Message to display on error. Defaults to \code{NULL}}

\item{word_limit}{Add a word limit to the display. Defaults to \code{NULL}}
}
\value{
a text area box HTML shiny tag object
}
\description{
This function create a text area input.
}
\examples{
text_area_Input(
  "taId",
  "Can you provide more detail?",
  paste(
    "Do not include personal or financial information, like your",
    "National Insurance number or credit card details."
  )
)
}
\seealso{
Other Govstyle text types: 
\code{\link{date_Input}()},
\code{\link{gov_list}()},
\code{\link{heading_text}()},
\code{\link{input_field}()},
\code{\link{text_Input}()},
\code{\link{word_count}()}
}
\concept{Govstyle text types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/value_box.R
\name{value_box}
\alias{value_box}
\title{Create a value text box with optional description and colour}
\usage{
value_box(value = "your value goes here", text = NA, colour = "blue")
}
\arguments{
\item{value}{Character. The primary value to display in the value box.
Defaults to "your value goes here"}

\item{text}{Character or NA. An optional description to appear below the
value. If not provided (default is NA), the description will not be
displayed}

\item{colour}{Character. A colour to apply to the value box. Defaults
to "blue". Choose from the following: "grey", "purple", "teal",
"blue", "yellow", "orange", "red", "magenta", or "green"}
}
\value{
A Shiny \code{div} tag representing the value box, styled according
to the specified parameters
}
\description{
This function generates a value text box with an optional description and
customisable colour.
}
\details{
The text box can be used in Shiny applications to display highlighted
information, such as statistics or key metrics.
}
\examples{
value_box(
  value = "1,000,000",
  text = "This is the latest value for the selected inputs.",
  colour = "purple"
)
}
\seealso{
Other Govstyle feedback types: 
\code{\link{details}()},
\code{\link{gov_summary}()},
\code{\link{insert_text}()},
\code{\link{label_hint}()},
\code{\link{noti_banner}()},
\code{\link{panel_output}()},
\code{\link{tag_Input}()},
\code{\link{warning_text}()}
}
\concept{Govstyle feedback types}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/govTable.R
\name{govTable}
\alias{govTable}
\title{Table Function}
\usage{
govTable(
  inputId,
  df,
  caption,
  caption_size = "l",
  num_col = NULL,
  width_overwrite = NULL
)
}
\arguments{
\item{inputId}{Input Id for the table}

\item{df}{expects a dataframe to create a table}

\item{caption}{adds a caption to the table as a header}

\item{caption_size}{adjust the size of caption. Options are s, m, l, xl,
with l as the default}

\item{num_col}{adds numeric class format to these columns}

\item{width_overwrite}{change width. Need to include width for every column.
Columns must add up to 1.
Options are three-quarters, two-thirds, one-half, one-third, one-quarter.
Default is \code{NULL}}
}
\value{
a table HTML shiny tag object
}
\description{
This function inserts a gov styled table. Format is with header looking
rows and columns
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    shinyGovstyle::govTable(
      "tab1",
      shinyGovstyle::transport_data_small,
      "Test",
      "l",
      num_col = c(2,3),
      width_overwrite = c("one-half", "one-quarter", "one-quarter")
    )
  )
)

server <- function(input, output, session) {}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle tables tabs and accordions: 
\code{\link{accordion}()},
\code{\link{govReactable}()},
\code{\link{govReactable-shiny}},
\code{\link{govTabs}()}
}
\concept{Govstyle tables tabs and accordions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gov_list.R
\name{gov_list}
\alias{gov_list}
\title{Gov List function}
\usage{
gov_list(list, style = "none")
}
\arguments{
\item{list}{vector of list}

\item{style}{options: "none", "bullet", "number". defaults to "none"}
}
\description{
Gov List function
}
\examples{
ui <- shiny::fluidPage(
  shinyGovstyle::header(
    org_name = "Example",
    service_name = "User Examples"
  ),
  shinyGovstyle::banner(
    inputId = "banner", type = "beta", 'This is a new service'
  ),
  shinyGovstyle::gov_layout(
    size = "two-thirds",
    shinyGovstyle::heading_text("gov_list", size = "s"),
    shinyGovstyle::gov_text("List:"),
    gov_list(list = c("a", "b", "c")),
    shinyGovstyle::gov_text("Bulleted list:"),
    gov_list(list = c("a", "b", "c"), style = "bullet"),
    shinyGovstyle::gov_text("Numbered list:"),
    gov_list(list = c("one", "two", "three"), style = "number")
  )
)

server <- function(input, output, session) {}

if (interactive()) shinyApp(ui = ui, server = server)
}
\seealso{
Other Govstyle text types: 
\code{\link{date_Input}()},
\code{\link{heading_text}()},
\code{\link{input_field}()},
\code{\link{text_Input}()},
\code{\link{text_area_Input}()},
\code{\link{word_count}()}
}
\concept{Govstyle text types}
